/* Scrutari - Copyright (c) 2008-2012 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.servlets;

import fr.exemole.sctserver.request.xml.XmlRequestUtils;
import fr.exemole.sctserver.storage.LogAccess;
import net.mapeadores.util.io.DocStream;
import net.mapeadores.util.io.MimeTypeConstants;
import net.mapeadores.util.io.docstream.ClassResourceDocStream;
import net.mapeadores.util.servlets.ResponseHandler;
import net.mapeadores.util.servlets.handlers.DocStreamResponseHandler;


/**
 *
 * @author Vincent Calame
 */
public class ResponseHandlerFactory {

    private ResponseHandlerFactory() {
    }

    public static ResponseHandler getCssHandler(String name) {
        DocStream docStream = getCssDocStream(name.substring(1));
        if (docStream == null) {
            return null;
        }
        return new DocStreamResponseHandler(docStream);
    }

    public static ResponseHandler getDtdHandler(String name) {
        DocStream docStream = getDtdDocStream(name.substring(1));
        if (docStream == null) {
            return null;
        }
        return new DocStreamResponseHandler(docStream);
    }

    public static ResponseHandler getJsHandler(String name) {
        DocStream docStream = getJsDocStream(name.substring(1));
        if (docStream == null) {
            return null;
        }
        return new DocStreamResponseHandler(docStream);
    }

    public static ResponseHandler getLogsHandler(String name, LogAccess logAccess) {
        DocStream docStream = getLogsDocStream(logAccess, name.substring(1));
        if (docStream == null) {
            return null;
        }
        return new DocStreamResponseHandler(docStream);
    }

    public static ResponseHandler getPngHandler(String name) {
        DocStream docStream = getPngDocStream(name.substring(1));
        if (docStream == null) {
            return null;
        }
        return new DocStreamResponseHandler(docStream);
    }

    private static DocStream getCssDocStream(String fileName) {
        ClassResourceDocStream docStream = getDocStream("resources/theme/css/" + fileName);
        if (docStream == null) {
            return null;
        }
        docStream.setMimeType(MimeTypeConstants.CSS);
        return docStream;
    }

    private static DocStream getDtdDocStream(String fileName) {
        ClassResourceDocStream docStream = ClassResourceDocStream.newInstance(XmlRequestUtils.class, "dtd/" + fileName);
        if (docStream == null) {
            return null;
        }
        docStream.setMimeType(MimeTypeConstants.DTD);
        return docStream;
    }

    private static DocStream getJsDocStream(String fileName) {
        ClassResourceDocStream docStream = getDocStream("resources/js/" + fileName);
        if (docStream == null) {
            return null;
        }
        docStream.setMimeType(MimeTypeConstants.JAVASCRIPT);
        return docStream;
    }

    private static DocStream getLogsDocStream(LogAccess logAccess, String fileName) {
        return logAccess.getLogDocStream(fileName);
    }

    private static DocStream getPngDocStream(String fileName) {
        ClassResourceDocStream docStream = getDocStream("resources/theme/png/" + fileName);
        if (docStream == null) {
            return null;
        }
        docStream.setMimeType("image/png");
        return docStream;
    }

    private static ClassResourceDocStream getDocStream(String resourcePath) {
        return ClassResourceDocStream.newInstance(ResponseHandlerFactory.class, resourcePath);
    }

}
