/* Scrutari - Copyright (c) 2008-2012 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.servlets.admin.htmlpages;

import fr.exemole.sctserver.SctServer;
import java.io.IOException;
import javax.servlet.http.HttpServletResponse;
import net.mapeadores.scrutari.data.BaseData;
import net.mapeadores.scrutari.data.CorpusData;
import net.mapeadores.scrutari.data.FicheData;
import net.mapeadores.scrutari.db.DataAccess;
import net.mapeadores.scrutari.db.ScrutariDB;
import net.mapeadores.scrutari.db.util.LibelleUtils;
import net.mapeadores.util.html.HTMLAttributes;
import net.mapeadores.util.langinteger.LangInteger;


/**
 *
 * @author Vincent Calame
 */
public class FichesPage extends AdminHtmlPage {

    HTMLAttributes baseAttributes = new HTMLAttributes();
    HTMLAttributes corpusAttributes = new HTMLAttributes();
    HTMLAttributes ficheAttributes = new HTMLAttributes();
    HTMLAttributes ulAttributes = new HTMLAttributes();

    {
        baseAttributes.setClasses("d_SCT_lst_base");
        corpusAttributes.setClasses("l_SCT_lst_corpus");
        ficheAttributes.setClasses("l_SCT_lst_fiche");
        ulAttributes.setStyle("display: none;");
        ulAttributes.setClasses("u_SCT_lst_u");
    }

    private ScrutariDB scrutariDB;
    private int langUi = LangInteger.build("fr");

    public FichesPage(SctServer sctServer) {
        init(sctServer);
        scrutariDB = sctServer.getScrutariDB();
        addCssUrl("../css/listes.css");
        addJsUrl("../js/affichage.js");
    }

    public void handleResponse(HttpServletResponse response) throws IOException {
        DataAccess dataAccess = scrutariDB.openDataAccess();
        initResponse(response);
        start();
        printTools();
        printMessage();

        int baseCount = dataAccess.getBaseCount();
        for (int i = 0; i < baseCount; i++) {
            BaseData baseData = dataAccess.getBaseDataByIndex(i);
            printBaseData(baseData, dataAccess);
        }
        end();
        dataAccess.close();
    }

    private void printBaseData(BaseData baseData, DataAccess dataAccess) {
        int codebase = baseData.getCodeBase();
        int corpusCount = baseData.getCorpusCount();
        String onClick = "affiche('ul_" + codebase + "');";
        baseAttributes.setOnClick(onClick);
        startDiv(baseAttributes);
        escape(LibelleUtils.getLibelleString(baseData, BaseData.INTITULE_LONG, langUi, "?"));
        append(" ");
        startCode();
        append("[");
        append(String.valueOf(corpusCount));
        append("]");
        endDiv();

        ulAttributes.setId("ul_" + codebase);
        startUl(ulAttributes);
        for (int i = 0; i < corpusCount; i++) {
            CorpusData corpusData = dataAccess.getCorpusDataByCode(baseData.getCodeCorpus(i));
            printCorpusData(corpusData, dataAccess);
        }
        endUl();
    }

    private int printCorpusData(CorpusData corpusData, DataAccess dataAccess) {
        int codecorpus = corpusData.getCodeCorpus();
        int ficheCount = corpusData.getFicheCount();
        String onClick = "affiche('ul_" + codecorpus + "');";
        corpusAttributes.setOnClick(onClick);
        startLi(corpusAttributes);
        escape(LibelleUtils.getLibelleString(corpusData, CorpusData.INTITULECORPUS_FIELDNUMBER, langUi, "?"));
        append(" ");
        startCode();
        append("[");
        append(String.valueOf(ficheCount));
        append("]");
        endCode();
        endLi();

        ulAttributes.setId("ul_" + codecorpus);
        ulAttributes.setOnClick(onClick);
        startUl(ulAttributes);
        for (int i = 0; i < ficheCount; i++) {
            FicheData ficheData = dataAccess.getFicheDataByCode(corpusData.getCodeFiche(i));
            printFicheData(ficheData);
        }
        endUl();
        return ficheCount;
    }

    private void printFicheData(FicheData ficheData) {
        startLi(ficheAttributes);
        escape(ficheData.getTitre());
        endLi();
    }

}
