/* Scrutari - Copyright (c) 2009-2012 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.servlets.admin.htmlpages.utils;

import fr.exemole.sctserver.admin.ScrutariInfo;
import fr.exemole.sctserver.admin.SourceInfoManager;
import fr.exemole.sctserver.servlets.admin.htmlpages.AdminHtmlPage;
import fr.exemole.sctserver.servlets.resources.theme.SctCssCatalog;
import net.mapeadores.util.exceptions.SwitchException;
import net.mapeadores.util.html.HTMLAttributes;
import net.mapeadores.util.primitives.DateInteger;


/**
 *
 * @author Vincent Calame
 */
public class SourcesPrinter implements SctCssCatalog {

    private SourcesPrinter() {
    }

    public static void printSourceInfoList(AdminHtmlPage page) {
        SourceInfoManager scrutariInfoManager = page.getSctServer().getScrutariInfoManager();
        int count = scrutariInfoManager.getSourceCount();
        if (count == 0) {
            page.startP();
            page.startSpan(HTMLAttributes.initClasses(INFO_SPANCLASS));
            page.localize("info_list_none");
            page.endSpan();
            page.endP();
            return;
        }
        page.startTable();
        for (int i = 0; i < count; i++) {
            ScrutariInfo scrutariInfo = scrutariInfoManager.getSourceInfo(i);
            page.startRow();
            page.startCell();
            page.append(scrutariInfo.getUrl().toString());
            page.endCell();

            short state = scrutariInfo.getState();
            page.startCell();
            page.startSpan(getSpanAttributes(state));
            page.localize(getLocKey(state));
            page.endSpan();
            page.endCell();
            page.startCell();
            page.append(DateInteger.toISOString(scrutariInfo.getLastUpdate()));
            page.endCell();
            page.endRow();
        }
        page.endTable();
    }

    private static HTMLAttributes getSpanAttributes(short state) {
        switch (state) {
            case ScrutariInfo.OK_STATE:
                return HTMLAttributes.initClasses(DONEMESSAGE_SPANCLASS);
            case ScrutariInfo.UNKNOWN_STATE:
            case ScrutariInfo.INFO_NOCONNECTION_STATE:
            case ScrutariInfo.INFO_BROKENURL_STATE:
            case ScrutariInfo.INFO_DOWNLOADERROR_STATE:
            case ScrutariInfo.INFO_NOTXML_STATE:
            case ScrutariInfo.INFO_WRONGXML_STATE:
            case ScrutariInfo.DATA_NOCONNECTION_STATE:
            case ScrutariInfo.DATA_BROKENURL_STATE:
            case ScrutariInfo.DATA_DOWNLOADERROR_STATE:
            case ScrutariInfo.DATA_NOTXML_STATE:
                return HTMLAttributes.initClasses(ERRORMESSAGE_SPANCLASS);
            default:
                throw new SwitchException("state = " + state);
        }
    }

    private static String getLocKey(short state) {
        switch (state) {
            case ScrutariInfo.OK_STATE:
                return "state_ok";
            case ScrutariInfo.UNKNOWN_STATE:
                return "state_unknown";
            case ScrutariInfo.INFO_NOCONNECTION_STATE:
                return "state_info_noconnection";
            case ScrutariInfo.INFO_BROKENURL_STATE:
                return "state_info_brokenurl";
            case ScrutariInfo.INFO_DOWNLOADERROR_STATE:
                return "state_info_downloaderror";
            case ScrutariInfo.INFO_NOTXML_STATE:
                return "state_info_notxml";
            case ScrutariInfo.INFO_WRONGXML_STATE:
                return "state_info_wrongxml";
            case ScrutariInfo.DATA_NOCONNECTION_STATE:
                return "state_data_noconnection";
            case ScrutariInfo.DATA_BROKENURL_STATE:
                return "state_data_brokenurl";
            case ScrutariInfo.DATA_DOWNLOADERROR_STATE:
                return "state_data_downloaderror";
            case ScrutariInfo.DATA_NOTXML_STATE:
                return "state_data_notxml";
            default:
                throw new SwitchException("state = " + state);
        }
    }

}
