/* Scrutari - Copyright (c) 2012 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.servlets.explore;

import fr.exemole.sctserver.SctServer;
import fr.exemole.sctserver.servlets.explore.xml.BaseListXMLPart;
import fr.exemole.sctserver.servlets.explore.xml.LogXMLPart;
import java.io.IOException;
import net.mapeadores.scrutari.db.DataAccess;
import net.mapeadores.util.xml.AppendableXMLWriter;
import net.mapeadores.util.xml.XMLUtils;
import net.mapeadores.util.xml.XmlProducer;


/**
 *
 * @author Vincent Calame
 */
public class ExploreXmlProducerFactory implements ExploreConstants {

    public static XmlProducer getBaseListXmlProducer(SctServer sctServer) {
        return new BaseListXmlProducer(sctServer);
    }

    public static XmlProducer getLogListXmlProducer(SctServer sctServer) {
        return new LogListXmlProducer(sctServer);
    }

    public static XmlProducer getMonthLogXmlProducer(SctServer sctServer, int mois) {
        return new MoisLogXmlProducer(sctServer, mois);
    }


    private static class BaseListXmlProducer implements XmlProducer {

        private SctServer sctServer;

        private BaseListXmlProducer(SctServer sctServer) {
            this.sctServer = sctServer;
        }

        public void writeXml(Appendable appendable) throws IOException {
            AppendableXMLWriter xmlWriter = XMLUtils.getXMLWriter(appendable);
            xmlWriter.appendXMLDeclaration();
            BaseListXMLPart baseListXMLPart = new BaseListXMLPart(xmlWriter);
            DataAccess dataAccess = sctServer.getScrutariDB().openDataAccess();
            baseListXMLPart.appendBaseList(sctServer, dataAccess);
            dataAccess.close();
        }

    }


    private static class LogListXmlProducer implements XmlProducer {

        private SctServer sctServer;

        private LogListXmlProducer(SctServer sctServer) {
            this.sctServer = sctServer;
        }

        public void writeXml(Appendable appendable) throws IOException {
            AppendableXMLWriter xmlWriter = XMLUtils.getXMLWriter(appendable);
            xmlWriter.appendXMLDeclaration();
            LogXMLPart logXMLPart = new LogXMLPart(xmlWriter, sctServer);
            logXMLPart.appendLogList();
        }

    }


    private static class MoisLogXmlProducer implements XmlProducer {

        private SctServer sctServer;
        private int mois;

        private MoisLogXmlProducer(SctServer sctServer, int mois) {
            this.sctServer = sctServer;
            this.mois = mois;
        }

        public void writeXml(Appendable appendable) throws IOException {
            AppendableXMLWriter xmlWriter = XMLUtils.getXMLWriter(appendable);
            xmlWriter.appendXMLDeclaration();
            LogXMLPart logXMLPart = new LogXMLPart(xmlWriter, sctServer);
            logXMLPart.appendLog(mois);
        }

    }

}
