/*Scrutari - Copyright (c) 2011-2012 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.servlets.explore.xml;

import fr.exemole.sctserver.SctServer;
import fr.exemole.sctserver.admin.ScrutariInfo;
import fr.exemole.sctserver.admin.SourceInfoManager;
import java.io.IOException;
import java.net.URL;
import net.mapeadores.scrutari.data.BaseData;
import net.mapeadores.scrutari.db.DataAccess;
import net.mapeadores.util.primitives.DateInteger;
import net.mapeadores.util.xml.XMLPart;
import net.mapeadores.util.xml.XMLUtils;
import net.mapeadores.util.xml.XMLWriter;
import net.scrutari.datauri.BaseURI;


/**
 *
 * @author Vincent Calame
 */
public class BaseListXMLPart extends XMLPart {

    public BaseListXMLPart(XMLWriter xmlWriter) {
        super(xmlWriter);
    }

    public void appendBaseList(SctServer sctServer, DataAccess dataAccess) throws IOException {
        SourceInfoManager infoManager = sctServer.getScrutariInfoManager();
        openTag("base-list");
        int count = infoManager.getSourceCount();
        for (int i = 0; i < count; i++) {
            ScrutariInfo scrutariInfo = infoManager.getSourceInfo(i);
            BaseURI baseURI = scrutariInfo.getBaseURI();
            if (baseURI != null) {
                int code = dataAccess.getCode(baseURI);
                if (code > 0) {
                    BaseData baseData = dataAccess.getBaseDataByCode(code);
                    startOpenTag("base");
                    addAttribute("uri", baseURI.toString());
                    endOpenTag();
                    addSimpleElement("info-url", getUrlString(scrutariInfo.getUrl()));
                    addSimpleElement("last-update", DateInteger.toISOString(scrutariInfo.getLastUpdate()));
                    addSimpleElement("scrutaridata-url", getUrlString(scrutariInfo.getScrutariDataUrl()));
                    addSimpleElement("authority-uuid", baseURI.getAuthorityUUID());
                    addSimpleElement("base-name", baseURI.getBaseName());
                    int langCount = baseData.getLangDataCount();
                    openTag("intitule-short");
                    for (int j = 0; j < langCount; j++) {
                        BaseData.BaseLangData baseLangData = baseData.getLangDataByIndex(j);
                        int langInteger = baseLangData.getLangInteger();
                        String lib = baseLangData.getLibelleString(BaseData.INTITULE_SHORT);
                        if (lib != null) {
                            XMLUtils.addLibElement(this, langInteger, lib);
                        }
                    }
                    closeTag("intitule-short");
                    openTag("intitule-long");
                    for (int j = 0; j < langCount; j++) {
                        BaseData.BaseLangData baseLangData = baseData.getLangDataByIndex(j);
                        int langInteger = baseLangData.getLangInteger();
                        String lib = baseLangData.getLibelleString(BaseData.INTITULE_LONG);
                        if (lib != null) {
                            XMLUtils.addLibElement(this, langInteger, lib);
                        }
                    }
                    closeTag("intitule-long");
                    closeTag("base");
                }
            }
        }

        closeTag("base-list");
    }

    private static String getUrlString(URL url) {
        if (url == null) {
            return null;
        }
        return url.toString();
    }

}
