/* Scrutari - Copyright (c) 2012 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.servlets.export;

import fr.exemole.sctserver.SctContext;
import fr.exemole.sctserver.SctServer;
import fr.exemole.sctserver.servlets.ScrutariRequestMap;
import fr.exemole.sctserver.servlets.export.contentstreams.ResultOdsStream;
import net.mapeadores.scrutari.searchengine.FicheSearchResult;
import net.mapeadores.util.langinteger.LangInteger;
import net.mapeadores.util.langinteger.LangIntegerException;
import net.mapeadores.util.servlets.ResponseHandler;


/**
 *
 * @author Vincent Calame
 */
public class ExportRequest {

    private boolean notFound = false;
    private SctServer sctServer;
    private ResultParameters resultParameters;

    private ExportRequest(SctContext sctContext, String pathInfo, ScrutariRequestMap requestMap) {
        this.sctServer = sctContext.getSctServer();
        if ((pathInfo.startsWith("/result_")) && (pathInfo.endsWith(".ods"))) {
            resultParameters = initResultParameters(pathInfo.substring(8, pathInfo.length() - 4));
            if (resultParameters == null) {
                setNotFound();
            }
        } else {
            setNotFound();
        }
    }

    private void setNotFound() {
        notFound = true;
    }

    public ResponseHandler getResponseHandler() {
        if (resultParameters != null) {
            return new ResultOdsStream(sctServer, resultParameters.ficheSearchResult, resultParameters.workingLangInteger);
        } else {
            return null;
        }
    }

    private ResultParameters initResultParameters(String param) {
        int idx2 = param.lastIndexOf('_');
        int workingLangInteger = 0;
        try {
            workingLangInteger = LangInteger.parse(param.substring(idx2 + 1));
        } catch (LangIntegerException lie) {
            return null;
        }
        String identifiant = param.substring(0, idx2);
        FicheSearchResult ficheSearchResult = sctServer.getSctServerStorage().getFicheSearchResult(identifiant);
        if (ficheSearchResult == null) {
            return null;
        }
        return new ResultParameters(ficheSearchResult, workingLangInteger);
    }

    public static ExportRequest init(SctContext sctContext, String pathInfo, ScrutariRequestMap requestMap) {
        ExportRequest adminRequest = new ExportRequest(sctContext, pathInfo, requestMap);
        if (adminRequest.notFound) {
            return null;
        } else {
            return adminRequest;
        }
    }


    private static class ResultParameters {

        private FicheSearchResult ficheSearchResult;
        private int workingLangInteger;

        private ResultParameters(FicheSearchResult ficheSearchResult, int workingLangInteger) {
            this.ficheSearchResult = ficheSearchResult;
            this.workingLangInteger = workingLangInteger;
        }

    }

}
