/* Scrutari - Copyright (c) 2012 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.servlets.export.contentstreams;

import fr.exemole.sctserver.SctServer;
import java.io.BufferedWriter;
import java.io.IOException;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.util.HashMap;
import java.util.Map;
import javax.servlet.http.HttpServletResponse;
import net.mapeadores.opendocument.io.OdContentWriter;
import net.mapeadores.opendocument.io.OdUtils;
import net.mapeadores.opendocument.io.OdZip;
import net.mapeadores.opendocument.io.ZipEntrySource;
import net.mapeadores.scrutari.data.BaseData;
import net.mapeadores.scrutari.data.FicheData;
import net.mapeadores.scrutari.data.MotcleData;
import net.mapeadores.scrutari.db.CodeList;
import net.mapeadores.scrutari.db.DataAccess;
import net.mapeadores.scrutari.db.IndexationCodeList;
import net.mapeadores.scrutari.db.ScrutariDB;
import net.mapeadores.scrutari.db.util.BaseCheck;
import net.mapeadores.scrutari.db.util.LibelleUtils;
import net.mapeadores.scrutari.searchengine.CorpusClasse;
import net.mapeadores.scrutari.searchengine.FicheSearchResult;
import net.mapeadores.scrutari.searchengine.FicheSearchResultGroup;
import net.mapeadores.scrutari.searchengine.FicheSearchResultInfo;
import net.mapeadores.scrutari.searchengine.GlobalSearchOptions;
import net.mapeadores.util.langinteger.LangInteger;
import net.mapeadores.util.primitives.DateInteger;
import net.mapeadores.util.servlets.ResponseHandler;
import net.mapeadores.util.text.Libelle;


/**
 *
 * @author Vincent Calame
 */
public class ResultOdsStream implements ResponseHandler {

    private SctServer sctServer;
    private FicheSearchResult ficheSearchResult;
    private int langUi;

    public ResultOdsStream(SctServer sctServer, FicheSearchResult ficheSearchResult, int langUi) {
        this.sctServer = sctServer;
        this.ficheSearchResult = ficheSearchResult;
        this.langUi = langUi;
    }

    public long getLastModified() {
        return ResponseHandler.LASTMODIFIED_NOCACHE;
    }

    public void handleResponse(HttpServletResponse response) throws IOException {
        response.setContentType(OdUtils.getMimeType(OdUtils.OD_SPREADSHEET));
        OutputStream outputStream = response.getOutputStream();
        ResultOdsWriter resultOdsWriter = new ResultOdsWriter(sctServer, ficheSearchResult, langUi);
        resultOdsWriter.export(outputStream);
        outputStream.close();
    }


    private static class ResultOdsWriter {

        private OdZip odZip;
        private SctServer sctServer;
        private FicheSearchResult ficheSearchResult;
        private int langUi;

        public ResultOdsWriter(SctServer sctServer, FicheSearchResult ficheSearchResult, int langUi) {
            this.sctServer = sctServer;
            this.ficheSearchResult = ficheSearchResult;
            this.langUi = langUi;
            odZip = new OdZip();
            odZip.setOdtype(OdUtils.OD_SPREADSHEET);
            odZip.setContentZipEntrySource(new ContentZipEntrySource());
        }

        public void export(OutputStream outputStream) throws IOException {
            odZip.zip(outputStream);
        }


        private class ContentZipEntrySource implements ZipEntrySource {

            public void writeEntry(OutputStream outputStream) throws IOException {
                BufferedWriter writer = new BufferedWriter(new OutputStreamWriter(outputStream, "UTF-8"));
                ResultXMLWriter xmlWriter = new ResultXMLWriter();
                xmlWriter.setAppendable(writer);
                xmlWriter.setIndentLength(0);
                xmlWriter.init();
                xmlWriter.start();
                xmlWriter.printResult();
                xmlWriter.end();
                writer.flush();
            }


            private class ResultXMLWriter extends OdContentWriter {

                private ScrutariDB scrutariDB;
                private DataAccess dataAccess;
                private GlobalSearchOptions globalSearchOptions;
                private Map<Integer, BaseInfo> baseInfoMap = new HashMap<Integer, BaseInfo>();
                private Map<Integer, String> motcleLibMap = new HashMap<Integer, String>();

                private void init() {
                    scrutariDB = sctServer.getScrutariDB();
                    dataAccess = scrutariDB.openDataAccess();
                    globalSearchOptions = sctServer.getOptionsManager().getGlobalSearchOptions();
                }

                private void start() throws IOException {
                    appendXMLDeclaration();
                    openDocumentContent();
                    openBody();
                    openSpreadsheet();

                }

                private void end() throws IOException {
                    closeSpreadsheet();
                    closeBody();
                    closeDocumentContent();
                    dataAccess.close();
                }

                private void printResult() throws IOException {
                    int groupCount = ficheSearchResult.getFicheSearchResultGroupCount();
                    int classeCount = globalSearchOptions.getCorpusClasseCount();
                    for (int i = 0; i < groupCount; i++) {
                        FicheSearchResultGroup ficheSearchResultGroup = ficheSearchResult.getFicheSearchResultGroup(i);
                        int ficheByGroupCount = ficheSearchResultGroup.getFicheSearchResultCount();
                        String tableName = null;
                        if (classeCount > 0) {
                            int corpusClasseRank = ficheSearchResultGroup.getCorpusClasseRank();
                            CorpusClasse corpusClasse = globalSearchOptions.getCorpusClasseByRank(corpusClasseRank);
                            tableName = corpusClasse.getLibelleString(langUi, true);
                        }
                        if (tableName == null) {
                            if (groupCount == 1) {
                                tableName = "Result";
                            } else {
                                int pos = i + 1;
                                tableName = "Result " + pos;
                            }
                        }
                        openTable(tableName);
                        for (int j = 0; j < ficheByGroupCount; j++) {
                            FicheSearchResultInfo ficheSearchResultInfo = ficheSearchResultGroup.getFicheSearchResultInfo(j);
                            int codefiche = ficheSearchResultInfo.getCode();
                            FicheData ficheData = dataAccess.getFicheDataByCode(codefiche);
                            printRow(ficheData);
                        }
                        closeTable();

                    }
                }

                private void printRow(FicheData ficheData) throws IOException {
                    int codefiche = ficheData.getCodeFiche();
                    BaseInfo baseInfo = getBaseInfo(ficheData.getCodeBase());
                    openTableRow();
                    addStringTableCell(ficheData.getTitre());
                    addStringTableCell(ficheData.getSoustitre());
                    addIntegerTableCell(DateInteger.getAnnee(ficheData.getDateInteger()));
                    addStringTableCell(baseInfo.libelle);
                    addStringTableCell(LangInteger.toString(ficheData.getLangInteger()));
                    addLinkStringTableCell(ficheData.getHref(baseInfo.baseCheck.getCheckedLangUi()));
                    int max = ficheData.getComplementMaxNumber();
                    dataAccess.getIndexationCodeList(langUi, max);
                    StringBuilder motcleBuilder = new StringBuilder();
                    CodeList thesaurusCodeList = dataAccess.getCodethesaurusList();
                    int thesaurusCount = thesaurusCodeList.getCodeCount();
                    for (int i = 0; i < thesaurusCount; i++) {
                        IndexationCodeList indexationCodeList = dataAccess.getIndexationCodeList(codefiche, thesaurusCodeList.getCode(i));
                        int indexationCount = indexationCodeList.getCodeCount();
                        for (int j = 0; j < indexationCount; j++) {
                            int codemotcle = indexationCodeList.getCode(j);
                            String libelleString = getMotcleLib(codemotcle);
                            if (libelleString.length() > 0) {
                                if (motcleBuilder.length() > 0) {
                                    motcleBuilder.append("; ");
                                }
                                motcleBuilder.append(libelleString);
                            }
                        }
                    }
                    addStringTableCell(motcleBuilder.toString());
                    for (int i = 1; i <= max; i++) {
                        addStringTableCell(ficheData.getComplementByNumber(i));
                    }
                    closeTableRow();
                }

                private String getMotcleLib(int codemotcle) {
                    String libelleString = motcleLibMap.get(codemotcle);
                    if (libelleString != null) {
                        return libelleString;
                    }
                    MotcleData motcleData = dataAccess.getMotcleDataByCode(codemotcle);
                    Libelle libelle = motcleData.getLibelleByLang(langUi);
                    if (libelle == null) {
                        if (motcleData.getLibelleCount() > 0) {
                            libelle = motcleData.getLibelleByIndex(0);
                        }
                    }
                    if (libelle != null) {
                        libelleString = libelle.getLibelleString();
                    } else {
                        libelleString = "";
                    }
                    motcleLibMap.put(codemotcle, libelleString);
                    return libelleString;
                }

                private BaseInfo getBaseInfo(int codebase) {
                    BaseInfo baseCheck = baseInfoMap.get(codebase);
                    if (baseCheck != null) {
                        return baseCheck;
                    }
                    BaseData baseData = dataAccess.getBaseDataByCode(codebase);
                    baseCheck = new BaseInfo(baseData);
                    baseInfoMap.put(codebase, baseCheck);
                    return baseCheck;
                }


                private class BaseInfo {

                    private BaseData baseData;
                    private BaseCheck baseCheck;
                    private String libelle;

                    private BaseInfo(BaseData baseData) {
                        this.baseData = baseData;
                        this.baseCheck = BaseCheck.check(baseData, langUi);
                        libelle = LibelleUtils.getLibelleString(baseData, BaseData.INTITULE_SHORT, baseCheck.getCheckedLangUi(), baseData.getBaseURI().getBaseName());
                    }

                }

            }

        }

    }

}
