/* Scrutari - Copyright (c) 2011-2012 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.servlets.feed;

import fr.exemole.sctserver.SctContext;
import fr.exemole.sctserver.feed.FeedConstants;
import fr.exemole.sctserver.feed.FeedOptions;
import java.util.HashSet;
import java.util.Set;
import net.mapeadores.util.exceptions.SwitchException;
import net.mapeadores.util.langinteger.LangInteger;
import net.mapeadores.util.langinteger.LangIntegerException;
import net.mapeadores.util.localisation.LocalisationInfo;
import net.mapeadores.util.servlets.ResponseHandler;


/**
 *
 * @author Vincent Calame
 */
public class FeedRequest {

    private final static String ATOM_EXTENSION = ".atom";
    private boolean isInit;
    private short responseHandlerType;
    private SctContext sctContext;
    private FeedOptions feedOptions;

    private FeedRequest(SctContext sctContext, String pathInfo) {
        this.sctContext = sctContext;
        if (!pathInfo.endsWith(ATOM_EXTENSION)) {
            return;
        }
        String base = pathInfo.substring(0, pathInfo.length() - ATOM_EXTENSION.length());
        int idx = base.lastIndexOf('_');
        if (idx == -1) {
            return;
        }
        LocalisationInfo localisationInfo;
        try {
            int langInteger = LangInteger.parse(base.substring(idx + 1));
            localisationInfo = new LocalisationInfo(langInteger);
        } catch (LangIntegerException lie) {
            return;
        }
        feedOptions = new FeedOptions(localisationInfo);
        String options;
        if (base.startsWith("/tree")) {
            options = base.substring(5, idx);
            feedOptions.setContentType(FeedConstants.TREE_CONTENTTYPE);
        } else if (base.startsWith("/classes")) {
            options = base.substring(8, idx);
            feedOptions.setContentType(FeedConstants.CLASSES_CONTENTTYPE);
        } else {
            return;
        }
        if (options.length() > 0) {
            if (!options.startsWith("-")) {
                return;
            }
            parse(options, feedOptions);
        }
        isInit = true;
        responseHandlerType = FeedConstants.ATOM_FEED;
    }

    public static FeedRequest init(SctContext sctContext, String pathInfo) {
        FeedRequest feedRequest = new FeedRequest(sctContext, pathInfo);
        if (feedRequest.isInit) {
            return feedRequest;
        } else {
            return null;
        }
    }

    public ResponseHandler getResponseHandler() {
        switch (responseHandlerType) {
            case FeedConstants.ATOM_FEED:
                return new AtomResponseHandler(sctContext, feedOptions);
            default:
                throw new SwitchException("responseHandlerType = " + responseHandlerType);
        }
    }

    public static void parse(String optionsString, FeedOptions feedOptions) {
        Set<Character> caracSet = new HashSet<Character>();
        int length = optionsString.length();
        for (int i = 0; i < length; i++) {
            char carac = optionsString.charAt(i);
            if (carac == '-') {
            } else {
                caracSet.add(carac);
            }
        }
        if (caracSet.contains('f')) {
            feedOptions.setFichesOnly(true);
        }
        if (caracSet.contains('s')) {
            feedOptions.setWithSoustitre(true);
        }
        if (caracSet.contains('c')) {
            feedOptions.setWithComplements(true);
        }
        if (caracSet.contains('m')) {
            feedOptions.setWithMotscles(true);
        }
        if (caracSet.contains('i')) {
            feedOptions.setWithIcon(true);
        }
        if (caracSet.contains('h')) {
            feedOptions.setWithIndent(true);
        }
        if (caracSet.contains('v')) {
            feedOptions.setTechnicalTitle(true);
        }
    }

}
