/* Scrutari - Copyright (c) 2008 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.servlets.selection;

import net.scrutari.datauri.ScrutariDataURI;
import net.scrutari.datauri.ThesaurusURI;
import net.scrutari.datauri.URIParseException;
import net.scrutari.datauri.URIParser;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.Reader;
import java.util.LinkedHashSet;
import java.util.Set;


/**
 *
 * @author Vincent Calame
 */
public class SelectionUnit {

    private Set<ThesaurusURI> thesaurusURISet = new LinkedHashSet<ThesaurusURI>();
    private String name;

    public SelectionUnit(String name) {
        this.name = name;
    }

    public static SelectionUnit read(String name, Reader reader) throws IOException {
        SelectionUnit selectionUnit = new SelectionUnit(name);
        BufferedReader bufread = new BufferedReader(reader);
        String ligne = null;
        while ((ligne = bufread.readLine()) != null) {
            ligne = ligne.trim();
            if (ligne.length() == 0) {
                continue;
            }
            try {
                ScrutariDataURI scrutariURI = URIParser.parse(ligne);
                selectionUnit.add(scrutariURI);
            } catch (URIParseException upe) {
                System.out.println(upe.getMessage());
            }
        }
        return selectionUnit;
    }

    public boolean contains(ThesaurusURI thesaurusURI) {
        boolean cts = thesaurusURISet.contains(thesaurusURI);
        return cts;
    }

    private void add(ScrutariDataURI scrutariURI) {
        switch (scrutariURI.getType()) {
            case ScrutariDataURI.THESAURUSURI_TYPE:
                addThesaurusURI((ThesaurusURI) scrutariURI);
                break;
        }
    }

    private void addThesaurusURI(ThesaurusURI thesaurusURI) {
        thesaurusURISet.add(thesaurusURI);
    }

    public String getName() {
        return name;
    }

}
