/* ScrutariDataURI - Copyright (c) 2005-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.scrutari.datauri;


/**
 *
 * @author  Vincent Calame
 */
public final class CorpusURI extends ScrutariDataURI implements Comparable<CorpusURI> {

    private BaseURI baseURI;
    private String corpusName;

    public CorpusURI(String authorityUUID, String baseName, String corpusName) {
        this.corpusName = corpusName;
        this.baseURI = new BaseURI(authorityUUID, baseName);
    }

    public CorpusURI(BaseURI baseURI, String corpusName) {
        this.corpusName = corpusName;
        this.baseURI = baseURI;
    }

    public BaseURI getBaseURI() {
        return baseURI;
    }

    public String getCorpusName() {
        return corpusName;
    }

    @Override
    public int hashCode() {
        return corpusName.hashCode() + baseURI.hashCode();
    }

    public int compareTo(CorpusURI corpusURI) {
        int result = this.corpusName.compareTo(corpusURI.corpusName);
        if (result == 0) {
            result = this.baseURI.compareTo(corpusURI.baseURI);
        }
        return result;
    }

    @Override
    public boolean equals(Object o) {
        if (o == this) {
            return true;
        }
        if (!(o instanceof CorpusURI)) {
            return false;
        }
        return (compareTo((CorpusURI) o) == 0);
    }

    public short getType() {
        return CORPUSURI_TYPE;
    }

    protected void appendPath(StringBuilder buf) {
        buf.append('/');
        buf.append(baseURI.getAuthorityUUID());
        buf.append('/');
        buf.append(baseURI.getBaseName());
        buf.append('/');
        buf.append(corpusName);
    }

}
