/* ScrutariDataURI - Copyright (c) 2005-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.scrutari.datauri;


/**
 *
 * @author  Vincent Calame
 */
public final class FicheURI extends ScrutariDataURI implements Comparable<FicheURI> {

    private CorpusURI corpusURI;
    private String ficheId;

    public FicheURI(CorpusURI corpusURI, String ficheId) {
        this.corpusURI = corpusURI;
        this.ficheId = ficheId;
    }

    public FicheURI(String authorityUUID, String nombase, String corpusName, String ficheId) {
        this.ficheId = ficheId;
        this.corpusURI = new CorpusURI(authorityUUID, nombase, corpusName);
    }

    public String getFicheId() {
        return ficheId;
    }

    public CorpusURI getCorpusURI() {
        return corpusURI;
    }

    @Override
    public int hashCode() {
        return ficheId.hashCode() + corpusURI.hashCode();
    }

    public int compareTo(FicheURI ficheURI) {
        int result = this.ficheId.compareTo(ficheURI.ficheId);
        if (result == 0) {
            result = this.corpusURI.compareTo(ficheURI.corpusURI);
        }
        return result;
    }

    @Override
    public boolean equals(Object o) {
        if (o == this) {
            return true;
        }
        if (!(o instanceof FicheURI)) {
            return false;
        }
        return (compareTo((FicheURI) o) == 0);
    }

    public short getType() {
        return FICHEURI_TYPE;
    }

    protected void appendPath(StringBuilder buf) {
        BaseURI baseURI = corpusURI.getBaseURI();
        buf.append('/');
        buf.append(baseURI.getAuthorityUUID());
        buf.append('/');
        buf.append(baseURI.getBaseName());
        buf.append('/');
        buf.append(corpusURI.getCorpusName());
        buf.append('/');
        buf.append(ficheId);
    }

}
