/* ScrutariDataURI - Copyright (c) 2005-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.scrutari.datauri;


/**
 *
 * @author  Vincent Calame
 */
public final class MotcleURI extends ScrutariDataURI implements Comparable<MotcleURI> {

    private ThesaurusURI thesaurusURI;
    private String motcleId;

    public MotcleURI(ThesaurusURI thesaurusURI, String motcleId) {
        this.thesaurusURI = thesaurusURI;
        this.motcleId = motcleId;
    }

    public MotcleURI(String authorityUUID, String nombase, String thesaurusName, String motcleId) {
        this.motcleId = motcleId;
        this.thesaurusURI = new ThesaurusURI(authorityUUID, nombase, thesaurusName);
    }

    public String getMotcleId() {
        return motcleId;
    }

    public ThesaurusURI getThesaurusURI() {
        return thesaurusURI;
    }

    @Override
    public int hashCode() {
        return motcleId.hashCode() + thesaurusURI.hashCode();
    }

    public int compareTo(MotcleURI motcleURI) {
        int result = this.motcleId.compareTo(motcleURI.motcleId);
        if (result == 0) {
            result = this.thesaurusURI.compareTo(motcleURI.thesaurusURI);
        }
        return result;
    }

    @Override
    public boolean equals(Object o) {
        if (o == this) {
            return true;
        }
        if (!(o instanceof MotcleURI)) {
            return false;
        }
        return (compareTo((MotcleURI) o) == 0);
    }

    public short getType() {
        return MOTCLEURI_TYPE;
    }

    protected void appendPath(StringBuilder buf) {
        BaseURI baseURI = thesaurusURI.getBaseURI();
        buf.append('/');
        buf.append(baseURI.getAuthorityUUID());
        buf.append('/');
        buf.append(baseURI.getBaseName());
        buf.append('/');
        buf.append(thesaurusURI.getThesaurusName());
        buf.append('/');
        buf.append(motcleId);
    }

}
