/* ScrutariLib_DB - Copyright (c) 2012 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.scrutari.db;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import net.mapeadores.scrutari.lexie.AlineaInteger;
import net.mapeadores.util.attr.AttributeDef;
import net.mapeadores.util.attr.AttributeDefList;
import net.mapeadores.util.attr.AttributeKey;
import net.mapeadores.util.attr.AttributeUtils;


/**
 *
 * @author Vincent Calame
 */
public class AttributeKeyCoderBuilder {

    public static AttributeKeyCoder build(Map<String, AttributeDefList> map) {
        InternalAttributeKeyCoder attributeKeyCoder = new InternalAttributeKeyCoder();
        Set<AttributeKey> keySet = new HashSet<AttributeKey>();
        List<AttributeDef> primaryList = new ArrayList<AttributeDef>();
        List<AttributeDef> secondaryList = new ArrayList<AttributeDef>();
        if (map != null) {
            AttributeDefList primary = map.get(AttributeKeyCoder.PRIMARY_GROUP);
            if (primary != null) {
                int count = primary.getAttributeDefCount();
                for (int i = 0; i < count; i++) {
                    AttributeDef attributeDef = primary.getAttributeDef(i);
                    AttributeKey attributeKey = attributeDef.getAttributeKey();
                    if (keySet.contains(attributeKey)) {
                        continue;
                    }
                    keySet.add(attributeKey);
                    primaryList.add(attributeDef);
                }
            }
            AttributeDefList secondary = map.get(AttributeKeyCoder.SECONDARY_GROUP);
            if (secondary != null) {
                int count = secondary.getAttributeDefCount();
                for (int i = 0; i < count; i++) {
                    AttributeDef attributeDef = secondary.getAttributeDef(i);
                    AttributeKey attributeKey = attributeDef.getAttributeKey();
                    if (keySet.contains(attributeKey)) {
                        continue;
                    }
                    keySet.add(attributeKey);
                    secondaryList.add(attributeDef);
                }
            }
        }
        attributeKeyCoder.addPrimaryList(primaryList);
        attributeKeyCoder.addSecondaryList(secondaryList);
        return attributeKeyCoder;
    }


    private static class InternalAttributeKeyCoder implements AttributeKeyCoder {

        private Map<AttributeKey, Integer> codeMap = new HashMap<AttributeKey, Integer>();
        private Map<AttributeKey, AttributeDef> attributeDefMap = new HashMap<AttributeKey, AttributeDef>();
        private AttributeDefList primaryAttributeDefList;
        private AttributeDefList secondaryAttributeDefList;

        public int getAttributeDefCount() {
            return attributeDefMap.size();
        }

        public int getAlineaRootCode(AttributeKey attributeKey) {
            Integer itg = codeMap.get(attributeKey);
            if (itg == null) {
                return -1;
            }
            return itg.intValue();
        }

        public AttributeDefList getAttributeDefList(String name) {
            if (name.equals(AttributeKeyCoder.PRIMARY_GROUP)) {
                return primaryAttributeDefList;
            }
            if (name.equals(AttributeKeyCoder.SECONDARY_GROUP)) {
                return secondaryAttributeDefList;
            }
            return null;
        }

        public AttributeDef getAttributeDef(AttributeKey attributeKey) {
            return attributeDefMap.get(attributeKey);
        }

        private void addPrimaryList(List<AttributeDef> list) {
            int primaryMax = AlineaInteger.PRIMARY_ATTRIBUTEGROUP_START;
            int size = list.size();
            if (size > 999) {
                throw new UnsupportedOperationException("Only 999 primary attribute key");
            }
            for (int i = 0; i < size; i++) {
                AttributeDef attributeDef = list.get(i);
                AttributeKey attributeKey = attributeDef.getAttributeKey();
                attributeDefMap.put(attributeKey, attributeDef);
                codeMap.put(attributeKey, primaryMax);
                primaryMax++;
            }
            primaryAttributeDefList = AttributeUtils.toAttributeDefList(list);
        }

        private void addSecondaryList(List<AttributeDef> list) {
            int secondaryMax = AlineaInteger.SECONDARY_ATTRIBUTEGROUP_START;
            int size = list.size();
            if (size > 999) {
                throw new UnsupportedOperationException("Only 999 secondary attribute key");
            }
            for (int i = 0; i < size; i++) {
                AttributeDef attributeDef = list.get(i);
                AttributeKey attributeKey = attributeDef.getAttributeKey();
                attributeDefMap.put(attributeKey, attributeDef);
                codeMap.put(attributeKey, secondaryMax);
                secondaryMax++;
            }
            secondaryAttributeDefList = AttributeUtils.toAttributeDefList(list);
        }

    }

}
