/* ScrutariLib_DB - Copyright (c) 2008-2012 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.scrutari.db;

import net.mapeadores.scrutari.data.BaseData;
import net.mapeadores.scrutari.data.CorpusData;
import net.mapeadores.scrutari.data.MotcleData;
import net.mapeadores.scrutari.data.ThesaurusData;
import net.mapeadores.util.collections.LinkedIntHashMap;
import net.mapeadores.util.text.Libelle;


/**
 *
 * @author Vincent Calame
 */
public class LangStats {

    private int[] allLangArray;
    private int[] allFicheLangArray;
    private int[] allMotcleLangArray;
    private int[] contentLangArray;
    private LinkedIntHashMap langArrayMap;

    public LangStats() {
    }

    public int[] getAllLangArray() {
        return allLangArray;
    }

    public int[] getAllFicheLangArray() {
        return allFicheLangArray;
    }

    public int[] getAllMotcleLangArray() {
        return allMotcleLangArray;
    }

    public int[] getContentLangArray() {
        return contentLangArray;
    }

    void update(ScrutariDB scrutariDB) {
        DataAccess dataAccess = scrutariDB.openDataAccess();
        LangStatsBuilder builder = new LangStatsBuilder(this, dataAccess);
        builder.run();
        dataAccess.close();
    }


    private static class LangStatsBuilder {

        private LangStats langStats;
        private DataAccess dataAccess;
        private LangueBundle allLang = new LangueBundle();
        private LangueBundle allFicheLang = new LangueBundle();
        private LangueBundle allMotcleLang = new LangueBundle();
        private LinkedIntHashMap langArrayMap = new LinkedIntHashMap();

        private LangStatsBuilder(LangStats langStats, DataAccess dataAccess) {
            this.langStats = langStats;
            this.dataAccess = dataAccess;
        }

        private void run() {
            checkBase();
            checkCorpus();
            checkThesaurus();
            end();
        }

        private void end() {
            langStats.allLangArray = allLang.toArray();
            langStats.langArrayMap = langArrayMap;
            langStats.allFicheLangArray = allFicheLang.toArray();
            langStats.allMotcleLangArray = allMotcleLang.toArray();
            allFicheLang.add(allMotcleLang);
            langStats.contentLangArray = allFicheLang.toArray();
        }

        private void checkBase() {
            int count = dataAccess.getBaseCount();
            for (int i = 0; i < count; i++) {
                BaseData baseData = dataAccess.getBaseDataByIndex(i);
                int langCount = baseData.getLangDataCount();
                for (int j = 0; j < langCount; j++) {
                    BaseData.BaseLangData langData = baseData.getLangDataByIndex(j);
                    allLang.put(langData.getLangInteger());
                }
            }
        }

        private void checkCorpus() {
            int count = dataAccess.getCorpusCount();
            for (int i = 0; i < count; i++) {
                CorpusData corpusData = dataAccess.getCorpusDataByIndex(i);
                int langCount = corpusData.getLangDataCount();
                for (int j = 0; j < langCount; j++) {
                    CorpusData.CorpusLangData langData = corpusData.getLangDataByIndex(j);
                    allLang.put(langData.getLangInteger());
                }
                LangueBundle ficheLang = new LangueBundle();
                int ficheCount = corpusData.getFicheCount();
                for (int j = 0; j < ficheCount; j++) {
                    int langInteger = dataAccess.getIntValue(DataAccess.FICHE_LANG_INTVALUE, corpusData.getCodeFiche(j));
                    allLang.put(langInteger);
                    ficheLang.put(langInteger);
                    allFicheLang.put(langInteger);
                }
                langArrayMap.put(corpusData.getCodeCorpus(), ficheLang.toArray());
            }

        }

        private void checkThesaurus() {
            int count = dataAccess.getThesaurusCount();
            for (int i = 0; i < count; i++) {
                ThesaurusData thesaurusData = dataAccess.getThesaurusDataByIndex(i);

                int langCount = thesaurusData.getThesaurusLangDataCount();
                for (int j = 0; j < langCount; j++) {
                    ThesaurusData.ThesaurusLangData langData = thesaurusData.getThesaurusLangDataByIndex(j);
                    allLang.put(langData.getLangInteger());
                }
                LangueBundle motcleLang = new LangueBundle();
                int motcleCount = thesaurusData.getMotcleCount();
                for (int j = 0; j < motcleCount; j++) {
                    MotcleData motcleData = dataAccess.getMotcleDataByCode(thesaurusData.getCodeMotcle(j));
                    int libelleCount = motcleData.getLibelleCount();
                    for (int k = 0; k < libelleCount; k++) {
                        Libelle libelle = motcleData.getLibelleByIndex(k);
                        int li = libelle.getLangInteger();
                        allLang.put(li);
                        motcleLang.put(li);
                        allMotcleLang.put(li);
                    }
                }
                langArrayMap.put(thesaurusData.getCodeThesaurus(), motcleLang.toArray());
            }
        }

    }


    private static class LangueBundle {

        private LinkedIntHashMap intHashMap = new LinkedIntHashMap();

        private LangueBundle() {
        }

        private void add(LangueBundle langBundle) {
            LinkedIntHashMap other = langBundle.intHashMap;
            int size = other.size();
            for (int i = 0; i < size; i++) {
                put(other.getKeyAt(i));
            }
        }

        private void put(int langInteger) {
            intHashMap.put(langInteger, Boolean.TRUE);
        }

        private int[] toArray() {
            return intHashMap.getKeyArray();
        }

    }

}
