/* ScrutariLib_DB - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.scrutari.db;

import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.SimpleTimeZone;


/**
 *
 * @author Vincent Calame
 */
public class ScrutariDBName {

    private final static DateFormat isoformat = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss'Z'");
    private final static DateFormat nameformat = new SimpleDateFormat("yyyyMMdd'T'HHmmss");

    static {
        isoformat.setTimeZone(new SimpleTimeZone(0, "GMT"));
        nameformat.setTimeZone(new SimpleTimeZone(0, "GMT"));
    }

    private String name;
    private Date date;

    private ScrutariDBName(String name, Date date) {
        this.name = name;
        this.date = date;
    }

    public Date getDate() {
        return date;
    }

    public String getName() {
        return name;
    }

    public synchronized static Date parseNameFormat(String s) throws ParseException {
        return nameformat.parse(s);
    }

    public synchronized static ScrutariDBName newName() {
        Date current = new Date();
        return newName(current);
    }

    public synchronized static ScrutariDBName newName(Date date) {
        String name = nameformat.format(date);
        return new ScrutariDBName(name, date);
    }

    public synchronized static ScrutariDBName parse(String name) throws ParseException {
        Date date = nameformat.parse(name);
        return new ScrutariDBName(name, date);
    }

    public synchronized static String toGmtIsoFormat(Date date) {
        return isoformat.format(date);
    }

    public synchronized static String toNameFormat(Date date) {
        return nameformat.format(date);
    }

}
