/* ScrutariLib_DB - Copyright (c) 2010-2012 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.scrutari.db.explore;

import java.util.ArrayList;
import java.util.List;
import net.mapeadores.scrutari.data.CorpusData;
import net.mapeadores.scrutari.data.ThesaurusData;
import net.mapeadores.scrutari.db.CodeFilter;
import net.mapeadores.scrutari.db.CodeList;
import net.mapeadores.scrutari.db.CodeListUtils;
import net.mapeadores.scrutari.db.DataAccess;
import net.mapeadores.scrutari.db.IndexationCodeList;


/**
 *
 * @author Vincent Calame
 */
public class ExploreUtils implements ExploreConstants {

    private ExploreUtils() {
    }

    public static CodeList getCodeficheList(DataAccess dataAccess, CodeFilter codeficheFilter, CodeList codecorpusList) {
        if (codecorpusList == null) {
            codecorpusList = dataAccess.getCodecorpusList();
        }
        List<Integer> list = new ArrayList<Integer>();
        int corpusCount = codecorpusList.getCodeCount();
        for (int i = 0; i < corpusCount; i++) {
            int codecorpus = codecorpusList.getCode(i);
            CorpusData corpusData = dataAccess.getCorpusDataByCode(codecorpus);
            int ficheCount = corpusData.getFicheCount();
            for (int j = 0; j < ficheCount; j++) {
                int codefiche = corpusData.getCodeFiche(j);
                if ((codeficheFilter == null) || (codeficheFilter.acceptCode(codefiche))) {
                    list.add(codefiche);
                }
            }
        }
        return CodeListUtils.toCodeList(list);
    }

    public static CodeList getCodeficheList(DataAccess dataAccess, int codemotcle, CodeList codecorpusList) {
        if (codecorpusList == null) {
            codecorpusList = dataAccess.getCodecorpusList();
        }
        List<Integer> list = new ArrayList<Integer>();
        int corpusCount = codecorpusList.getCodeCount();
        for (int i = 0; i < corpusCount; i++) {
            int codecorpus = codecorpusList.getCode(i);
            IndexationCodeList indexationCodeList = dataAccess.getIndexationCodeList(codemotcle, codecorpus);
            int count = indexationCodeList.getCodeCount();
            for (int j = 0; j < count; j++) {
                int codefiche = indexationCodeList.getCode(j);
                list.add(codefiche);
            }
        }
        return CodeListUtils.toCodeList(list);
    }

    public static CodeList getCodemotcleList(DataAccess dataAccess, CodeFilter codemotcleFilter, CodeList codethesaurusList) {
        if (codethesaurusList == null) {
            codethesaurusList = dataAccess.getCodethesaurusList();
        }
        List<Integer> list = new ArrayList<Integer>();
        int thesaurusCount = codethesaurusList.getCodeCount();
        for (int i = 0; i < thesaurusCount; i++) {
            int codethesaurus = codethesaurusList.getCode(i);
            ThesaurusData thesaurusData = dataAccess.getThesaurusDataByCode(codethesaurus);
            int motcleCount = thesaurusData.getMotcleCount();
            for (int j = 0; j < motcleCount; j++) {
                int codemotcle = thesaurusData.getCodeMotcle(j);
                if ((codemotcleFilter == null) || (codemotcleFilter.acceptCode(codemotcle))) {
                    list.add(codemotcle);
                }
            }
        }
        return CodeListUtils.toCodeList(list);
    }

}
