/* ScrutariLib_DB - Copyright (c) 2008-2012 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.scrutari.db.lexiebuilder;

import net.mapeadores.scrutari.lexie.ScrutariLexieUnit;
import net.mapeadores.util.text.collation.CollationUnit;
import net.mapeadores.util.text.collation.map.CollatedKeyMap;
import net.mapeadores.util.text.collation.map.SortedCollatedKeyMap;
import net.mapeadores.util.primitives.PrimUtils;
import java.util.Map;
import java.util.LinkedHashMap;
import java.util.Collection;
import java.util.HashMap;


/**
 *
 * @author Vincent Calame
 */
public class LexieUnitBuilder {

    private Map<Integer, Integer> incrementMap = new HashMap<Integer, Integer>();
    private Map<Integer, CollatedKeyMap> lexieMapByLangMap = new HashMap<Integer, CollatedKeyMap>();
    private Map<Long, ScrutariLexieUnit> lexieUnitMap = new LinkedHashMap<Long, ScrutariLexieUnit>();

    public LexieUnitBuilder() {
    }

    public Collection<ScrutariLexieUnit> getLexieUnitList() {
        return lexieUnitMap.values();
    }

    public LexieOccurence getLexieOccurence(int langInteger, String sourceLexie, boolean inFiche) {
        CollatedKeyMap map = getOrCreateLexieMap(langInteger);
        CollationUnit collationUnit = map.collateString(sourceLexie);
        String collatedString = collationUnit.getCollatedString();
        Long lg = (Long) map.getValueByCollatedKey(collatedString);
        ScrutariLexieUnit scrutariLexieUnit = null;
        if (lg == null) {
            lg = new Long(getNewLexieId(langInteger));
            scrutariLexieUnit = new ScrutariLexieUnit(lg.longValue(), collatedString);
            scrutariLexieUnit.setCanonicalLexie(collationUnit.getSourceString());
            lexieUnitMap.put(lg, scrutariLexieUnit);
            map.putValueByCollatedKey(collatedString, lg);
        } else {
            scrutariLexieUnit = lexieUnitMap.get(lg);
        }
        if (inFiche) {
            scrutariLexieUnit.setHasFicheLink(true);
        } else {
            scrutariLexieUnit.setHasMotcleLink(true);
        }
        return new LexieOccurence(lg.longValue(), collationUnit);
    }

    public Map<Integer, CollatedKeyMap> getLexieMapByLangMap() {
        return lexieMapByLangMap;
    }

    private CollatedKeyMap getOrCreateLexieMap(int langInt) {
        CollatedKeyMap collatedStringMap = lexieMapByLangMap.get(langInt);
        if (collatedStringMap == null) {
            collatedStringMap = new SortedCollatedKeyMap(langInt);
            lexieMapByLangMap.put(langInt, collatedStringMap);
            incrementMap.put(langInt, 1);
        }
        return collatedStringMap;
    }

    private long getNewLexieId(int langInteger) {
        Integer itg = incrementMap.get(langInteger);
        int newid = 1;
        if (itg != null) {
            newid = itg.intValue();
        }
        long l = PrimUtils.toLong(langInteger, newid);
        newid++;
        incrementMap.put(langInteger, newid);
        return l;
    }

}
