/* ScrutariLib_DB - Copyright (c) 2005-2012 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.scrutari.db.util;

import net.mapeadores.scrutari.data.BaseData;
import net.mapeadores.scrutari.data.CorpusData;
import net.mapeadores.scrutari.data.FicheData;
import net.mapeadores.scrutari.data.ThesaurusData;
import net.mapeadores.scrutari.db.DataAccess;
import net.mapeadores.scrutari.db.CodeList;


/**
 * Propose des méthodes statitiques utilitaires pour récupérer des Datarmations sur les bases.
 *
 * @author Vincent Calame
 */
public class DataUtils {

    static final char SEUIL1 = '\u0080';
    static final char SEUIL2 = '\u0800';

    /**
     * Retourne le nombre de caractères supplémentaires introduit par le codage UTF8 du caractère en argument.
     * Par exemple, la méthode retournera 0  pour le caractère « a » qui en UTF-8 s'écrit « a » et elle retournera 1 pour le caractère « é » qui en UTF-8 s'écrit « Ã© ».
     */
    public static int getDecalageUTF8(char c) {
        if (c < SEUIL1) {
            return 0;
        }
        if (c < SEUIL2) {
            return 1;
        }
        return 2;
    }

    private DataUtils() {
    }

    public static String getHref(FicheData ficheData, DataAccess dataAccess, int langUiInteger) {
        BaseData baseData = dataAccess.getBaseDataByCode(dataAccess.getCorpusDataByCode(ficheData.getCodeCorpus()).getCodeBase());
        int checkedLangInteger = baseData.checkLangUi(langUiInteger);
        return ficheData.getHref(checkedLangInteger);
    }

    public static int getFicheCountByCorpus(DataAccess dataAccess, CodeList codecorpusList, int[] intlangList) {
        int resultat = 0;
        if (intlangList != null) {
            int taille = intlangList.length;
            if (codecorpusList == null) {
                for (int i = 0; i < dataAccess.getCorpusCount(); i++) {
                    CorpusData corpusData = dataAccess.getCorpusDataByIndex(i);
                    for (int k = 0; k < taille; k++) {
                        resultat = resultat + corpusData.getFicheCountByLang(intlangList[k]);
                    }
                }
            } else {
                int corpusCount = codecorpusList.getCodeCount();
                for (int i = 0; i < corpusCount; i++) {
                    CorpusData corpusData = dataAccess.getCorpusDataByCode(codecorpusList.getCode(i));
                    for (int k = 0; k < taille; k++) {
                        resultat = resultat + corpusData.getFicheCountByLang(intlangList[k]);
                    }
                }
            }
        } else {
            if (codecorpusList == null) {
                for (int i = 0; i < dataAccess.getCorpusCount(); i++) {
                    CorpusData corpusData = dataAccess.getCorpusDataByIndex(i);
                    resultat = resultat + corpusData.getFicheCount();
                }
            } else {
                int corpusCount = codecorpusList.getCodeCount();
                for (int i = 0; i < corpusCount; i++) {
                    CorpusData corpusData = dataAccess.getCorpusDataByCode(codecorpusList.getCode(i));
                    if (corpusData != null) {
                        resultat = resultat + corpusData.getFicheCount();
                    }
                }
            }
        }
        return resultat;
    }

    public static int getFicheCountByBase(DataAccess dataAccess, CodeList codebaseList, int[] intlangList) {
        int resultat = 0;

        if (intlangList != null) {
            int length = intlangList.length;
            if (codebaseList == null) {
                for (int i = 0; i < dataAccess.getBaseCount(); i++) {
                    BaseData basData = dataAccess.getBaseDataByIndex(i);
                    for (int j = 0; j < basData.getCorpusCount(); j++) {
                        CorpusData corpusData = dataAccess.getCorpusDataByCode(basData.getCodeCorpus(j));
                        for (int k = 0; k < length; k++) {
                            resultat = resultat + corpusData.getFicheCountByLang(intlangList[k]);
                        }
                    }
                }
            } else {
                int baseCount = codebaseList.getCodeCount();
                for (int i = 0; i < baseCount; i++) {
                    BaseData basData = dataAccess.getBaseDataByCode(codebaseList.getCode(i));
                    if (basData != null) {
                        for (int j = 0; j < basData.getCorpusCount(); j++) {
                            CorpusData corpusData = dataAccess.getCorpusDataByCode(basData.getCodeCorpus(j));
                            for (int k = 0; k < length; k++) {
                                resultat = resultat + corpusData.getFicheCountByLang(intlangList[k]);
                            }
                        }
                    }
                }
            }
        } else {
            if (codebaseList == null) {
                for (int i = 0; i < dataAccess.getBaseCount(); i++) {
                    BaseData baseData = dataAccess.getBaseDataByIndex(i);
                    for (int j = 0; j < baseData.getCorpusCount(); j++) {
                        CorpusData corpusData = dataAccess.getCorpusDataByCode(baseData.getCodeCorpus(j));
                        resultat = resultat + corpusData.getFicheCount();
                    }
                }
            } else {
                int baseCount = codebaseList.getCodeCount();
                for (int i = 0; i < baseCount; i++) {
                    BaseData basData = dataAccess.getBaseDataByCode(codebaseList.getCode(i));
                    if (basData != null) {
                        for (int j = 0; j < basData.getCorpusCount(); j++) {
                            CorpusData corpusData = dataAccess.getCorpusDataByCode(basData.getCodeCorpus(j));
                            resultat = resultat + corpusData.getFicheCount();
                        }
                    }
                }
            }
        }
        return resultat;
    }

    public static int getMotcleCountByBase(DataAccess dataAccess, CodeList codebaseList) {
        int resultat = 0;
        if (codebaseList == null) {
            for (int i = 0; i < dataAccess.getBaseCount(); i++) {
                BaseData baseData = dataAccess.getBaseDataByIndex(i);
                for (int j = 0; j < baseData.getThesaurusCount(); j++) {
                    ThesaurusData corpusData = dataAccess.getThesaurusDataByCode(baseData.getCodeThesaurus(j));
                    resultat = resultat + corpusData.getMotcleCount();
                }
            }
        } else {
            int baseCount = codebaseList.getCodeCount();
            for (int i = 0; i < baseCount; i++) {
                BaseData basData = dataAccess.getBaseDataByCode(codebaseList.getCode(i));
                if (basData != null) {
                    for (int j = 0; j < basData.getCorpusCount(); j++) {
                        ThesaurusData corpusData = dataAccess.getThesaurusDataByCode(basData.getCodeThesaurus(j));
                        resultat = resultat + corpusData.getMotcleCount();
                    }
                }
            }
        }
        return resultat;
    }

}
