/* ScrutariLib_Data - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.scrutari.data;

import java.util.ArrayList;
import java.util.List;
import net.scrutari.datauri.CorpusURI;
import net.scrutari.datauri.IndexationKey;
import net.scrutari.datauri.ThesaurusURI;


/**
 *
 * @author Vincent Calame
 */
public class IndexationBuilder {

    private List<Group> groupList = new ArrayList<Group>();
    private Group currentGroup;

    public IndexationBuilder() {
    }

    public int getGroupCount() {
        return groupList.size();
    }

    public Group getGroup(int i) {
        return groupList.get(i);
    }

    public void setCurrentGroup(CorpusURI corpusURI, ThesaurusURI thesaurusURI) {
        Group group = new Group(corpusURI, thesaurusURI);
        groupList.add(group);
        currentGroup = group;
    }

    public void addIndexation(IndexationKey indexationKey, int poids) {
        currentGroup.addIndexation(indexationKey, poids);
    }


    public static class Group {

        private CorpusURI corpusURI;
        private ThesaurusURI thesaurusURI;
        private List<IndexationKey> indexationKeyList = new ArrayList<IndexationKey>();
        private List<Integer> poidsList = new ArrayList<Integer>();
        private int poidsMax = 1;

        private Group(CorpusURI corpusURI, ThesaurusURI thesaurusURI) {
            this.corpusURI = corpusURI;
            this.thesaurusURI = thesaurusURI;
        }

        public CorpusURI getCorpusURI() {
            return corpusURI;
        }

        public ThesaurusURI getThesaurusURI() {
            return thesaurusURI;
        }

        public int getIndexationCount() {
            return indexationKeyList.size();
        }

        public IndexationKey getIndexationKey(int i) {
            return indexationKeyList.get(i);
        }

        public int getPoids(int i) {
            return poidsList.get(i);
        }

        public int getPoidsMax() {
            return poidsMax;
        }

        private void addIndexation(IndexationKey indexationKey, int poids) {
            indexationKeyList.add(indexationKey);
            poidsList.add(poids);
            poidsMax = Math.max(poids, poidsMax);
        }

    }

}
