/* ScrutariLib_Data - Copyright (c) 2005-2012 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.scrutari.data;

import java.io.Serializable;
import java.io.IOException;
import net.mapeadores.util.primitives.io.PrimitivesReader;
import net.mapeadores.util.primitives.io.PrimitivesWriter;


/**
 *
 * @author Vincent Calame
 */
public final class IndexationData implements Serializable {

    private int codecorpus;
    private int codethesaurus;
    private int codemotcle;
    private int codefiche;
    private float ponderation;

    private IndexationData() {
    }

    private IndexationData(int codefiche, int codecorpus, int codemotcle, int codethesaurus, float ponderation) {
        this.codecorpus = codecorpus;
        this.codefiche = codefiche;
        this.codethesaurus = codethesaurus;
        this.codemotcle = codemotcle;
        this.ponderation = ponderation;
    }

    public int getCodeMotcle() {
        return codemotcle;
    }

    public int getCodeThesaurus() {
        return codethesaurus;
    }

    public int getCodeFiche() {
        return codefiche;
    }

    public int getCodeCorpus() {
        return codecorpus;
    }

    public float getPonderation() {
        return ponderation;
    }

    public static IndexationData fromPrimitives(PrimitivesReader primitivesReader) throws IOException {
        IndexationData indexationData = new IndexationData();
        indexationData.codefiche = primitivesReader.readInt();
        indexationData.codemotcle = primitivesReader.readInt();
        indexationData.codecorpus = primitivesReader.readInt();
        indexationData.codethesaurus = primitivesReader.readInt();
        indexationData.ponderation = primitivesReader.readFloat();
        return indexationData;
    }

    public static void toPrimitives(IndexationData indexationData, PrimitivesWriter primitivesWriter) throws IOException {
        primitivesWriter.writeInt(indexationData.codefiche);
        primitivesWriter.writeInt(indexationData.codemotcle);
        primitivesWriter.writeInt(indexationData.codecorpus);
        primitivesWriter.writeInt(indexationData.codethesaurus);
        primitivesWriter.writeFloat(indexationData.ponderation);
    }

    public static IndexationData newInstance(int codefiche, int codecorpus, int codemotcle, int codethesaurus, int poids, int poidsMax) {
        float ponderation = 0;
        if (poids > 1) {
            ponderation = ((float) poids) / ((float) (poidsMax + 1));
        }
        return new IndexationData(codefiche, codecorpus, codemotcle, codethesaurus, ponderation);
    }

}
