/* ScrutariLib_SearchEngine - Copyright (c) 2008-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.scrutari.searchengine;

import net.mapeadores.util.text.SearchSequence;


/**
 *
 * @author Vincent Calame
 */
public interface FicheSearchResult {

    /**
     * Retourne l'identifiant de la recherche.
     */
    public String getIdentifiant();

    /**
     * Retourne la séquence de recherche à l'origine de la recherche.
     */
    public SearchSequence getSearchSequence();

    /**
     * Retourne les langues de la recherche (peut-être null)
     */
    public int[] getLangIntegerArray();

    /**
     *  Langue d'interface à utiliser de préférence.
     */
    public int getLangUi();

    /**
     * Retourne le nombre de fiches dans le résultat de recherche.
     */
    public int getFicheCount();

    /**
     * Retourne le nombre maximum de fiches qui auraient pu se trouver dans
     * le résultat de recherche. Autrement dit, c'est le nombre de fiches qui répondent aux
     * options de recherche (limitation à certaines bases ou corpus) sur lesquelles ont a fait
     * réellement une recherche de présence de mots.
     */
    public int getFicheMaximum();

    /**
     * Retourne le nombre total de mots-clés qui contiennent au moins un mot de
     * la séquence de recherche.
     */
    public int getMotcleCount();

    /**
     * Retourne le nombre total de mots-clés qui contiennent tous les mots de la
     * séquence de recherche.
     */
    public int getCompleteMotcleCount();

    public int getFicheSearchResultGroupCount();

    public FicheSearchResultGroup getFicheSearchResultGroup(int i);

    /**
     * Retourne la i-ième instance de MotcleResult appartenant au résultat.
     * 0 =< i < getAllMotcleCount().
     */
    public MotcleSearchResultInfo getMotcleResult(int i);

}
