/* ScrutariLib_SearchEngine - Copyright (c) 2008 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.scrutari.searchengine.pertinence;

import java.util.List;
import java.util.ArrayList;
import java.util.Arrays;


/**
 *
 * @author Vincent Calame
 */
public class IndexationPertinence {

    private MotclePertinence motclePertinence;
    private List<FichePertinence> fichePertinenceList = new ArrayList<FichePertinence>();
    private FichePertinence[] fichePertinenceArray;
    private int acceptedFicheCount;

    public IndexationPertinence(MotclePertinence motclePertinence) {
        this.motclePertinence = motclePertinence;
    }

    public MotclePertinence getMotclePertinence() {
        return motclePertinence;
    }

    /**
     * Retourne le nombre total de fiches finalement acceptées indexées par le mot-clé.
     */
    public int getAcceptedFicheCount() {
        return acceptedFicheCount;
    }

    /**
     * Retourne la i-ième fiche acceptée indexée par le mot-clé.
     * Ces fiches sont classées par ordre de pertinence.
     */
    public FichePertinence getAcceptedFiche(int i) {
        return fichePertinenceArray[i];
    }

    /**
     * Retourne le nombre maximum de fiches susceptibles d'être associées suivant 
     * les options de recherche. Autrement dit, c'est le nombre total de fiches
     * indexées par le mot-clé dans les corpus et bases sélectionnées.
     */
    public int getFicheMaximum() {
        return fichePertinenceArray.length;
    }

    /*
     * Permet d'indiquer l'indexation d'une fiche par un mot-clé.
     */
    public void addFichePertinence(FichePertinence fichePertinence) {
        fichePertinenceList.add(fichePertinence);
    }

    /*
     * Trie les fiches par pertinence. Appelé en fin de traitement (puisque cela se base sur la valeur de pertinence).
     */
    public void sortPertinenceFiche() {
        int size = fichePertinenceList.size();
        FichePertinence[] array = new FichePertinence[size];
        int acceptIndex = 0;
        int refuseIndex = size - 1;
        for (int i = 0; i < size; i++) {
            FichePertinence fichePertinence = fichePertinenceList.get(i);
            if (fichePertinence.getPertinenceRate() == 0) {
                array[refuseIndex] = fichePertinence;
                refuseIndex--;
            } else {
                array[acceptIndex] = fichePertinence;
                acceptIndex++;
            }
        }
        if (acceptIndex > 1) {
            Arrays.sort(array, 0, acceptIndex, FichePertinence.ACCEPTED_COMPARATOR);
        }
        acceptedFicheCount = acceptIndex;
        fichePertinenceArray = array;
        fichePertinenceList = null;
    }

}
