/* ScrutariLib_SearchEngine - Copyright (c) 2008-2012 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.scrutari.searchengine.seeker;

import net.mapeadores.scrutari.db.CodeList;
import net.mapeadores.scrutari.lexie.LexieOccurrences;
import net.mapeadores.scrutari.db.CodeFilter;
import net.mapeadores.scrutari.db.LexieAccess;
import net.mapeadores.scrutari.lexie.LexieOccurrencesList;
import net.mapeadores.scrutari.searchengine.SearchFilter;
import net.mapeadores.util.text.SearchSequence;
import net.mapeadores.util.text.SubstringPosition;
import net.mapeadores.util.text.collation.CollationUnitPosition;
import net.mapeadores.util.text.collation.map.SearchResultUnit;
import net.mapeadores.util.text.collation.map.SearchResultUnitList;


/**
 *
 * @author Vincent Calame
 */
class SimpleTokenSeeker {

    private LexieSeeker lexieSeeker;
    private int langInteger;
    private SearchSequence.SimpleToken simpleToken;

    SimpleTokenSeeker(LexieSeeker lexieSeeker) {
        this.lexieSeeker = lexieSeeker;
    }

    void setToken(SearchSequence.SimpleToken simpleToken) {
        this.simpleToken = simpleToken;
    }

    void setLangInteger(int langInteger) {
        this.langInteger = langInteger;
    }

    void seek() {
        SearchResultUnitList list = lexieSeeker.search(simpleToken.toString(false), langInteger, simpleToken.getSearchType());
        if (list != null) {
            int count = list.getSearchResultUnitCount();
            for (int j = 0; j < count; j++) {
                SearchResultUnit searchResultItem = list.getSearchResultUnit(j);
                Long lexieId = (Long) searchResultItem.getValue();
                add(lexieId.longValue(), searchResultItem.getSearchTextPositionInCollatedKey());
            }
        }
    }

    private void add(long lexieId, SubstringPosition searchTextePositionInCollatedKey) {
        LexieAccess lexieAccess = lexieSeeker.getLexieAccess();
        int tokenIndex = simpleToken.getTokenIndex();
        SeekOptions seekOptions = lexieSeeker.getSeekOptions();
        SeekHandler seekHandler = lexieSeeker.getSeekHandler();
        SearchFilter searchFilter = seekOptions.getSearchFilter();
        if (seekOptions.isWithFiches()) {
            CodeList codecorpusList = searchFilter.getCodecorpusList();
            CodeFilter codeficheFilter = searchFilter.getCodeficheFilter();
            int corpusCount = codecorpusList.getCodeCount();
            for (int j = 0; j < corpusCount; j++) {
                int codecorpus = codecorpusList.getCode(j);
                LexieOccurrencesList lexieOccurrencesList = lexieAccess.getLexieOccurrencesList(lexieId, codecorpus);
                int lexieCount = lexieOccurrencesList.getLexieOccurrencesCount();
                for (int k = 0; k < lexieCount; k++) {
                    LexieOccurrences lexieOccurences = lexieOccurrencesList.getLexieOccurrences(k);
                    int codefiche = lexieOccurences.getCode();
                    if (codeficheFilter.acceptCode(codefiche)) {
                        CollationUnitPosition bestCollationUnitPosition = lexieOccurences.getBestPositionInAlinea(0);
                        int bestAlineaInteger = lexieOccurences.getAlineaInteger(0);
                        SubstringPosition sourcePosition = bestCollationUnitPosition.convertToPositionInWholeSource(searchTextePositionInCollatedKey);
                        seekHandler.addFichePertinence(codefiche, codecorpus, tokenIndex, bestAlineaInteger, sourcePosition);
                        lexieSeeker.addFiche(codefiche, codecorpus);
                    }
                }
            }
        }
        if (seekOptions.isWithMotscles()) {
            CodeList codethesaurusList = searchFilter.getCodethesaurusList();
            CodeFilter codemotcleFilter = searchFilter.getCodemotcleFilter();
            int thesaurusCount = codethesaurusList.getCodeCount();
            for (int j = 0; j < thesaurusCount; j++) {
                int codethesaurus = codethesaurusList.getCode(j);
                LexieOccurrencesList lexieOccurrencesList = lexieAccess.getLexieOccurrencesList(lexieId, codethesaurus);
                int lexieCount = lexieOccurrencesList.getLexieOccurrencesCount();
                for (int k = 0; k < lexieCount; k++) {
                    LexieOccurrences p = lexieOccurrencesList.getLexieOccurrences(k);
                    int codemotcle = p.getCode();
                    if (codemotcleFilter.acceptCode(codemotcle)) {
                        CollationUnitPosition bestCollationUnitPosition = p.getBestPositionInAlinea(0);
                        SubstringPosition sourcePosition = bestCollationUnitPosition.convertToPositionInWholeSource(searchTextePositionInCollatedKey);
                        float coverRate = ((float) sourcePosition.getLength()) / (p.getAlineaWholeLength(0));
                        seekHandler.addMotclePertinence(codemotcle, codethesaurus, tokenIndex, langInteger, sourcePosition, coverRate);
                        lexieSeeker.addMotcle(codemotcle, codethesaurus);
                    }
                }
            }
        }
    }

}
