/* ScrutariLib_SearchEngine - Copyright (c) 2010-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.scrutari.searchengine.supermotcle;

import net.mapeadores.scrutari.supermotcle.Supermotcle;
import net.mapeadores.scrutari.supermotcle.SupermotcleMap;
import net.mapeadores.scrutari.searchengine.SearchTokenPositionList;
import net.mapeadores.scrutari.searchengine.SearchTokenPositionListComparator;
import net.mapeadores.scrutari.searchengine.SearchUtils;
import net.mapeadores.scrutari.searchengine.SupermotcleSearchResult;
import net.mapeadores.scrutari.searchengine.SupermotcleSearchResultInfo;
import net.mapeadores.scrutari.searchengine.util.SearchTokenPositionArrayBuilder;
import net.mapeadores.util.text.SearchSequence;
import net.mapeadores.util.text.SubstringPosition;
import net.mapeadores.util.text.TextConstants;
import net.mapeadores.util.text.collation.CollationUnit;
import net.mapeadores.util.text.collation.map.SearchResultUnit;
import net.mapeadores.util.text.collation.map.SearchResultUnitList;
import net.mapeadores.util.text.collation.map.ValueFilter;
import java.util.Comparator;
import java.util.HashMap;
import java.util.Map;
import java.util.TreeSet;
import net.mapeadores.scrutari.searchengine.impl.MutableSearchTokenPositionList;


/**
 *
 * @author Vincent Calame
 */
public class SupermotcleSeeker {

    private Map<Integer, InternalSupermotcleSearchResultInfo> previousMap;
    private boolean empty = false;
    private SupermotcleMap supermotcleMap;
    private InternalValueFilter valueFilter = new InternalValueFilter();
    private int tokenCount;

    private SupermotcleSeeker(SupermotcleMap supermotcleMap, int tokenCount) {
        this.supermotcleMap = supermotcleMap;
        this.tokenCount = tokenCount;
    }

    public static SupermotcleSearchResult search(SupermotcleMap supermotcleMap, SearchSequence searchSequence) {
        int tokenCount = searchSequence.getTokenCount();
        SupermotcleSeeker seeker = new SupermotcleSeeker(supermotcleMap, tokenCount);
        for (int i = 0; i < tokenCount; i++) {
            SearchSequence.Token token = searchSequence.getToken(i);
            if (token.isMultiToken()) {
                return SearchUtils.EMPTY_SUPERMOTCLE_SEARCHRESULT;
            } else {
                SearchSequence.SimpleToken simpleToken = (SearchSequence.SimpleToken) token;
                boolean suite = seeker.seekToken(simpleToken);
                if (!suite) {
                    break;
                }
            }
        }
        return seeker.getResult();
    }

    private boolean seekToken(SearchSequence.SimpleToken token) {
        SearchResultUnitList list = supermotcleMap.search(token.toString(false), TextConstants.CONTAINS, valueFilter);
        int unitCount = list.getSearchResultUnitCount();
        if (unitCount == 0) {
            empty = true;
            return false;
        }
        Map<Integer, InternalSupermotcleSearchResultInfo> currentMap = new HashMap<Integer, InternalSupermotcleSearchResultInfo>();
        for (int k = 0; k < unitCount; k++) {
            SearchResultUnit searchResultUnit = list.getSearchResultUnit(k);
            Supermotcle supermotcle = (Supermotcle) searchResultUnit.getValue();
            int code = supermotcle.getCode();
            InternalSupermotcleSearchResultInfo currentInfo = null;
            if (previousMap == null) {
                currentInfo = new InternalSupermotcleSearchResultInfo(supermotcle);
            } else {
                currentInfo = previousMap.get(code); //existe du fait de ValueFilter

            }
            currentInfo.addSearchResultUnit(token.getTokenIndex(), searchResultUnit);
            currentMap.put(code, currentInfo);
        }
        if (currentMap.size() == 0) {
            empty = true;
            return false;
        }
        previousMap = currentMap;
        return true;
    }

    private SupermotcleSearchResult getResult() {
        if ((empty) || (previousMap == null)) {
            return SearchUtils.EMPTY_SUPERMOTCLE_SEARCHRESULT;
        }
        InternalSupermotcleSearchResult supermotcleSearchResult = new InternalSupermotcleSearchResult();
        supermotcleSearchResult.init(previousMap);
        return supermotcleSearchResult;
    }

    public boolean isEmpty() {
        return empty;
    }


    private class InternalValueFilter implements ValueFilter {

        public boolean acceptValue(Object value) {
            if (previousMap == null) {
                return true;
            }
            return previousMap.containsKey(((Supermotcle) value).getCode());
        }

    }


    private static class InternalSupermotcleSearchResult implements SupermotcleSearchResult {

        private InternalSupermotcleSearchResultInfo[] array;

        InternalSupermotcleSearchResult() {
        }

        public int getSupermotcleSearchResultInfoCount() {
            return array.length;
        }

        public SupermotcleSearchResultInfo getSupermotcleSearchResultInfo(int i) {
            return array[i];
        }

        private void init(Map<Integer, InternalSupermotcleSearchResultInfo> map) {
            TreeSet<InternalSupermotcleSearchResultInfo> sortedSet = new TreeSet<InternalSupermotcleSearchResultInfo>(new ResultComparator());
            sortedSet.addAll(map.values());
            int size = sortedSet.size();
            array = new InternalSupermotcleSearchResultInfo[size];
            int p = (size - 1);
            for (InternalSupermotcleSearchResultInfo info : sortedSet) {
                array[p] = info;
                p--;
            }
        }

    }


    private class InternalSupermotcleSearchResultInfo implements SupermotcleSearchResultInfo {

        private SearchTokenPositionArrayBuilder searchTokenPositionArrayBuilder;
        private MutableSearchTokenPositionList searchTokenPositionList;
        private Supermotcle supermotcle;

        InternalSupermotcleSearchResultInfo(Supermotcle supermotcle) {
            this.supermotcle = supermotcle;
            this.searchTokenPositionArrayBuilder = new SearchTokenPositionArrayBuilder(tokenCount);
        }

        public Supermotcle getSupermotcle() {
            return supermotcle;
        }

        public SearchTokenPositionList getSearchTokentPositionList() {
            if (searchTokenPositionList == null) {
                searchTokenPositionList = new MutableSearchTokenPositionList();
                searchTokenPositionList.setAllTokens(true);
                searchTokenPositionList.setSearchTokenPositionArray(searchTokenPositionArrayBuilder.getSearchTokenPositionArray());
            }
            return searchTokenPositionList;
        }

        private void addSearchResultUnit(int tokenIndex, SearchResultUnit sru) {
            CollationUnit collationUnit = supermotcle.getCollationUnit();
            SubstringPosition positionInSourceString = collationUnit.getSubstringPositionInSourceString(sru.getSearchTextPositionInCollatedKey());
            searchTokenPositionArrayBuilder.add(tokenIndex, positionInSourceString);
        }

    }


    private static class ResultComparator implements Comparator<SupermotcleSearchResultInfo> {

        private SearchTokenPositionListComparator searchTokenPositionListComparator = new SearchTokenPositionListComparator();

        ResultComparator() {
        }

        public int compare(SupermotcleSearchResultInfo s1, SupermotcleSearchResultInfo s2) {
            Supermotcle smc1 = s1.getSupermotcle();
            Supermotcle smc2 = s2.getSupermotcle();
            int cd1 = smc1.getCode();
            int cd2 = smc2.getCode();
            if (cd1 == cd2) {
                return 0;
            }
            int baseCount1 = smc1.getSupermotcleIndexationCount();
            int baseCount2 = smc2.getSupermotcleIndexationCount();
            if (baseCount1 > baseCount2) {
                return 1;
            }
            if (baseCount2 < baseCount1) {
                return -1;
            }
            int indexationCount1 = smc1.getIndexationCount();
            int indexationCount2 = smc2.getIndexationCount();
            if (indexationCount1 > indexationCount2) {
                return 1;
            }
            if (indexationCount1 < indexationCount2) {
                return -1;
            }
            int compare = searchTokenPositionListComparator.compare(s1.getSearchTokentPositionList(), s2.getSearchTokentPositionList());
            if (compare != 0) {
                return compare;
            }
            int ssl1 = smc1.getCollationUnit().getSourceString().length();
            int ssl2 = smc1.getCollationUnit().getSourceString().length();
            if (ssl1 < ssl2) {
                return 1;
            }
            if (ssl1 > ssl2) {
                return -1;
            }
            if (cd1 < cd2) {
                return 1;
            }
            if (cd1 > cd2) {
                return -1;
            }
            return 0;
        }

        public boolean equals(Object obj) {
            return (obj == this);
        }

    }

}
