/* ScrutariLib_Supermotcle - Copyright (c) 2010-2012 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.scrutari.supermotcle;

import net.mapeadores.scrutari.data.BaseData;
import net.mapeadores.scrutari.db.DataAccess;
import net.mapeadores.scrutari.db.CodeList;
import net.mapeadores.scrutari.db.ElementCodeList;
import net.mapeadores.scrutari.db.CodeListUtils;
import net.mapeadores.util.text.collation.CollationUnit;
import java.util.Comparator;
import java.util.List;
import java.util.ArrayList;
import java.util.Set;
import java.util.LinkedHashSet;
import java.util.TreeSet;
import net.mapeadores.scrutari.db.IndexationCodeList;


/**
 *
 * @author Vincent Calame
 */
public class SupermotcleBuilder {

    private int langInteger;
    private int code;
    private CollationUnit collationUnit;
    private List<Integer> codemotcleList = new ArrayList<Integer>();
    private List<Integer> codethesaurusList = new ArrayList<Integer>();
    private DataAccess dataAccess;
    private List<SupermotcleIndexationBuilder> builderList = new ArrayList<SupermotcleIndexationBuilder>();
    private final static SupermotcleIndexationComparator comparator = new SupermotcleIndexationComparator();

    public SupermotcleBuilder(DataAccess dataAccess, int langInteger, int code, int codethesaurus, int codemotcle, CollationUnit collationUnit) {
        this.dataAccess = dataAccess;
        this.langInteger = langInteger;
        this.code = code;
        this.collationUnit = collationUnit;
        codemotcleList.add(codemotcle);
        codethesaurusList.add(codethesaurus);
        int baseCount = dataAccess.getBaseCount();
        for (int i = 0; i < baseCount; i++) {
            BaseData baseData = dataAccess.getBaseDataByIndex(i);
            int codebase = baseData.getCodeBase();
            SupermotcleIndexationBuilder builder = new SupermotcleIndexationBuilder(codebase);
            builderList.add(builder);
            int corpusCount = baseData.getCorpusCount();
            for (int j = 0; j < corpusCount; j++) {
                int codecorpus = baseData.getCodeCorpus(j);
                builder.initCorpus(codecorpus);
            }
        }
        checkIndexation(codemotcle);
    }

    public void addMotcle(int codethesaurus, int codemotcle, CollationUnit newCollationUnit) { //On compare pour voir quelle est la meilleur « collation »
        String oldSourceString = collationUnit.getSourceString();
        String newSourceString = newCollationUnit.getSourceString();
        boolean replace = false;
        int newLength = newSourceString.length();
        int oldLength = oldSourceString.length();
        if (newLength < oldLength) {
            replace = true;
        } else if (newLength == oldLength) {
            if (newSourceString.compareTo(oldSourceString) > 0) {
                replace = true;
            }
        }
        if (replace) {
            collationUnit = newCollationUnit;
        }
        codemotcleList.add(codemotcle);
        codethesaurusList.add(codethesaurus);
        checkIndexation(codemotcle);
    }

    public Supermotcle toSupermotcle() {
        InternalSupermotcle is = new InternalSupermotcle(this);
        return is;
    }

    private void checkIndexation(int codemotcle) {
        int baseCount = builderList.size();
        for (int i = 0; i < baseCount; i++) {
            SupermotcleIndexationBuilder builder = builderList.get(i);
            builder.checkMotcle(codemotcle);
        }
    }


    private class SupermotcleIndexationBuilder {

        private int codebase;
        private List<CorpusListBuilder> corpusListBuilderList = new ArrayList<CorpusListBuilder>();

        SupermotcleIndexationBuilder(int codebase) {
            this.codebase = codebase;
        }

        private void initCorpus(int codecorpus) {
            corpusListBuilderList.add(new CorpusListBuilder(codecorpus));
        }

        private void checkMotcle(int codemotcle) {
            int size = corpusListBuilderList.size();
            for (int i = 0; i < size; i++) {
                CorpusListBuilder corpusListBuilder = corpusListBuilderList.get(i);
                IndexationCodeList indexationCodeList = dataAccess.getIndexationCodeList(codemotcle, corpusListBuilder.codecorpus);
                int ficheCount = indexationCodeList.getCodeCount();
                for (int j = 0; j < ficheCount; j++) {
                    corpusListBuilder.addCodeFiche(indexationCodeList.getCode(j));
                }
            }
        }

    }


    private class CorpusListBuilder {

        private int codecorpus;
        private Set<Integer> codeficheSet = new LinkedHashSet<Integer>();
        private CodeList codeList;

        CorpusListBuilder(int codecorpus) {
            this.codecorpus = codecorpus;
        }

        private void addCodeFiche(int codefiche) {
            codeficheSet.add(codefiche);
        }

        private boolean hasFiche() {
            return (!codeficheSet.isEmpty());
        }

        private void endInit() {
            int size = codeficheSet.size();
            int[] array = new int[size];
            int p = 0;
            for (Integer itg : codeficheSet) {
                array[p] = itg.intValue();
                p++;
            }
            codeList = CodeListUtils.toCodeList(array);
        }

    }


    private static class InternalSupermotcle implements Supermotcle {

        private int langInteger;
        private int code;
        private CollationUnit collationUnit;
        private ElementCodeList codemotcleList;
        private int indexationCount = 0;
        private SupermotcleIndexation[] supermotcleIndexationArray;

        InternalSupermotcle(SupermotcleBuilder builder) {
            this.langInteger = builder.langInteger;
            this.code = builder.code;
            this.collationUnit = builder.collationUnit;
            codemotcleList = CodeListUtils.toElementCodeList(builder.codemotcleList, builder.codethesaurusList);
            List<SupermotcleIndexationBuilder> builderList = builder.builderList;
            int builderSize = builderList.size();
            Set<SupermotcleIndexation> resultSet = new TreeSet<SupermotcleIndexation>(comparator);
            for (int i = 0; i < builderSize; i++) {
                SupermotcleIndexationBuilder supermotcleIndexationBuiler = builderList.get(i);
                SupermotcleIndexation supermotcleIndexation = new InternalSupermotcleIndexation(supermotcleIndexationBuiler);
                int ic = supermotcleIndexation.getIndexationCount();
                if (ic > 0) {
                    indexationCount = indexationCount + ic;
                    resultSet.add(supermotcleIndexation);
                }
            }
            supermotcleIndexationArray = resultSet.toArray(new SupermotcleIndexation[resultSet.size()]);
        }

        public int getLangInteger() {
            return langInteger;
        }

        public int getCode() {
            return code;
        }

        public CollationUnit getCollationUnit() {
            return collationUnit;
        }

        public ElementCodeList getCodemotcleList() {
            return codemotcleList;
        }

        public int getSupermotcleIndexationCount() {
            return supermotcleIndexationArray.length;
        }

        public SupermotcleIndexation getSupermotcleIndexation(int i) {
            return supermotcleIndexationArray[i];
        }

        public int getIndexationCount() {
            return indexationCount;
        }

    }


    private static class SupermotcleIndexationComparator implements Comparator<SupermotcleIndexation> {

        public int compare(SupermotcleIndexation s1, SupermotcleIndexation s2) {
            int ic1 = s1.getIndexationCount();
            int ic2 = s2.getIndexationCount();
            if (ic1 > ic2) {
                return -1;
            }
            if (ic1 < ic2) {
                return 1;
            }
            int c1 = s1.getCodeBase();
            int c2 = s2.getCodeBase();
            if (c1 < c2) {
                return -1;
            }
            if (c2 > c1) {
                return 1;
            }
            return 0;
        }

    }


    private static class InternalSupermotcleIndexation implements SupermotcleIndexation {

        private int codebase;
        private int indexationCount = 0;
        private int[] codecorpusArray;
        private CodeList[] codeficheListByCorpusArray;

        InternalSupermotcleIndexation(SupermotcleIndexationBuilder builder) {
            this.codebase = builder.codebase;
            List<CorpusListBuilder> corpusListBuilderList = builder.corpusListBuilderList;
            int size = corpusListBuilderList.size();
            List<CorpusListBuilder> validList = new ArrayList<CorpusListBuilder>();
            for (int i = 0; i < size; i++) {
                CorpusListBuilder corpusListBuilder = corpusListBuilderList.get(i);
                if (corpusListBuilder.hasFiche()) {
                    corpusListBuilder.endInit();
                    validList.add(corpusListBuilder);
                }
            }
            int size2 = validList.size();
            codecorpusArray = new int[size2];
            codeficheListByCorpusArray = new CodeList[size2];
            for (int i = 0; i < size2; i++) {
                CorpusListBuilder corpusListBuilder = validList.get(i);
                codecorpusArray[i] = corpusListBuilder.codecorpus;
                codeficheListByCorpusArray[i] = corpusListBuilder.codeList;
                indexationCount = indexationCount + corpusListBuilder.codeList.getCodeCount();
            }
        }

        public int getCodeBase() {
            return codebase;
        }

        public int getIndexationCount() {
            return indexationCount;
        }

        public int getCorpusCount() {
            return codecorpusArray.length;
        }

        public int getCodeCorpus(int i) {
            return codecorpusArray[i];
        }

        public CodeList getCodeficheListByCorpus(int i) {
            return codeficheListByCorpusArray[i];
        }

    }

}
