/* ScrutariLib_Supermotcle - Copyright (c) 2009-2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.scrutari.supermotcle;

import net.mapeadores.util.collections.IntHashMap;
import net.mapeadores.util.primitives.PrimUtils;
import java.util.Map;
import java.util.HashMap;
import java.util.List;
import java.util.ArrayList;


/**
 *
 * @author Vincent Calame
 */
public class SupermotcleCroisement {

    private List<Couple> coupleList = new ArrayList<Couple>();
    private IntHashMap ficheMap = new IntHashMap();
    private Map<Long, Couple> coupleMap = new HashMap<Long, Couple>();

    public SupermotcleCroisement(int langInteger) {
    }

    void addIndexation(int codesupermotcle, int codefiche) {
        FicheInfo ficheInfo = getOrCreateFicheInfo(codefiche);
        ficheInfo.add(codesupermotcle);
    }

    void endInit() {
    }

    private FicheInfo getOrCreateFicheInfo(int codefiche) {
        FicheInfo ficheInfo = (FicheInfo) ficheMap.get(codefiche);
        if (ficheInfo == null) {
            ficheInfo = new FicheInfo();
            ficheMap.put(codefiche, ficheInfo);
        }
        return ficheInfo;
    }

    public int getCoupleCount() {
        return coupleList.size();
    }

    public Couple getCouple(int i) {
        return coupleList.get(i);
    }


    public static class Couple {

        private int code1;
        private int code2;
        private int total = 1;

        Couple(int code1, int code2) {
            this.code1 = code1;
            this.code2 = code2;
        }

        public int getCode1() {
            return code1;
        }

        public int getCode2() {
            return code2;
        }

        public int getTotal() {
            return total;
        }

        void incremente() {
            total++;
        }

    }

    private void checkCroisement(int code1, int code2) {
        Long l = getKey(code1, code2);
        Couple couple = coupleMap.get(l);
        if (couple == null) {
            couple = new Couple(code1, code2);
            coupleMap.put(l, couple);
            coupleList.add(couple);
        } else {
            couple.incremente();
        }
    }


    private class FicheInfo {

        List<Integer> supermotcleList = new ArrayList<Integer>();

        FicheInfo() {
        }

        void add(int codesupermotcle) {
            int size = supermotcleList.size();
            for (int i = 0; i < size; i++) {
                int otherCode = supermotcleList.get(i);
                checkCroisement(otherCode, codesupermotcle);
            }
            supermotcleList.add(codesupermotcle);
        }

    }

    private Long getKey(int code1, int code2) {
        if (code1 > code2) {
            return new Long(PrimUtils.toLong(code2, code1));
        } else {
            return new Long(PrimUtils.toLong(code1, code2));
        }
    }

}
