/* ScrutariLib_Supermotcle - Copyright (c) 2009-2012 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.scrutari.supermotcle;

import net.mapeadores.scrutari.data.ThesaurusData;
import net.mapeadores.scrutari.data.MotcleData;
import net.mapeadores.scrutari.db.DataAccess;
import net.mapeadores.util.text.Libelle;
import java.util.Map;
import java.util.HashMap;
import net.mapeadores.scrutari.db.ScrutariDB;


/**
 *
 * @author Vincent Calame
 */
public class SupermotcleManager {

    private boolean isInit = false;
    private Map<Integer, SupermotcleMap> mapOfSupermotcleMap = new HashMap<Integer, SupermotcleMap>();
    private Map<Integer, SupermotcleCroisement> croisementMap = new HashMap<Integer, SupermotcleCroisement>();

    public SupermotcleManager() {
    }

    public void clear() {
        mapOfSupermotcleMap.clear();
        croisementMap.clear();
        isInit = false;
    }

    public boolean isInit() {
        return isInit;
    }

    public void update(ScrutariDB scrutariDB) {
        clear();
        DataAccess dataAccess = scrutariDB.openDataAccess();
        SupermotcleManagerBuilder builder = new SupermotcleManagerBuilder(dataAccess);
        builder.build();
        dataAccess.close();
        isInit = true;
    }

    public SupermotcleMap getSupermotcleMap(int langInteger) {
        return mapOfSupermotcleMap.get(langInteger);
    }

    public SupermotcleCroisement getSupermotcleCroisement(int langInteger) {
        SupermotcleCroisement supermotcleCroisement = croisementMap.get(langInteger);
        if (supermotcleCroisement == null) {
            supermotcleCroisement = initSupermotcleCroisement(langInteger);
            croisementMap.put(langInteger, supermotcleCroisement);
        }
        return supermotcleCroisement;
    }

    private SupermotcleCroisement initSupermotcleCroisement(int langInteger) {
        SupermotcleCroisement supermotcleCroisement = new SupermotcleCroisement(langInteger);
        SupermotcleMap supermotcleMap = getSupermotcleMap(langInteger);
        if (supermotcleMap == null) {
            return supermotcleCroisement;
        }
        supermotcleCroisement.endInit();
        return supermotcleCroisement;
    }


    private class SupermotcleManagerBuilder {

        private Map<Integer, SupermotcleMapBuilder> builderMap = new HashMap<Integer, SupermotcleMapBuilder>();
        private DataAccess dataManager;

        SupermotcleManagerBuilder(DataAccess dataManager) {
            this.dataManager = dataManager;
        }

        private void build() {
            int thesaurusCount = dataManager.getThesaurusCount();
            for (int i = 0; i < thesaurusCount; i++) {
                ThesaurusData thesaurusData = dataManager.getThesaurusDataByIndex(i);
                int motcleCount = thesaurusData.getMotcleCount();
                for (int j = 0; j < motcleCount; j++) {
                    int codemotcle = thesaurusData.getCodeMotcle(j);
                    MotcleData motcleData = dataManager.getMotcleDataByCode(codemotcle);
                    initMotcle(thesaurusData, motcleData);
                }
            }
            for (Map.Entry<Integer, SupermotcleMapBuilder> entry : builderMap.entrySet()) {
                Integer itg = entry.getKey();
                SupermotcleMapBuilder builder = entry.getValue();
                SupermotcleMap supermotcleMap = builder.toSupermotcleMap();
                mapOfSupermotcleMap.put(itg, supermotcleMap);
            }
        }

        private SupermotcleMapBuilder getOrCreateSupermotcleMapBuilder(int langInteger) {
            SupermotcleMapBuilder supermotcleMapBuilder = builderMap.get(langInteger);
            if (supermotcleMapBuilder == null) {
                supermotcleMapBuilder = new SupermotcleMapBuilder(dataManager, langInteger);
                builderMap.put(langInteger, supermotcleMapBuilder);
            }
            return supermotcleMapBuilder;
        }

        private void initMotcle(ThesaurusData thesaurusData, MotcleData motcleData) {
            int codemotcle = motcleData.getCodeMotcle();
            int libelleCount = motcleData.getLibelleCount();
            for (int i = 0; i < libelleCount; i++) {
                Libelle libelle = motcleData.getLibelleByIndex(i);
                SupermotcleMapBuilder supermotcleMapBuilder = getOrCreateSupermotcleMapBuilder(libelle.getLangInteger());
                supermotcleMapBuilder.addMotcle(thesaurusData.getCodeBase(), thesaurusData.getCodeThesaurus(), codemotcle, libelle.getLibelleString());
            }
        }

    }

}
