/* ScrutariLib_Supermotcle - Copyright (c) 2010-2012 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.scrutari.supermotcle;

import net.mapeadores.scrutari.db.DataAccess;
import net.mapeadores.util.text.collation.CollationUnit;
import net.mapeadores.util.text.collation.map.CollatedKeyMap;
import net.mapeadores.util.text.collation.map.SortedCollatedKeyMap;
import net.mapeadores.util.text.collation.map.SearchResultUnit;
import net.mapeadores.util.text.collation.map.SearchResultUnitList;
import net.mapeadores.util.text.collation.map.ValueFilter;
import net.mapeadores.util.text.SearchSequence;
import net.mapeadores.util.collections.IntHashMap;
import java.util.Map;
import java.util.HashMap;
import java.util.List;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Set;
import java.util.HashSet;
import java.util.Collections;


/**
 *
 * @author Vincent Calame
 */
public class SupermotcleMapBuilder {

    private DataAccess dataAccess;
    private Map<String, SupermotcleBuilder> builderMap = new HashMap<String, SupermotcleBuilder>();
    private CollatedKeyMap collatedKeyMap;
    private int langInteger;
    private int currentCode = 1;

    public SupermotcleMapBuilder(DataAccess dataAccess, int langInteger) {
        this.dataAccess = dataAccess;
        this.langInteger = langInteger;
        this.collatedKeyMap = new SortedCollatedKeyMap(langInteger);
    }

    public void addMotcle(int codebase, int codethesaurus, int codemotcle, String libelleString) {
        CollationUnit collationUnit = collatedKeyMap.collateString(libelleString);
        String collatedKey = collationUnit.getCollatedString();
        SupermotcleBuilder supermotcleBuilder = builderMap.get(collatedKey);
        if (supermotcleBuilder == null) {
            supermotcleBuilder = new SupermotcleBuilder(dataAccess, langInteger, currentCode, codethesaurus, codemotcle, collationUnit);
            currentCode++;
            builderMap.put(collatedKey, supermotcleBuilder);
            collatedKeyMap.putValueByCollatedKey(collatedKey, supermotcleBuilder);
        } else {
            supermotcleBuilder.addMotcle(codethesaurus, codemotcle, collationUnit);
        }
    }

    public SupermotcleMap toSupermotcleMap() {
        InternalSupermotcleMap supermotcleMap = new InternalSupermotcleMap(this);
        return supermotcleMap;
    }


    private static class InternalSupermotcleMap implements SupermotcleMap {

        private CollatedKeyMap collatedKeyMap;
        private IntHashMap supermotcleMap = new IntHashMap();

        InternalSupermotcleMap(SupermotcleMapBuilder builder) {
            this.collatedKeyMap = builder.collatedKeyMap;
            Map<String, SupermotcleBuilder> builderMap = builder.builderMap;
            for (SupermotcleBuilder supermotcleBuilder : builderMap.values()) {
                Supermotcle supermotcle = supermotcleBuilder.toSupermotcle();
                supermotcleMap.put(supermotcle.getCode(), supermotcle);
                collatedKeyMap.putValueByCollatedKey(supermotcle.getCollationUnit().getCollatedString(), supermotcle);
            }
        }

        public Supermotcle getSupermotcleByCode(int code) {
            return (Supermotcle) supermotcleMap.get(code);
        }

        public SearchResultUnitList search(String s, int searchType, ValueFilter valueFilter) {
            return collatedKeyMap.search(s, searchType, valueFilter);
        }

        public List<Supermotcle> getSupermotcleList(SearchSequence filterSearchSequence) {
            if (filterSearchSequence == null) {
                return getSupermotcleList(collatedKeyMap.values());
            }
            int count = filterSearchSequence.getTokenCount();
            Set<Supermotcle> resultSet = new HashSet<Supermotcle>();
            for (int i = 0; i < count; i++) {
                SearchSequence.Token token = filterSearchSequence.getToken(i);
                if (token.isMultiToken()) {
                    return null;
                } else {
                    SearchSequence.SimpleToken simpleToken = (SearchSequence.SimpleToken) token;
                    SearchResultUnitList list = collatedKeyMap.search(simpleToken.toString(false), simpleToken.getSearchType(), null);
                    int unitCount = list.getSearchResultUnitCount();
                    if (unitCount == 0) {
                        return Collections.emptyList();
                    }
                    Set<Supermotcle> newSet = new HashSet<Supermotcle>();
                    for (int k = 0; k < unitCount; k++) {
                        SearchResultUnit searchResultItem = list.getSearchResultUnit(i);
                        Supermotcle obj = (Supermotcle) searchResultItem.getValue();
                        if (i == 0) {
                            resultSet.add(obj);
                        } else if (resultSet.contains(obj)) {
                            newSet.add(obj);
                        }
                    }
                    if (i != 0) {
                        if (newSet.size() == 0) {
                            return Collections.emptyList();
                        }
                        resultSet = newSet;
                    }
                }
            }
            return getSupermotcleList(resultSet);
        }

        private List<Supermotcle> getSupermotcleList(Collection collection) {
            List<Supermotcle> list = new ArrayList<Supermotcle>();
            for (Object obj : collection) {
                list.add((Supermotcle) obj);
            }
            return list;
        }

    }

}
