/* ScrutariLib_Xml - Copyright (c) 2008-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.scrutari.xml.xmlpull;

import net.scrutari.datauri.ThesaurusURI;
import net.scrutari.datauri.MotcleURI;
import net.mapeadores.scrutari.data.ThesaurusData;
import net.mapeadores.scrutari.data.MotcleData;
import net.mapeadores.util.langinteger.LangIntegerException;
import net.mapeadores.util.text.Libelle;
import net.mapeadores.util.xml.xmlpull.DefaultElementHandler;
import net.mapeadores.util.xml.xmlpull.XmlPullElementHandler;
import net.mapeadores.util.xml.xmlpull.LibelleElementHandler;
import net.mapeadores.scrutari.xml.*;
import java.util.Map;


/**
 *
 * @author Vincent Calame
 */
public class ThesaurusHandler extends DefaultElementHandler implements ErrorKeys {

    private Parameters parameters;
    private ThesaurusURI thesaurusURI;
    private ThesaurusData.Builder thesaurusDataBuilder;
    private boolean metadataDone = false;
    private MotcleHandler motcleHandler = new MotcleHandler();

    public ThesaurusHandler(Map<String, String> attributeMap, Parameters parameters) {
        String thesaurusName = getAttributeString(attributeMap, "thesaurus-name");
        if (thesaurusName == null) {
            parameters.getScrutariXmlErrorHandler().addWarningLog(WARNING_XML_THESAURUS_MISSINGTHESAURUSNAME);
            throw new IllegalArgumentException();
        }
        this.parameters = parameters;
        thesaurusURI = new ThesaurusURI(parameters.getBaseURI(), thesaurusName);
        if (!parameters.testDoublon(thesaurusURI)) {
            parameters.getScrutariXmlErrorHandler().addWarningLog(WARNING_XML_THESAURUS_DOUBLETHESAURUSNAME, thesaurusURI);
            throw new IllegalArgumentException();
        }
        thesaurusURI = (ThesaurusURI) parameters.intern(thesaurusURI);
        thesaurusDataBuilder = new ThesaurusData.Builder(thesaurusURI);
    }

    public ThesaurusData.Builder getThesaurusDataBuilder() {
        if (!metadataDone) {
            return null;
        }
        return thesaurusDataBuilder;
    }

    @Override
    public XmlPullElementHandler startSubElement(String tagname, Map<String, String> attributeMap) {
        if (!metadataDone) {
            if (tagname.equals("thesaurus-metadata")) {
                metadataDone = true;
                return new MetadataHandler();
            } else {
                parameters.getScrutariXmlErrorHandler().addWarningLog(WARNING_XML_THESAURUS_MISSINGMETADATA, thesaurusURI);
                return null;
            }
        } else {
            if (tagname.equals("motcle")) {
                String motcleId = getAttributeString(attributeMap, "motcle-id");
                if (motcleId == null) {
                    parameters.getScrutariXmlErrorHandler().addWarningLog(WARNING_XML_MOTCLE_MISSINGMOTCLEID);
                    return null;
                }
                MotcleURI motcleURI = new MotcleURI(thesaurusURI, motcleId);
                if (!parameters.testDoublon(motcleURI)) {
                    parameters.getScrutariXmlErrorHandler().addWarningLog(WARNING_XML_MOTCLE_DOUBLEMOTCLEID, motcleURI);
                    return null;
                }
                motcleURI = (MotcleURI) parameters.intern(motcleURI);
                motcleHandler.setMotcleURI(motcleURI);
                return motcleHandler;
            } else {
                return null;
            }
        }
    }

    @Override
    public void endSubElement(XmlPullElementHandler elementHandler) {
        if (elementHandler instanceof MotcleHandler) {
            MotcleData.Builder motcleDataBuilder = ((MotcleHandler) elementHandler).getMotcleDataBuilder();
            thesaurusDataBuilder.addMotcleDataBuilder(motcleDataBuilder);
        }
    }


    private class MetadataHandler extends DefaultElementHandler {

        @Override
        public XmlPullElementHandler startSubElement(String tagname, Map<String, String> attributeMap) {
            if (tagname.equals("intitule-thesaurus")) {
                return new IntituleHandler();
            } else {
                return null;
            }
        }

    }


    private class IntituleHandler extends DefaultElementHandler {

        IntituleHandler() {
        }

        @Override
        public XmlPullElementHandler startSubElement(String tagname, Map<String, String> attributeMap) {
            if (tagname.equals("lib")) {
                try {
                    return new LibelleElementHandler(attributeMap);
                } catch (LangIntegerException lie) {
                    parameters.getScrutariXmlErrorHandler().addWarningLog(WARNING_XML_THESAURUS_LANGERROR, thesaurusURI);
                    return null;
                }
            }
            return null;
        }

        @Override
        public void endSubElement(XmlPullElementHandler elementHandler) {
            Libelle libelle = XmlPullUtils.testLibelle(elementHandler);
            if (libelle != null) {
                thesaurusDataBuilder.addLibelle(libelle);
            }
        }

    }


    private class MotcleHandler extends DefaultElementHandler {

        private MotcleURI motcleURI;
        private MotcleData.Builder motcleDataBuilder;

        MotcleHandler() {
        }

        public void setMotcleURI(MotcleURI motcleURI) {
            this.motcleURI = motcleURI;
            motcleDataBuilder = new MotcleData.Builder(motcleURI);
        }

        public MotcleData.Builder getMotcleDataBuilder() {
            return motcleDataBuilder;
        }

        @Override
        public XmlPullElementHandler startSubElement(String tagname, Map<String, String> attributeMap) {
            if (tagname.equals("lib")) {
                try {
                    return new LibelleElementHandler(attributeMap);
                } catch (LangIntegerException lie) {
                    parameters.getScrutariXmlErrorHandler().addWarningLog(WARNING_XML_MOTCLE_LANGERROR, motcleURI);
                    return null;
                }
            }
            return null;
        }

        @Override
        public void endSubElement(XmlPullElementHandler elementHandler) {
            Libelle libelle = XmlPullUtils.testLibelle(elementHandler);
            if (libelle != null) {
                motcleDataBuilder.addLibelle(libelle);
            }
        }

    }

}
