/* SctServer - Copyright (c) 2012 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.conf;

import fr.exemole.sctserver.ErrorUtils;
import fr.exemole.sctserver.SctServer;
import fr.exemole.sctserver.conf.xml.AttributesDOMReader;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import net.mapeadores.scrutari.db.AttributeKeyCoder;
import net.mapeadores.util.attr.AttributeDef;
import net.mapeadores.util.attr.AttributeDefList;
import net.mapeadores.util.attr.AttributeKey;
import net.mapeadores.util.attr.AttributeUtils;
import net.mapeadores.util.logging.ProcessLogBuilder;
import net.mapeadores.util.logging.ProcessMessageHandler;
import net.mapeadores.util.logging.ProcessMessageList;
import net.mapeadores.util.logging.ProcessURI;
import org.w3c.dom.Document;
import org.xml.sax.SAXException;


/**
 *
 * @author Vincent Calame
 */
public class AttributesConf {

    private ProcessURI processURI;
    private ProcessMessageList processMessageList;
    private Map<String, AttributeDefList> attributeDefListMap;

    private AttributesConf() {
    }

    public Map<String, AttributeDefList> getAttributeDefListMap() {
        return attributeDefListMap;
    }

    public ProcessURI getProcessURI() {
        return processURI;
    }

    public ProcessMessageList getProcessMessageList() {
        return processMessageList;
    }

    public static AttributesConf load(File f) throws IOException, ParserConfigurationException {
        AttributesConf conf = new AttributesConf();
        conf.init(f);
        return conf;
    }

    private void init(File f) throws IOException, ParserConfigurationException {
        processURI = ProcessURI.fromFile(f);
        ProcessLogBuilder logBuilder = new ProcessLogBuilder();
        DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
        Document document;
        try {
            DocumentBuilder db = dbf.newDocumentBuilder();
            document = db.parse(f);
        } catch (SAXException saxe) {
            ErrorUtils.saxException(logBuilder, saxe);
            processMessageList = logBuilder.toList();
            return;
        }
        AttributesBuilder builder = new AttributesBuilder(logBuilder);
        AttributesDOMReader domReader = new AttributesDOMReader(builder, logBuilder);
        domReader.readAttributes(document.getDocumentElement());
        attributeDefListMap = builder.toMap();
        processMessageList = logBuilder.toList();
    }


    private static class AttributesBuilder implements AttributesDOMReader.Builder {

        private Set<AttributeKey> keySet = new HashSet<AttributeKey>();
        private ProcessMessageHandler processMessageHandler;
        private List<AttributeDef> primaryAttributeDefList = new ArrayList<AttributeDef>();
        private List<AttributeDef> secondaryAttributeDefList = new ArrayList<AttributeDef>();
        private List<AttributeDef> currentAttributeDefList;

        private AttributesBuilder(ProcessMessageHandler processMessageHandler) {
            this.processMessageHandler = processMessageHandler;
        }

        public boolean startGroup(String name) {
            if (name.equals(AttributeKeyCoder.PRIMARY_GROUP)) {
                currentAttributeDefList = primaryAttributeDefList;
                return true;
            }
            if (name.equals(AttributeKeyCoder.SECONDARY_GROUP)) {
                currentAttributeDefList = secondaryAttributeDefList;
                return true;
            }
            return false;
        }

        public void addAttributeDef(AttributeDef attributeDef) {
            AttributeKey attributeKey = attributeDef.getAttributeKey();
            if (keySet.contains(attributeKey)) {
                processMessageHandler.addMessage(SctServer.WARNING_CATEGORY, "attrDefDoublon", attributeKey.toString());
                return;
            }
            if (currentAttributeDefList == null) {
                throw new IllegalStateException("startGroup must be successfully invoked first");
            }
            currentAttributeDefList.add(attributeDef);
            keySet.add(attributeKey);
        }

        private Map<String, AttributeDefList> toMap() {
            Map<String, AttributeDefList> map = new HashMap<String, AttributeDefList>();
            map.put(AttributeKeyCoder.PRIMARY_GROUP, AttributeUtils.toAttributeDefList(primaryAttributeDefList));
            map.put(AttributeKeyCoder.SECONDARY_GROUP, AttributeUtils.toAttributeDefList(secondaryAttributeDefList));
            return map;
        }

    }

}
