/* SctServer - Copyright (c) 2009-2012 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.conf;

import fr.exemole.sctserver.ErrorUtils;
import fr.exemole.sctserver.admin.SearchLogDeny;
import fr.exemole.sctserver.conf.xml.SearchLogFilterDOMReader;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import net.mapeadores.util.conditions.TextTest;
import net.mapeadores.util.logging.ProcessLogBuilder;
import net.mapeadores.util.logging.ProcessMessageList;
import net.mapeadores.util.logging.ProcessURI;
import org.w3c.dom.Document;
import org.xml.sax.SAXException;


/**
 *
 * @author Vincent Calame
 */
public class SearchLogFilterConf {

    private SearchLogDeny[] searchLogDenyArray;
    private ProcessURI processURI;
    private ProcessMessageList processMessageList;

    private SearchLogFilterConf() {
    }

    public SearchLogDeny[] getSearchLogDenyArray() {
        return searchLogDenyArray;
    }

    public static SearchLogFilterConf load(File f) throws IOException, ParserConfigurationException {
        SearchLogFilterConf conf = new SearchLogFilterConf();
        conf.init(f);
        return conf;
    }

    public ProcessURI getProcessURI() {
        return processURI;
    }

    public ProcessMessageList getProcessMessageList() {
        return processMessageList;
    }

    private void init(File f) throws IOException, ParserConfigurationException {
        searchLogDenyArray = null;
        processURI = ProcessURI.fromFile(f);
        ProcessLogBuilder logBuilder = new ProcessLogBuilder();
        DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
        Document document = null;
        try {
            DocumentBuilder db = dbf.newDocumentBuilder();
            document = db.parse(f);
        } catch (SAXException saxe) {
            ErrorUtils.saxException(logBuilder, saxe);
            processMessageList = logBuilder.toList();
            return;
        }
        SearchLogFilterBuilder builder = new SearchLogFilterBuilder();
        SearchLogFilterDOMReader domReader = new SearchLogFilterDOMReader(builder, logBuilder);
        domReader.readSearchLogFilter(document.getDocumentElement());
        searchLogDenyArray = builder.toSearchLogDenyArray();
        processMessageList = logBuilder.toList();
    }


    private static class SearchLogFilterBuilder implements SearchLogFilterDOMReader.Builder {

        List<SearchLogDeny> denyList = new ArrayList<SearchLogDeny>();

        public void addDeny(TextTest ipTest, TextTest siteTest, TextTest refTest) {
            denyList.add(new InternalSearchLogDeny(ipTest, siteTest, refTest));
        }

        public SearchLogDeny[] toSearchLogDenyArray() {
            int size = denyList.size();
            if (size == 0) {
                return null;
            }
            SearchLogDeny[] sld = new SearchLogDeny[size];
            for (int i = 0; i < size; i++) {
                sld[i] = denyList.get(i);
            }
            return sld;
        }

    }


    private static class InternalSearchLogDeny implements SearchLogDeny {

        private TextTest ipTest;
        private TextTest siteTest;
        private TextTest refTest;

        InternalSearchLogDeny(TextTest ipTest, TextTest siteTest, TextTest refTest) {
            this.ipTest = ipTest;
            this.siteTest = siteTest;
            this.refTest = refTest;
        }

        public TextTest getTextTest(short type) {
            switch (type) {
                case SearchLogDeny.IP_DENY:
                    return ipTest;
                case SearchLogDeny.SITE_DENY:
                    return siteTest;
                case SearchLogDeny.REF_DENY:
                    return refTest;
                default:
                    throw new IllegalArgumentException("Wrong type : " + type);
            }
        }

    }

}
