/* SctServer - Copyright (c) 2009-2012 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.conf;

import fr.exemole.sctserver.ErrorUtils;
import fr.exemole.sctserver.conf.xml.SourcesDOMReader;
import java.io.File;
import java.io.IOException;
import java.util.LinkedHashSet;
import java.util.Set;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import net.mapeadores.util.logging.ProcessLogBuilder;
import net.mapeadores.util.logging.ProcessMessageList;
import net.mapeadores.util.logging.ProcessURI;
import net.mapeadores.util.text.UrlKey;
import org.w3c.dom.Document;
import org.xml.sax.SAXException;


/**
 *
 * @author Vincent Calame
 */
public class SourcesConf {

    private UrlKey[] infoUrlKeyArray = null;
    private ProcessURI processURI;
    private ProcessMessageList processMessageList;

    public SourcesConf() {
    }

    public ProcessURI getProcessURI() {
        return processURI;
    }

    public ProcessMessageList getProcessMessageList() {
        return processMessageList;
    }

    public UrlKey[] getInfoUrlKeyArray() {
        return infoUrlKeyArray;
    }

    public static SourcesConf load(File f) throws IOException, ParserConfigurationException {
        SourcesConf conf = new SourcesConf();
        conf.init(f);
        return conf;
    }

    private void init(File f) throws IOException, ParserConfigurationException {
        infoUrlKeyArray = null;
        processURI = ProcessURI.fromFile(f);
        ProcessLogBuilder logBuilder = new ProcessLogBuilder();
        DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
        Document document;
        try {
            DocumentBuilder db = dbf.newDocumentBuilder();
            document = db.parse(f);
        } catch (SAXException saxe) {
            ErrorUtils.saxException(logBuilder, saxe);
            processMessageList = logBuilder.toList();
            return;
        }
        SourcesBuilder builder = new SourcesBuilder();
        SourcesDOMReader domReader = new SourcesDOMReader(builder, logBuilder);
        domReader.readSources(document.getDocumentElement());
        infoUrlKeyArray = builder.toInfoUrlKeyArray();
        processMessageList = logBuilder.toList();
    }


    private static class SourcesBuilder implements SourcesDOMReader.Builder {

        Set<UrlKey> set = new LinkedHashSet<UrlKey>();

        public boolean addInfoUrl(UrlKey urlKey) {
            if (set.contains(urlKey)) {
                return false;
            }
            set.add(urlKey);
            return true;
        }

        private UrlKey[] toInfoUrlKeyArray() {
            int size = set.size();
            if (size == 0) {
                return null;
            }
            UrlKey[] infoUrlKeyArray = new UrlKey[size];
            int p = 0;
            for (UrlKey urlKey : set) {
                infoUrlKeyArray[p] = urlKey;
                p++;
            }
            return infoUrlKeyArray;
        }

    }

}
