/* SctServer - Copyright (c) 2011-2012 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.conf;

import fr.exemole.sctserver.ErrorUtils;
import fr.exemole.sctserver.SctServer;
import fr.exemole.sctserver.conf.xml.UriCodesDOMReader;
import java.io.File;
import java.io.IOException;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.Map;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import net.mapeadores.scrutari.db.ScrutariDataURICodeList;
import net.mapeadores.util.logging.ProcessLogBuilder;
import net.mapeadores.util.logging.ProcessMessageHandler;
import net.mapeadores.util.logging.ProcessMessageList;
import net.mapeadores.util.logging.ProcessURI;
import net.scrutari.datauri.ScrutariDataURI;
import org.w3c.dom.Document;
import org.xml.sax.SAXException;


/**
 *
 * @author Vincent Calame
 */
public class UriCodesConf {

    private ScrutariDataURICodeList scrutariDataURICodeList;
    private ProcessURI processURI;
    private ProcessMessageList processMessageList;

    private UriCodesConf() {
    }

    public ScrutariDataURICodeList getScrutariDataURICodeList() {
        return scrutariDataURICodeList;
    }

    public static UriCodesConf load(File f) throws IOException, ParserConfigurationException {
        UriCodesConf conf = new UriCodesConf();
        conf.init(f);
        return conf;
    }

    public ProcessURI getProcessURI() {
        return processURI;
    }

    public ProcessMessageList getProcessMessageList() {
        return processMessageList;
    }

    private void init(File f) throws IOException, ParserConfigurationException {
        processURI = ProcessURI.fromFile(f);
        ProcessLogBuilder logBuilder = new ProcessLogBuilder();
        DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
        Document document;
        try {
            DocumentBuilder db = dbf.newDocumentBuilder();
            document = db.parse(f);
        } catch (SAXException saxe) {
            ErrorUtils.saxException(logBuilder, saxe);
            processMessageList = logBuilder.toList();
            return;
        }
        UriCodesBuilder builder = new UriCodesBuilder(logBuilder);
        UriCodesDOMReader domReader = new UriCodesDOMReader(builder, logBuilder);
        domReader.readUriCodes(document.getDocumentElement());
        scrutariDataURICodeList = builder.toScrutariDataURICodeList();
        processMessageList = logBuilder.toList();
    }


    private static class UriCodesBuilder implements UriCodesDOMReader.Builder {

        private ProcessMessageHandler errorHandler;
        private Map<Integer, ScrutariDataURI> codeMap = new LinkedHashMap<Integer, ScrutariDataURI>();
        private Map<ScrutariDataURI, Integer> uriMap = new HashMap<ScrutariDataURI, Integer>();

        public UriCodesBuilder(ProcessMessageHandler uriCodesLog) {
            this.errorHandler = uriCodesLog;
        }

        public void addScrutariDataURI(int code, ScrutariDataURI scrutariDataURI) {
            if (codeMap.containsKey(code)) {
                errorHandler.addMessage(SctServer.WARNING_CATEGORY, "codeDoublon", new Integer(code));
                return;
            }
            if (uriMap.containsKey(scrutariDataURI)) {
                errorHandler.addMessage(SctServer.WARNING_CATEGORY, "uriDoublon", scrutariDataURI);
                return;
            }
            codeMap.put(code, scrutariDataURI);
            uriMap.put(scrutariDataURI, code);
        }

        public ScrutariDataURICodeList toScrutariDataURICodeList() {
            int size = codeMap.size();
            if (size == 0) {
                return null;
            }
            int[] codeArray = new int[size];
            ScrutariDataURI[] uriArray = new ScrutariDataURI[size];
            int p = 0;
            for (Map.Entry<Integer, ScrutariDataURI> entry : codeMap.entrySet()) {
                codeArray[p] = entry.getKey();
                uriArray[p] = entry.getValue();
                p++;
            }
            return new ArrayScrutariDataURICodeList(codeArray, uriArray);
        }

    }


    private static class ArrayScrutariDataURICodeList implements ScrutariDataURICodeList {

        private int[] codeArray;
        private ScrutariDataURI[] uriArray;

        public ArrayScrutariDataURICodeList(int[] codeArray, ScrutariDataURI[] uriArray) {
            this.codeArray = codeArray;
            this.uriArray = uriArray;
        }

        public int getCodeCount() {
            return codeArray.length;
        }

        public int getCode(int i) {
            return codeArray[i];
        }

        public ScrutariDataURI getScrutariDataURI(int i) {
            return uriArray[i];
        }

    }

}
