/* SctServer - Copyright (c) 2011-2012 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.request;

import net.mapeadores.util.text.StringUtils;


/**
 *
 * @author Vincent Calame
 */
public class FicheFieldsFilter {

    public final static short NONE_FILTER = 0;
    public final static short CONCERNED_FILTER = 1;
    public final static short ALL_FILTER = 2;
    public final static short PRIMARY_FILTER = 3;
    private boolean withCodeCorpus = false;
    private boolean withCodeBase = false;
    private boolean withFicheId = false;
    private boolean withTitre = false;
    private boolean withSoustitre = false;
    private boolean withHref = false;
    private boolean withLang = false;
    private boolean withAnnee = false;
    private boolean withFicheIcon = false;
    private boolean withGeo = false;
    private boolean withMTitre = false;
    private boolean withMSoustitre = false;
    private boolean withComplements = false;
    private boolean withMComplements = false;
    private boolean allMComplements = false;
    private boolean withPertinence = false;
    private boolean withCorpusName = false;
    private boolean withBaseName = false;
    private boolean withIcon = false;
    private short mAttributesFilter = NONE_FILTER;

    public FicheFieldsFilter() {
    }

    public boolean isWithCodeCorpus() {
        return withCodeCorpus;
    }

    public void setWithCodeCorpus(boolean withCodeCorpus) {
        this.withCodeCorpus = withCodeCorpus;
    }

    public boolean isWithCodeBase() {
        return withCodeBase;
    }

    public void setWithCodeBase(boolean withCodeBase) {
        this.withCodeBase = withCodeBase;
    }

    public boolean isWithTitre() {
        return withTitre;
    }

    public void setWithTitre(boolean withTitre) {
        this.withTitre = withTitre;
    }

    public boolean isWithFicheId() {
        return withFicheId;
    }

    public void setWithFicheId(boolean withFicheId) {
        this.withFicheId = withFicheId;
    }

    public boolean isWithSoustitre() {
        return withSoustitre;
    }

    public void setWithSoustitre(boolean withSoustitre) {
        this.withSoustitre = withSoustitre;
    }

    public boolean isWithHref() {
        return withHref;
    }

    public void setWithHref(boolean withUrl) {
        this.withHref = withUrl;
    }

    public boolean isWithLang() {
        return withLang;
    }

    public void setWithLang(boolean withLang) {
        this.withLang = withLang;
    }

    public boolean isWithAnnee() {
        return withAnnee;
    }

    public void setWithAnnee(boolean withAnnee) {
        this.withAnnee = withAnnee;
    }

    public boolean isWithFicheIcon() {
        return withFicheIcon;
    }

    public void setWithFicheIcon(boolean withFicheIcon) {
        this.withFicheIcon = withFicheIcon;
    }

    public boolean isWithGeo() {
        return withGeo;
    }

    public void setWithGeo(boolean withGeo) {
        this.withGeo = withGeo;
    }

    public boolean isWithMTitre() {
        return withMTitre;
    }

    public void setWithMTitre(boolean withMTitre) {
        this.withMTitre = withMTitre;
    }

    public boolean isWithMSoustitre() {
        return withMSoustitre;
    }

    public void setWithMSoustitre(boolean withMSoustitre) {
        this.withMSoustitre = withMSoustitre;
    }

    public boolean isWithComplements() {
        return withComplements;
    }

    public void setWithComplements(boolean withComplements) {
        this.withComplements = withComplements;
    }

    public boolean isWithMComplements() {
        return withMComplements;
    }

    public void setWithMComplements(boolean withMComplements) {
        this.withMComplements = withMComplements;
    }

    public boolean isAllMComplements() {
        return allMComplements;
    }

    public void setAllMComplements(boolean withComplementsDefault) {
        this.allMComplements = withComplementsDefault;
    }

    public short getMAttributesFilter() {
        return mAttributesFilter;
    }

    public void setMAttributesFilter(short mattributesFilter) {
        this.mAttributesFilter = mattributesFilter;
    }

    public boolean isWithPertinence() {
        return withPertinence;
    }

    public void setWithPertinence(boolean withPertinence) {
        this.withPertinence = withPertinence;
    }

    public boolean isWithCorpusName() {
        return withCorpusName;
    }

    public void setWithCorpusName(boolean withCorpusName) {
        this.withCorpusName = withCorpusName;
    }

    public boolean isWithBaseName() {
        return withBaseName;
    }

    public void setWithBaseName(boolean withBaseName) {
        this.withBaseName = withBaseName;
    }

    public void buildFromString(String s) {
        String[] tokens = StringUtils.getCleanedTokens(s, ',');
        int count = tokens.length;
        for (int i = 0; i < count; i++) {
            String token = tokens[i];
            if (token.equals("codecorpus")) {
                setWithCodeCorpus(true);
            } else if (token.equals("codebase")) {
                setWithCodeBase(true);
            } else if (token.equals("ficheid")) {
                setWithFicheId(true);
            } else if (token.equals("titre")) {
                setWithTitre(true);
            } else if (token.equals("soustitre")) {
                setWithSoustitre(true);
            } else if (token.equals("href")) {
                setWithHref(true);
            } else if (token.equals("lang")) {
                setWithLang(true);
            } else if (token.equals("annee")) {
                setWithAnnee(true);
            } else if (token.equals("ficheicon")) {
                setWithFicheIcon(true);
            } else if (token.equals("geo")) {
                setWithGeo(true);
            } else if (token.equals("mtitre") || token.equals("mtitre_def")) {
                setWithMTitre(true);
            } else if (token.equals("msoustitre") || token.equals("msoustitre_def")) {
                setWithMSoustitre(true);
            } else if (token.equals("complements")) {
                setWithComplements(true);
            } else if (token.equals("mcomplements")) {
                setWithMComplements(true);
            } else if (token.equals("mcomplements_all")) {
                setWithMComplements(true);
                setAllMComplements(true);
            } else if (token.equals("pertinence")) {
                setWithPertinence(true);
            } else if (token.equals("corpusname")) {
                setWithCorpusName(true);
            } else if (token.equals("basename")) {
                setWithBaseName(true);
            } else if (token.equals("icon")) {
                setWithIcon(true);
            } else if (token.equals("mattrs")) {
                setMAttributesFilter(CONCERNED_FILTER);
            } else if (token.equals("mattrs_all")) {
                setMAttributesFilter(ALL_FILTER);
            } else if (token.equals("mattrs_primary")) {
                setMAttributesFilter(PRIMARY_FILTER);
            }
        }
    }

    public boolean isWithIcon() {
        return withIcon;
    }

    public void setWithIcon(boolean withIcon) {
        this.withIcon = withIcon;
    }

}
