/* SctServer - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.request.json;

import java.io.IOException;
import net.mapeadores.util.exceptions.ShouldNotOccurException;
import net.mapeadores.util.json.JSONException;
import net.mapeadores.util.json.JSONWriter;


/**
 *
 * @author Vincent Calame
 */
public class ErrorJsonProducer extends AbstractJsonProducer {

    private String errorKey;
    private String parameter;
    private String value;

    public ErrorJsonProducer(String errorKey, String parameter, String value) {
        this.errorKey = errorKey;
        this.parameter = parameter;
        this.value = value;
    }

    public void writeJson(Appendable appendable) throws IOException {
        JSONWriter jsonWriter = new JSONWriter(appendable);
        try {
            jsonWriter.object();
            jsonWriter.key("error");
            jsonWriter.object();
            jsonWriter.key("key");
            jsonWriter.value(errorKey);
            jsonWriter.key("parameter");
            jsonWriter.value(parameter);
            if (value != null) {
                jsonWriter.key("value");
                jsonWriter.value(value);
            }
            jsonWriter.endObject();
            writeWarnings(jsonWriter);
            jsonWriter.endObject();
        } catch (JSONException je) {
            Throwable throwable = je.getCause();
            if (throwable == null) {
                throw new ShouldNotOccurException(je.getMessage());
            } else {
                if (throwable instanceof IOException) {
                    throw ((IOException) throwable);
                } else {
                    throw new ShouldNotOccurException(je.getMessage(), (Exception) throwable);
                }
            }
        }
    }

}
