/* SctServer - Copyright (c) 2010-2012 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.request.json;

import fr.exemole.sctserver.request.FicheFieldsFilter;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;
import net.mapeadores.scrutari.data.BaseData;
import net.mapeadores.scrutari.data.FicheData;
import net.mapeadores.scrutari.db.CodeList;
import net.mapeadores.scrutari.db.DataAccess;
import net.mapeadores.scrutari.db.ScrutariDB;
import net.mapeadores.scrutari.db.util.BaseCheck;
import net.mapeadores.util.exceptions.ShouldNotOccurException;
import net.mapeadores.util.json.JSONException;
import net.mapeadores.util.json.JSONWriter;


/**
 *
 * @author Vincent Calame
 */
public class FicheDataJsonProducer extends AbstractJsonProducer {

    private ScrutariDB scrutariDB;
    private CodeList codeficheList;
    private FicheFieldsFilter ficheFieldsFilter;
    private int langUi;

    public FicheDataJsonProducer(ScrutariDB scrutariDB, CodeList codeficheList, FicheFieldsFilter ficheFieldsFilter, int langUi) {
        this.scrutariDB = scrutariDB;
        this.codeficheList = codeficheList;
        this.ficheFieldsFilter = ficheFieldsFilter;
        this.langUi = langUi;
    }

    public void writeJson(Appendable appendable) throws IOException {
        DataAccess dataAccess = scrutariDB.openDataAccess();
        Map<Integer, BaseCheck> bufferMap = new HashMap<Integer, BaseCheck>();
        JSONWriter jsonWriter = new JSONWriter(appendable);
        try {
            jsonWriter.object();
            jsonWriter.key("ficheData");
            jsonWriter.object();
            int ficheCount = codeficheList.getCodeCount();
            jsonWriter.key("ficheCount");
            jsonWriter.value(ficheCount);
            if (ficheCount > 0) {
                jsonWriter.key("ficheArray");
                jsonWriter.array();
                for (int i = 0; i < ficheCount; i++) {
                    int codefiche = codeficheList.getCode(i);
                    FicheData ficheData = dataAccess.getFicheDataByCode(codefiche);
                    BaseCheck baseCheck = getBaseCheck(dataAccess, ficheData.getCodeBase(), bufferMap, langUi);
                    jsonWriter.object();
                    JsonUtils.addFicheData(jsonWriter, ficheData, ficheFieldsFilter, baseCheck, null, null);
                    jsonWriter.endObject();
                }
                jsonWriter.endArray();
            }
            jsonWriter.endObject();
            writeWarnings(jsonWriter);
            jsonWriter.endObject();
        } catch (JSONException je) {
            Throwable throwable = je.getCause();
            if (throwable == null) {
                throw new ShouldNotOccurException(je.getMessage(), je);
            } else {
                if (throwable instanceof IOException) {
                    throw ((IOException) throwable);
                } else {
                    throw new ShouldNotOccurException(je.getMessage(), (Exception) throwable);
                }
            }
        }
        dataAccess.close();
    }

    private BaseCheck getBaseCheck(DataAccess dataAccess, int codebase, Map<Integer, BaseCheck> bufferMap, int langUi) {
        BaseCheck baseCheck = bufferMap.get(codebase);
        if (baseCheck != null) {
            return baseCheck;
        }
        BaseData baseData = dataAccess.getBaseDataByCode(codebase);
        baseCheck = BaseCheck.check(baseData, langUi);
        bufferMap.put(codebase, baseCheck);
        return baseCheck;
    }

}
