/* SctServer - Copyright (c) 2012 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.request.json;

import fr.exemole.sctserver.request.MotcleFieldsFilter;
import java.io.IOException;
import net.mapeadores.scrutari.data.MotcleData;
import net.mapeadores.scrutari.db.CodeList;
import net.mapeadores.scrutari.db.DataAccess;
import net.mapeadores.scrutari.db.ScrutariDB;
import net.mapeadores.util.exceptions.ShouldNotOccurException;
import net.mapeadores.util.json.JSONException;
import net.mapeadores.util.json.JSONWriter;


/**
 *
 * @author Vincent Calame
 */
public class MotcleDataJsonProducer extends AbstractJsonProducer {

    private ScrutariDB scrutariDB;
    private CodeList codemotcleList;
    private MotcleFieldsFilter motcleFieldsFilter;

    public MotcleDataJsonProducer(ScrutariDB scrutariDB, CodeList codemotcleList, MotcleFieldsFilter motcleFieldsFilter) {
        this.scrutariDB = scrutariDB;
        this.codemotcleList = codemotcleList;
        this.motcleFieldsFilter = motcleFieldsFilter;
    }

    public void writeJson(Appendable appendable) throws IOException {
        DataAccess dataAccess = scrutariDB.openDataAccess();
        JSONWriter jsonWriter = new JSONWriter(appendable);
        try {
            jsonWriter.object();
            jsonWriter.key("motcleData");
            jsonWriter.object();
            int motcleCount = codemotcleList.getCodeCount();
            jsonWriter.key("motcleCount");
            jsonWriter.value(motcleCount);
            if (motcleCount > 0) {
                jsonWriter.key("motcleArray");
                jsonWriter.array();
                for (int i = 0; i < motcleCount; i++) {
                    int codemotcle = codemotcleList.getCode(i);
                    MotcleData motcleData = dataAccess.getMotcleDataByCode(codemotcle);
                    jsonWriter.object();
                    JsonUtils.addMotcleData(jsonWriter, motcleData, motcleFieldsFilter, null);
                    jsonWriter.endObject();
                }
                jsonWriter.endArray();
            }
            jsonWriter.endObject();
            writeWarnings(jsonWriter);
            jsonWriter.endObject();
        } catch (JSONException je) {
            Throwable throwable = je.getCause();
            if (throwable == null) {
                throw new ShouldNotOccurException(je.getMessage(), je);
            } else {
                if (throwable instanceof IOException) {
                    throw ((IOException) throwable);
                } else {
                    throw new ShouldNotOccurException(je.getMessage(), (Exception) throwable);
                }
            }
        }
        dataAccess.close();
    }

}
