/* SctServer - Copyright (c) 2010-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.request.json;

import net.mapeadores.scrutari.searchengine.SupermotcleSearchResult;
import net.mapeadores.scrutari.searchengine.SupermotcleSearchResultInfo;
import net.mapeadores.scrutari.supermotcle.Supermotcle;
import net.mapeadores.scrutari.supermotcle.SupermotcleIndexation;
import fr.exemole.sctserver.SctServer;
import net.mapeadores.util.json.JSONException;
import net.mapeadores.util.json.JSONWriter;
import net.mapeadores.util.text.collation.CollationUnit;
import net.mapeadores.util.exceptions.ShouldNotOccurException;
import java.io.IOException;


/**
 *
 * @author Vincent Calame
 */
public class SupermotcleSearchResultJsonProducer extends AbstractJsonProducer {

    private SupermotcleSearchResult supermotcleSearchResult;
    private int langInteger;
    private int limit;
    private SctServer sctServer;

    public SupermotcleSearchResultJsonProducer(SupermotcleSearchResult supermotcleSearchResult, SctServer sctServer, int langInteger, int limit) {
        this.supermotcleSearchResult = supermotcleSearchResult;
        this.langInteger = langInteger;
        this.sctServer = sctServer;
        this.limit = limit;
    }

    public void writeJson(Appendable appendable) throws IOException {
        JSONWriter jsonWriter = new JSONWriter(appendable);
        try {
            jsonWriter.object();
            jsonWriter.key("supermotcleSearchResult");
            jsonWriter.object();
            int count = supermotcleSearchResult.getSupermotcleSearchResultInfoCount();
            if (limit != -1) {
                count = Math.min(count, limit);
            }
            jsonWriter.key("supermotcleCount");
            jsonWriter.value(count);
            if (count > 0) {
                jsonWriter.key("supermotcleArray");
                jsonWriter.array();
                for (int i = 0; i < count; i++) {
                    SupermotcleSearchResultInfo info = supermotcleSearchResult.getSupermotcleSearchResultInfo(i);
                    Supermotcle supermotcle = info.getSupermotcle();
                    CollationUnit collationUnit = supermotcle.getCollationUnit();
                    String lib = collationUnit.getSourceString();
                    jsonWriter.object();
                    jsonWriter.key("codesupermotcle");
                    jsonWriter.value(supermotcle.getCode());
                    jsonWriter.key("lib");
                    jsonWriter.value(lib);
                    jsonWriter.key("mlib");
                    JsonUtils.writeMLib(jsonWriter, lib, info.getSearchTokentPositionList());
                    int baseCount = supermotcle.getSupermotcleIndexationCount();
                    jsonWriter.key("indexationCount");
                    jsonWriter.value(supermotcle.getIndexationCount());
                    jsonWriter.key("baseArray");
                    jsonWriter.array();
                    for (int j = 0; j < baseCount; j++) {
                        SupermotcleIndexation supermotcleIndexation = supermotcle.getSupermotcleIndexation(j);
                        int codebase = supermotcleIndexation.getCodeBase();
                        jsonWriter.object();
                        jsonWriter.key("codebase");
                        jsonWriter.value(codebase);
                        jsonWriter.key("indexationCount");
                        jsonWriter.value(supermotcleIndexation.getIndexationCount());
                        jsonWriter.endObject();

                    }
                    jsonWriter.endArray();
                    jsonWriter.endObject();
                }
                jsonWriter.endArray();
            }
            jsonWriter.endObject();
            writeWarnings(jsonWriter);
            jsonWriter.endObject();
        } catch (JSONException je) {
            Throwable throwable = je.getCause();
            if (throwable == null) {
                throw new ShouldNotOccurException(je.getMessage(), je);
            } else {
                if (throwable instanceof IOException) {
                    throw ((IOException) throwable);
                } else {
                    throw new ShouldNotOccurException(je.getMessage(), (Exception) throwable);
                }
            }
        }
    }

}
