/* SctServer - Copyright (c) 2008-2012 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.request.simple.catalog;

import net.scrutari.datauri.CorpusURI;
import net.mapeadores.scrutari.data.BaseData;
import net.mapeadores.scrutari.data.CorpusData;
import net.mapeadores.scrutari.db.DataAccess;
import net.mapeadores.scrutari.db.ScrutariDB;
import net.mapeadores.scrutari.db.util.LibelleUtils;
import fr.exemole.sctserver.request.simple.SimpleRequest;
import java.io.IOException;
import java.io.PrintWriter;


/**
 *
 * @author Vincent Calame
 */
public class CorpusDataRequest extends SimpleRequest {

    private int langUi;
    private CorpusData corpusData;

    public CorpusDataRequest(ScrutariDB scrutariDB, int langUi) {
        super(scrutariDB);
        this.langUi = langUi;
    }

    public CorpusDataRequest(ScrutariDB scrutariDB, int langUi, CorpusData corpusData) {
        super(scrutariDB);
        this.langUi = langUi;
        this.corpusData = corpusData;
    }

    public void printResponse(PrintWriter pw) throws IOException {
        DataAccess dataAccess = scrutariDB.openDataAccess();
        if (corpusData != null) {
            printCorpus(pw, corpusData, dataAccess);
        } else {
            for (int i = 0; i < dataAccess.getCorpusCount(); i++) {
                CorpusData currentCorpusData = dataAccess.getCorpusDataByIndex(i);
                printCorpus(pw, currentCorpusData, dataAccess);
            }
        }
        dataAccess.close();
    }

    void printCorpus(PrintWriter pw, CorpusData currentCorpusData, DataAccess dataAccess) {
        BaseData currentBaseData = dataAccess.getBaseDataByCode(currentCorpusData.getCodeBase());
        int checkedLangInteger = currentBaseData.checkLangUi(langUi);
        int codecrp = currentCorpusData.getCodeCorpus();
        int totalcomplements = currentCorpusData.getComplementMaxNumber();
        CorpusURI corpusURI = currentCorpusData.getCorpusURI();
        String name = corpusURI.getCorpusName();
        pw.print("#c");
        pw.print('\n');
        pw.print("cdcrp=");
        pw.print(codecrp);
        pw.print('\n');
        pw.print("cdbdf=");
        pw.print(currentCorpusData.getCodeBase());
        pw.print('\n');
        pw.print("nmcrp=");
        pw.print(corpusURI.getCorpusName());
        pw.print('\n');
        pw.print("ttcmp=");
        pw.print(totalcomplements);
        pw.print('\n');
        pw.print("incrp=");
        pw.print(LibelleUtils.getLibelleString(currentCorpusData, CorpusData.INTITULECORPUS_FIELDNUMBER, checkedLangInteger, name));
        pw.print('\n');
        pw.print("infch=");
        pw.print(LibelleUtils.getLibelleString(currentCorpusData, CorpusData.INTITULEFICHE_FIELDNUMBER, checkedLangInteger, name));
        pw.print('\n');
        for (int i = 1; i <= totalcomplements; i++) {
            pw.print("inc");
            if (i < 10) {
                pw.print('0');
            }
            pw.print(i);
            pw.print('=');
            pw.print(LibelleUtils.getLibelleString(currentCorpusData, i, checkedLangInteger, "c" + i));
            pw.print('\n');
        }
    }

}
