/* SctServer - Copyright (c) 2008-2012 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.request.simple.catalog;

import fr.exemole.sctserver.request.simple.SimpleRequest;
import java.io.IOException;
import java.io.PrintWriter;
import net.mapeadores.scrutari.data.BaseData;
import net.mapeadores.scrutari.data.FicheData;
import net.mapeadores.scrutari.db.DataAccess;
import net.mapeadores.scrutari.db.ScrutariDB;
import net.mapeadores.scrutari.db.util.DataUtils;
import net.mapeadores.scrutari.lexie.AlineaInteger;
import net.mapeadores.scrutari.searchengine.FicheSearchResult;
import net.mapeadores.scrutari.searchengine.FicheSearchResultGroup;
import net.mapeadores.scrutari.searchengine.FicheSearchResultInfo;
import net.mapeadores.scrutari.searchengine.GlobalSearchOptions;
import net.mapeadores.scrutari.searchengine.SearchTokenPositionList;
import net.mapeadores.util.primitives.DateInteger;
import net.mapeadores.util.text.SubstringPosition;
import net.scrutari.datauri.FicheURI;


/**
 *
 * @author Vincent Calame
 */
public class FicheResultRequest extends SimpleRequest {

    private int langUi;
    private FicheSearchResult searchResult;
    private boolean decalageutf8;
    private int plageStart = 0;
    private int plageLength = -1;
    private GlobalSearchOptions globalSearchOptions;

    public FicheResultRequest(ScrutariDB scrutariDB, FicheSearchResult searchResult) {
        super(scrutariDB);
        this.langUi = searchResult.getLangUi();
        this.searchResult = searchResult;
    }

    public FicheResultRequest(ScrutariDB scrutariDB, FicheSearchResult searchResult, int plageStart, int plageLength) {
        this(scrutariDB, searchResult);
        this.plageStart = plageStart;
        this.plageLength = plageLength;
    }

    public void setDecalageUTF8(boolean decalageutf8) {
        this.decalageutf8 = decalageutf8;
    }

    public void setGlobalSearchOptions(GlobalSearchOptions globalSearchOptions) {
        this.globalSearchOptions = globalSearchOptions;
    }

    public void printResponse(PrintWriter pw) throws IOException {
        DataAccess dataAccess = scrutariDB.openDataAccess();
        int groupCount = searchResult.getFicheSearchResultGroupCount();
        int previousGroupCount = 0;
        if (plageLength == -1) {
            boolean suite = false;
            for (int i = 0; i < groupCount; i++) {
                FicheSearchResultGroup ficheSearchResultGroup = searchResult.getFicheSearchResultGroup(i);
                int infoCount = ficheSearchResultGroup.getFicheSearchResultCount();
                if (suite) {
                    for (int j = 0; j < infoCount; j++) {
                        printFicheResult(pw, ficheSearchResultGroup.getFicheSearchResultInfo(j), dataAccess);
                    }
                    continue;
                }
                if ((previousGroupCount + groupCount) < plageStart) {
                    previousGroupCount = previousGroupCount + groupCount;
                    continue;
                }
                int debut = plageStart - previousGroupCount;
                for (int j = debut; j < infoCount; j++) {
                    printFicheResult(pw, ficheSearchResultGroup.getFicheSearchResultInfo(j), dataAccess);
                }
                suite = true;
            }
        } else {
            boolean suite = false;
            int total = 0;
            for (int i = 0; i < groupCount; i++) {
                FicheSearchResultGroup ficheSearchResultGroup = searchResult.getFicheSearchResultGroup(i);
                int infoCount = ficheSearchResultGroup.getFicheSearchResultCount();
                if (suite) {
                    int min = Math.min(plageLength - total, infoCount);
                    for (int j = 0; j < min; j++) {
                        printFicheResult(pw, ficheSearchResultGroup.getFicheSearchResultInfo(j), dataAccess);
                    }
                    total = total + min;
                    if (total == plageLength) {
                        break;
                    } else {
                        continue;
                    }
                }
                if ((previousGroupCount + infoCount) < plageStart) {
                    previousGroupCount = previousGroupCount + infoCount;
                    continue;
                }
                int debut = plageStart - previousGroupCount;
                int min = Math.min(plageLength, (infoCount - debut));
                int fin = debut + min;
                for (int j = debut; j < fin; j++) {
                    printFicheResult(pw, ficheSearchResultGroup.getFicheSearchResultInfo(j), dataAccess);
                }
                total = min;
                suite = true;
                if (total == plageLength) {
                    break;
                }
            }
        }
        dataAccess.close();
    }

    private void printFicheResult(PrintWriter pw, FicheSearchResultInfo currentFicheResult, DataAccess dataAccess) {
        int codefiche = currentFicheResult.getCode();
        FicheData ficheData = dataAccess.getFicheDataByCode(codefiche);
        int codecorpus = ficheData.getCodeCorpus();
        int codebase = ficheData.getCodeBase();
        BaseData baseInfo = dataAccess.getBaseDataByCode(codebase);
        pw.print("#f");
        pw.print('\n');
        pw.print("cdfch=");
        pw.print(codefiche);
        pw.print('\n');
        pw.print("cdcrp=");
        pw.print(codecorpus);
        pw.print('\n');
        pw.print("cdbdf=");
        pw.print(codebase);
        pw.print('\n');
        FicheURI ficheURI = ficheData.getFicheURI();
        pw.print("idcrp=");
        pw.print(ficheURI.getFicheId());
        pw.print('\n');
        pw.print("clcrp=");
        if (globalSearchOptions != null) {
            pw.print(globalSearchOptions.getCorpusClasseRank(codecorpus));
        } else {
            pw.print('0');
        }
        pw.print('\n');
        String ficheIcon = ficheData.getFicheIcon();
        pw.print("fcico=");
        if (ficheIcon != null) {
            pw.print(ficheIcon);
        }
        pw.print('\n');
        String titre = ficheData.getTitre();
        String soustitre = ficheData.getSoustitre();
        if (titre == null) {
            titre = "?";
        }
        if (soustitre == null) {
            soustitre = "";
        }
        if (decalageutf8) {
            int[] decalagetitre = new int[titre.length()];
            int[] decalagesoustitre = new int[soustitre.length()];
            int decalage = 0;
            pw.print("titre=");
            for (int i = 0; i < titre.length(); i++) {
                char c = titre.charAt(i);
                pw.print(c);
                decalage = decalage + DataUtils.getDecalageUTF8(c);
                decalagetitre[i] = decalage;
            }
            pw.print('\n');
            pw.print("ssttr=");
            decalage = 0;
            for (int i = 0; i < soustitre.length(); i++) {
                char c = soustitre.charAt(i);
                pw.print(c);
                decalage = decalage + DataUtils.getDecalageUTF8(c);
                decalagesoustitre[i] = decalage;
            }
            pw.print('\n');
            printPosition(pw, AlineaInteger.TITRE, currentFicheResult.getSubstringPositionList(AlineaInteger.TITRE), decalagetitre);
            printPosition(pw, AlineaInteger.SOUSTITRE, currentFicheResult.getSubstringPositionList(AlineaInteger.SOUSTITRE), decalagesoustitre);
            for (int p = 1; p <= ficheData.getComplementMaxNumber(); p++) {
                int alineaInteger = AlineaInteger.getComplementAlineaInteger(p);
                SearchTokenPositionList positionList = currentFicheResult.getSubstringPositionList(alineaInteger);
                if (positionList != null) {
                    pw.print("cmp");
                    if (p < 10) {
                        pw.print('0');
                    }
                    pw.print(p);
                    pw.print('=');
                    String strg_complement = ficheData.getComplementByNumber(p);
                    int[] decalagecomp = new int[strg_complement.length()];
                    decalage = 0;
                    for (int j = 0; j < strg_complement.length(); j++) {
                        char c = strg_complement.charAt(j);
                        pw.print(c);
                        decalage = decalage + DataUtils.getDecalageUTF8(c);
                        decalagecomp[j] = decalage;
                    }
                    pw.print('\n');
                    printPosition(pw, alineaInteger, positionList, decalagecomp);
                }
            }
        } else {
            pw.print("titre=");
            pw.print(titre);
            pw.print('\n');
            pw.print("ssttr=");
            pw.print(soustitre);
            pw.print('\n');
            for (int p = 1; p <= ficheData.getComplementMaxNumber(); p++) {
                SearchTokenPositionList positionList = currentFicheResult.getSubstringPositionList(AlineaInteger.getComplementAlineaInteger(p));
                if (positionList != null) {
                    pw.print("cmp");
                    if (p < 10) {
                        pw.print('0');
                    }
                    pw.print(p);
                    pw.print('=');
                    pw.print(ficheData.getComplementByNumber(p));
                    pw.print('\n');
                }
            }
        }
        pw.print("hreff=");
        int checkedLangInteger = baseInfo.checkLangUi(langUi);
        String href = ficheData.getHref(checkedLangInteger);
        if (href != null) {
            pw.print(href);
        }
        pw.print('\n');
        int dateInteger = ficheData.getDateInteger();
        if (dateInteger != DateInteger.NULL) {
            pw.print("datan=");
            pw.print(DateInteger.getAnnee(dateInteger));
            pw.print('\n');
            short typedate = DateInteger.getDateIntegerType(dateInteger);
            switch (typedate) {
                case DateInteger.TYPE_ANNEE:
                    pw.print("dattp=a\n");
                    break;
                case DateInteger.TYPE_SEMESTRE:
                    pw.print("dattp=s\n");
                    pw.print("datsm=");
                    pw.print(DateInteger.getSemestre(dateInteger));
                    pw.print('\n');
                    break;
                case DateInteger.TYPE_TRIMESTRE:
                    pw.print("dattp=t\n");
                    pw.print("dattm=");
                    pw.print(DateInteger.getTrimestre(dateInteger));
                    pw.print('\n');
                    break;
                case DateInteger.TYPE_MOIS:
                    pw.print("dattp=m\n");
                    pw.print("datms=");
                    pw.print(DateInteger.getMois(dateInteger));
                    pw.print('\n');
                    break;
                case DateInteger.TYPE_JOUR:
                    pw.print("dattp=j\n");
                    pw.print("datms=");
                    pw.print(DateInteger.getMois(dateInteger));
                    pw.print('\n');
                    pw.print("datjr=");
                    pw.print(DateInteger.getJour(dateInteger));
                    pw.print('\n');
                    break;
            }
        } else {
            pw.print("dattp=o\n");
        }
        int motcleCount = currentFicheResult.getMotcleCount();
        if (motcleCount > 0) {
            for (int i = 0; i < motcleCount; i++) {
                int codemotcle = currentFicheResult.getMotcleCode(i);
                pw.print("mtc");
                if (i < 10) {
                    pw.print('0');
                }
                pw.print(i);
                pw.print('=');
                pw.print(codemotcle);
                pw.print('\n');
            }
        }
    }

    private void printPosition(PrintWriter pw, int alineaInteger, SearchTokenPositionList positionList, int[] decalages) {
        if (positionList == null) {
            return;
        }
        for (int k = 0; k < positionList.size(); k++) {
            SubstringPosition substringPosition = positionList.getSearchTokenPosition(k);
            int tokenIndex = positionList.getSearchTokenIndexForPosition(k);
            int positiondebut = substringPosition.getBeginIndex();
            int decalagedebut = 0;
            if (positiondebut > 0) {
                decalagedebut = decalages[positiondebut - 1];
            }
            pw.print("loc");
            if (tokenIndex < 10) {
                pw.print('0');
            }
            pw.print(tokenIndex);
            pw.print('=');
            printAlineaInteger(pw, alineaInteger);
            pw.print("pos");
            if (tokenIndex < 10) {
                pw.print('0');
            }
            pw.print(tokenIndex);
            pw.print('=');
            pw.print(positiondebut + decalagedebut);
            pw.print('\n');
            pw.print("lng");
            if (tokenIndex < 10) {
                pw.print('0');
            }
            pw.print(tokenIndex);
            pw.print('=');
            int longueur = substringPosition.getLength();
            longueur = longueur + decalages[positiondebut - 1 + longueur] - decalagedebut;
            pw.print(longueur);
            pw.print('\n');
        }
    }

    private void printAlineaInteger(PrintWriter pw, int alineaInteger) {
        short category = AlineaInteger.getCategory(alineaInteger);
        if (category == AlineaInteger.SPECIAL_CATEGORY) {
            if (alineaInteger == AlineaInteger.TITRE) {
                pw.print("titre");
            } else if (alineaInteger == AlineaInteger.SOUSTITRE) {
                pw.print("ssttr");
            }
        } else if (category == AlineaInteger.COMPLEMENT_CATEGORY) {
            int complementNumber = AlineaInteger.toComplementNumber(alineaInteger);
            if (complementNumber < 10) {
                pw.print("cmp0");
                pw.print(complementNumber);
            } else {
                pw.print("cmp");
                pw.print(complementNumber);
            }
        }
        pw.print('\n');
    }

}
