/* UtilLib - Copyright (c) 2006-2012 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.html;

import java.io.Serializable;


/**
 *
 * @author Vincent Calame
 */
public class HTMLAttributes implements Serializable, Cloneable {

    public final static String MULTIPART_ENCTYPE = "multipart/form-data";
    public final static String CHECKBOX_TYPE = "checkbox";
    public final static String RADIO_TYPE = "radio";
    public final static String HIDDEN_TYPE = "hidden";
    public final static String TEXT_TYPE = "text";
    public final static String FILE_TYPE = "file";
    public final static String SUBMIT_TYPE = "submit";
    public final static String BUTTON_TYPE = "button";
    public final static String PASSWORD_TYPE = "password";
    public final static String GET_METHOD = "GET";
    public final static String POST_METHOD = "POST";
    public final static String TOP_TARGET = "_top";
    public final static String BLANK_TARGET = "_blank";
    private String name;
    private String classes;
    private String id;
    private String style;
    private String onChange;
    private String onClick;
    private String onBlur;
    private String onFocus;
    private String onMouseOver;
    private String onMouseDown;
    private String onMouseUp;
    private String onMouseOut;
    private String onKeyUp;
    private String onSubmit;
    private String target;
    private String href;
    private String src;
    private String alt;
    private String title;
    private String width;
    private String height;
    private String size;
    private String value;
    private String type;
    private String action;
    private String method;
    private String label;
    private String rel;
    private String content;
    private String httpEquiv;
    private String scrolling;
    private String enctype;
    private String span;
    private boolean checked;
    private boolean selected;
    private boolean disabled;
    private boolean readonly;
    private boolean defer;
    private boolean multiple;
    private boolean required;
    private int colSpan;
    private int rowSpan;
    private int cols;
    private int rows;

    public HTMLAttributes() {
    }

    public HTMLAttributes copy() {
        return (HTMLAttributes) clone();
    }

    @Override
    public Object clone() {
        try {
            return super.clone();
        } catch (CloneNotSupportedException e) {
            return null;
        }
    }

    public String getClasses() {
        return classes;
    }

    public String getId() {
        return id;
    }

    public String getName() {
        return name;
    }

    public HTMLAttributes setName(String name) {
        this.name = name;
        return this;
    }

    public HTMLAttributes setClasses(String className) {
        this.classes = className;
        return this;
    }

    public HTMLAttributes addClass(String className) {
        if ((className == null) || (className.length() == 0)) {
            return this;
        }
        if ((classes == null) || (classes.length() == 0)) {
            classes = className;
        } else {
            classes = classes + " " + className;
        }
        return this;
    }

    public HTMLAttributes setId(String id) {
        this.id = id;
        return this;
    }

    public String getStyle() {
        return style;
    }

    public HTMLAttributes setStyle(String style) {
        this.style = style;
        return this;
    }

    public String getOnChange() {
        return onChange;
    }

    public HTMLAttributes setOnChange(String onChange) {
        this.onChange = onChange;
        return this;
    }

    public String getOnClick() {
        return onClick;
    }

    public HTMLAttributes setOnClick(String onClick) {
        this.onClick = onClick;
        return this;
    }

    public String getOnBlur() {
        return onBlur;
    }

    public HTMLAttributes setOnBlur(String onBlur) {
        this.onBlur = onBlur;
        return this;
    }

    public String getOnFocus() {
        return onFocus;
    }

    public HTMLAttributes setOnFocus(String onFocus) {
        this.onFocus = onFocus;
        return this;
    }

    public String getOnMouseOver() {
        return onMouseOver;
    }

    public HTMLAttributes setOnMouseOver(String onMouseOver) {
        this.onMouseOver = onMouseOver;
        return this;
    }

    public String getOnMouseDown() {
        return onMouseDown;
    }

    public HTMLAttributes setOnMouseDown(String onMouseDown) {
        this.onMouseDown = onMouseDown;
        return this;
    }

    public String getOnMouseUp() {
        return onMouseUp;
    }

    public HTMLAttributes setOnMouseUp(String onMouseUp) {
        this.onMouseUp = onMouseUp;
        return this;
    }

    public String getOnMouseOut() {
        return onMouseOut;
    }

    public HTMLAttributes setOnMouseOut(String onMouseOut) {
        this.onMouseOut = onMouseOut;
        return this;
    }

    public String getTarget() {
        return target;
    }

    public String getHref() {
        return href;
    }

    public String getWidth() {
        return width;
    }

    public String getHeight() {
        return height;
    }

    public String getTitle() {
        return title;
    }

    public String getSrc() {
        return src;
    }

    public String getAlt() {
        return alt;
    }

    public HTMLAttributes setTitle(String title) {
        this.title = title;
        return this;
    }

    public HTMLAttributes setSrc(String src) {
        this.src = src;
        return this;
    }

    public HTMLAttributes setAlt(String alt) {
        this.alt = alt;
        return this;
    }

    public HTMLAttributes setTarget(String target) {
        this.target = target;
        return this;
    }

    public HTMLAttributes setHref(String href) {
        this.href = href;
        return this;
    }

    public HTMLAttributes setWidth(String width) {
        this.width = width;
        return this;
    }

    public HTMLAttributes setHeight(String height) {
        this.height = height;
        return this;
    }

    public String getSize() {
        return size;
    }

    public HTMLAttributes setSize(String size) {
        this.size = size;
        return this;
    }

    public HTMLAttributes setValue(String value) {
        this.value = value;
        return this;
    }

    public String getValue() {
        return value;
    }

    public HTMLAttributes setType(String type) {
        this.type = type;
        return this;
    }

    public String getType() {
        return type;
    }

    public boolean isChecked() {
        return checked;
    }

    public HTMLAttributes setChecked(boolean checked) {
        this.checked = checked;
        return this;
    }

    public boolean isDefer() {
        return defer;
    }

    public HTMLAttributes setDefer(boolean defer) {
        this.defer = defer;
        return this;
    }

    public boolean isSelected() {
        return selected;
    }

    public HTMLAttributes setSelected(boolean selected) {
        this.selected = selected;
        return this;
    }

    public boolean isDisabled() {
        return disabled;
    }

    public HTMLAttributes setDisabled(boolean disabled) {
        this.disabled = disabled;
        return this;
    }

    public boolean isMultiple() {
        return multiple;
    }

    public HTMLAttributes setMultiple(boolean multiple) {
        this.multiple = multiple;
        return this;
    }

    public String getAction() {
        return action;
    }

    public HTMLAttributes setAction(String action) {
        this.action = action;
        return this;
    }

    public String getMethod() {
        return method;
    }

    public HTMLAttributes setMethod(String method) {
        this.method = method;
        return this;
    }

    public int getColSpan() {
        return colSpan;
    }

    public HTMLAttributes setColSpan(int colSpan) {
        this.colSpan = colSpan;
        return this;
    }

    public int getRowSpan() {
        return rowSpan;
    }

    public HTMLAttributes setRowSpan(int rowSpan) {
        this.rowSpan = rowSpan;
        return this;
    }

    public String getLabel() {
        return label;
    }

    public HTMLAttributes setLabel(String label) {
        this.label = label;
        return this;
    }

    public String getOnSubmit() {
        return onSubmit;
    }

    public HTMLAttributes setOnSubmit(String onSubmit) {
        this.onSubmit = onSubmit;
        return this;
    }

    public int getCols() {
        return cols;
    }

    public HTMLAttributes setCols(int cols) {
        this.cols = cols;
        return this;
    }

    public int getRows() {
        return rows;
    }

    public HTMLAttributes setRows(int rows) {
        this.rows = rows;
        return this;
    }

    public boolean isReadonly() {
        return readonly;
    }

    public HTMLAttributes setReadonly(boolean readonly) {
        this.readonly = readonly;
        return this;
    }

    public String getRel() {
        return rel;
    }

    public HTMLAttributes setRel(String rel) {
        this.rel = rel;
        return this;
    }

    public String getContent() {
        return content;
    }

    public HTMLAttributes setContent(String content) {
        this.content = content;
        return this;
    }

    public String getHttpEquiv() {
        return httpEquiv;
    }

    public HTMLAttributes setHttpEquiv(String httpEquiv) {
        this.httpEquiv = httpEquiv;
        return this;
    }

    public String getScrolling() {
        return scrolling;
    }

    public HTMLAttributes setScrolling(String scrolling) {
        this.scrolling = scrolling;
        return this;
    }

    public String getEnctype() {
        return enctype;
    }

    public HTMLAttributes setEnctype(String enctype) {
        this.enctype = enctype;
        return this;
    }

    public String getOnKeyUp() {
        return onKeyUp;
    }

    public HTMLAttributes setOnKeyUp(String onKeyUp) {
        this.onKeyUp = onKeyUp;
        return this;
    }

    public String getSpan() {
        return span;
    }

    public HTMLAttributes setSpan(String span) {
        this.span = span;
        return this;
    }

    public boolean isRequired() {
        return required;
    }

    public HTMLAttributes setRequired(boolean required) {
        this.required = required;
        return this;
    }

    public static HTMLAttributes init() {
        HTMLAttributes htmlAttributes = new HTMLAttributes();
        return htmlAttributes;
    }

    public static HTMLAttributes initId(String Id) {
        HTMLAttributes htmlAttributes = new HTMLAttributes();
        return htmlAttributes.setId(Id);
    }

    public static HTMLAttributes initClasses(String classes) {
        HTMLAttributes htmlAttributes = new HTMLAttributes();
        return htmlAttributes.setClasses(classes);
    }

    public static HTMLAttributes initName(String name) {
        HTMLAttributes htmlAttributes = new HTMLAttributes();
        return htmlAttributes.setName(name);
    }

    public static HTMLAttributes initAction(String action) {
        HTMLAttributes htmlAttributes = new HTMLAttributes();
        return htmlAttributes.setAction(action);
    }

    public static HTMLAttributes initType(String type) {
        HTMLAttributes htmlAttributes = new HTMLAttributes();
        return htmlAttributes.setType(type);
    }

}
