/* UtilLib - Copyright (c) 2012 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.logging;

import java.io.IOException;
import java.util.List;


/**
 *
 * @author Vincent Calame
 */
public class LogUtils {

    public final static ProcessMessageList EMPTY_PROCESSMESSAGELIST = new EmptyProcessMessageList();
    public final static ProcessMessageLog EMPTY_PROCESSMESSAGELOG = new EmptyProcessMessageLog();

    private LogUtils() {
    }

    public static String[] toValue(Object s1) {
        String[] array = new String[1];
        array[0] = s1.toString();
        return array;
    }

    public static String[] toValue(Object s1, Object s2) {
        String[] array = new String[2];
        array[0] = s1.toString();
        array[1] = s2.toString();
        return array;
    }

    public static String[] toValue(Object s1, Object s2, Object s3) {
        String[] array = new String[3];
        array[0] = s1.toString();
        array[1] = s2.toString();
        array[2] = s3.toString();
        return array;
    }

    public static String[] toValue(Object s1, Object s2, Object s3, Object s4) {
        String[] array = new String[4];
        array[0] = s1.toString();
        array[1] = s2.toString();
        array[2] = s3.toString();
        array[3] = s4.toString();
        return array;
    }

    public static String[] toValue(Object s1, Object s2, Object s3, Object s4, Object s5) {
        String[] array = new String[5];
        array[0] = s1.toString();
        array[1] = s2.toString();
        array[2] = s3.toString();
        array[3] = s4.toString();
        array[4] = s5.toString();
        return array;
    }

    public static String[] toValue(Object s1, Object s2, Object s3, Object s4, Object s5, Object s6) {
        String[] array = new String[6];
        array[0] = s1.toString();
        array[1] = s2.toString();
        array[2] = s3.toString();
        array[3] = s4.toString();
        array[4] = s5.toString();
        array[5] = s6.toString();
        return array;
    }

    public static String[] toValue(Object s1, Object s2, Object s3, Object s4, Object s5, Object s6, Object s7) {
        String[] array = new String[7];
        array[0] = s1.toString();
        array[1] = s2.toString();
        array[2] = s3.toString();
        array[3] = s4.toString();
        array[4] = s5.toString();
        array[5] = s6.toString();
        array[6] = s7.toString();
        return array;
    }

    public static String[] toValue(Object s1, Object s2, Object s3, Object s4, Object s5, Object s6, Object s7, Object s8) {
        String[] array = new String[8];
        array[0] = s1.toString();
        array[1] = s2.toString();
        array[2] = s3.toString();
        array[3] = s4.toString();
        array[4] = s5.toString();
        array[5] = s6.toString();
        array[6] = s7.toString();
        array[7] = s8.toString();
        return array;
    }

    public static String[] toValue(Object s1, Object s2, Object s3, Object s4, Object s5, Object s6, Object s7, Object s8, Object s9) {
        String[] array = new String[9];
        array[0] = s1.toString();
        array[1] = s2.toString();
        array[2] = s3.toString();
        array[3] = s4.toString();
        array[4] = s5.toString();
        array[5] = s6.toString();
        array[6] = s7.toString();
        array[7] = s8.toString();
        array[8] = s9.toString();
        return array;
    }

    public static ProcessMessage newProcessMessage(String category, String key) {
        if (category == null) {
            throw new IllegalArgumentException("key is null");
        }
        if (key == null) {
            throw new IllegalArgumentException("key is null");
        }
        return new InternalProcessMessage(category, key, null);
    }

    public static ProcessMessage newProcessMessage(String category, String key, Object value) {
        if (category == null) {
            throw new IllegalArgumentException("key is null");
        }
        if (key == null) {
            throw new IllegalArgumentException("key is null");
        }
        return new InternalProcessMessage(category, key, value);
    }

    public static ProcessMessageList toProcessMessageList(List<ProcessMessage> list) {
        int size = list.size();
        if (size == 0) {
            return EMPTY_PROCESSMESSAGELIST;
        }
        return new ArrayProcessMessageList(list.toArray(new ProcessMessage[size]));
    }

    public static String toString(ProcessURI processURI, ProcessMessageList processMessageList) {
        StringBuilder buf = new StringBuilder();
        try {
            append(buf, processURI, processMessageList);
        } catch (IOException ioe) {
        }
        return buf.toString();
    }

    public static void append(Appendable buf, ProcessURI processURI, ProcessMessageList processMessageList) throws IOException {
        int count = processMessageList.getProcessMessageCount();
        if (count == 0) {
            return;
        }
        buf.append("[[");
        buf.append(processURI.getProcessURIString());
        buf.append("]]");
        for (int j = 0; j < count; j++) {
            ProcessMessage processMessage = processMessageList.getProcessMessage(j);
            buf.append("\n      ");
            buf.append("[");
            buf.append(processMessage.getCategory());
            buf.append("] ");
            buf.append(processMessage.getMessageKey());
            Object value = processMessage.getMessageValue();
            if (value != null) {
                buf.append(" = ");
                if (value instanceof Object[]) {
                    Object[] array = (Object[]) value;
                    int length = array.length;
                    for (int i = 0; i < length; i++) {
                        if (i > 0) {
                            buf.append(" | ");
                        }
                        buf.append(array[i].toString());
                    }
                } else {
                    buf.append(value.toString());
                }
            }
        }
    }


    private static class InternalProcessMessage implements ProcessMessage {

        private String category;
        private String key;
        private Object value;

        private InternalProcessMessage(String category, String key, Object value) {
            this.category = category;
            this.key = key;
            this.value = value;
        }

        public String getCategory() {
            return category;
        }

        public String getMessageKey() {
            return key;
        }

        public Object getMessageValue() {
            return value;
        }

    }


    private static class EmptyProcessMessageList implements ProcessMessageList {

        private EmptyProcessMessageList() {
        }

        public int getProcessMessageCount() {
            return 0;
        }

        public ProcessMessage getProcessMessage(int i) {
            throw new IndexOutOfBoundsException();
        }

    }


    private static class ArrayProcessMessageList implements ProcessMessageList {

        private ProcessMessage[] array;

        private ArrayProcessMessageList(ProcessMessage[] array) {
            this.array = array;
        }

        public int getProcessMessageCount() {
            return array.length;
        }

        public ProcessMessage getProcessMessage(int i) {
            return array[i];
        }

    }


    private static class EmptyProcessMessageLog implements ProcessMessageLog {

        public boolean isEmpty() {
            return true;
        }

        public ProcessMessageList getWholeList() {
            return EMPTY_PROCESSMESSAGELIST;
        }

        public ProcessMessageList filter(String category) {
            return EMPTY_PROCESSMESSAGELIST;
        }

    }

}
