/* UtilLib - Copyright (c) 2012 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.logging;

import java.util.ArrayList;
import java.util.List;


/**
 *
 * @author Vincent Calame
 */
public class ProcessLogBuilder implements ProcessMessageHandler {

    private List<ProcessMessage> list = new ArrayList<ProcessMessage>();

    public ProcessLogBuilder() {
    }

    public void addMessage(String category, String key) {
        list.add(LogUtils.newProcessMessage(category, key));
    }

    public void addMessage(String category, String key, Object value) {
        list.add(LogUtils.newProcessMessage(category, key, value));
    }

    public boolean hasMessage() {
        return (!list.isEmpty());
    }

    public ProcessMessageList toList() {
        return LogUtils.toProcessMessageList(list);
    }

    public ProcessMessageList filter(String category) {
        List<ProcessMessage> result = new ArrayList<ProcessMessage>();
        for (ProcessMessage processMessage : list) {
            if (processMessage.getCategory().equals(category)) {
                result.add(processMessage);
            }
        }
        return LogUtils.toProcessMessageList(result);
    }

    public ProcessMessageLog toProcessMessageLog() {
        if (list.isEmpty()) {
            return LogUtils.EMPTY_PROCESSMESSAGELOG;
        }
        return new InternalProcessMessageLog(LogUtils.toProcessMessageList(list));
    }

    @Override
    public String toString() {
        StringBuilder buf = new StringBuilder();
        for (ProcessMessage processMessage : list) {
            if (buf.length() > 0) {
                buf.append('\"');
            }
            buf.append(processMessage.getMessageKey());
        }
        return buf.toString();
    }


    private static class InternalProcessMessageLog implements ProcessMessageLog {

        private ProcessMessageList list;

        private InternalProcessMessageLog(ProcessMessageList list) {
            this.list = list;
        }

        public boolean isEmpty() {
            return (list.getProcessMessageCount() == 0);
        }

        public ProcessMessageList getWholeList() {
            return list;
        }

        public ProcessMessageList filter(String category) {
            List<ProcessMessage> result = new ArrayList<ProcessMessage>();
            int count = list.getProcessMessageCount();
            for (int i = 0; i < count; i++) {
                ProcessMessage processMessage = list.getProcessMessage(i);
                if (processMessage.getCategory().equals(category)) {
                    result.add(processMessage);
                }
            }
            return LogUtils.toProcessMessageList(result);
        }

    }

}
