/* UtilLib - Copyright (c) 2012 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.primitives;


/**
 *
 * @author Vincent Calame
 */
public class DateIntegerFilterFactory {

    private DateIntegerFilterFactory() {
    }

    public static RangeDateIntegerFilter newInstance(int dateInteger) {
        return new UniqueDateIntegerFilter(dateInteger);
    }

    public static RangeDateIntegerFilter newInstance(int dateInteger1, int dateInteger2) {
        if (dateInteger1 == dateInteger2) {
            return new UniqueDateIntegerFilter(dateInteger1);
        }
        if (dateInteger1 > dateInteger2) {
            return newInstance(dateInteger2, dateInteger1);
        }
        return new DoubleDateIntegerFilter(dateInteger1, dateInteger2);

    }


    private static class UniqueDateIntegerFilter implements RangeDateIntegerFilter {

        private int filterDateInteger;
        private short truncateType;

        private UniqueDateIntegerFilter(int filterDateInteger) {
            this.filterDateInteger = filterDateInteger;
            this.truncateType = DateInteger.getDateIntegerType(filterDateInteger);
        }

        public boolean containsDateInteger(int dateInteger) {
            dateInteger = DateInteger.troncateDateInteger(dateInteger, truncateType);
            return (dateInteger == filterDateInteger);
        }

        public int getFilterDateInteger() {
            return filterDateInteger;
        }

        public int getMinDateInteger() {
            return filterDateInteger;
        }

        public int getMaxDateInteger() {
            return filterDateInteger;
        }

        public int testInRange(int dateInteger) {
            dateInteger = DateInteger.troncateDateInteger(dateInteger, truncateType);
            if (dateInteger == filterDateInteger) {
                return 0;
            } else if (dateInteger > filterDateInteger) {
                return 1;
            } else {
                return -1;
            }
        }

    }


    private static class DoubleDateIntegerFilter implements RangeDateIntegerFilter {

        private int filterDateInteger1;
        private int filterDateInteger2;
        private short truncateType1;
        private short truncateType2;

        private DoubleDateIntegerFilter(int filterDateInteger1, int filterDateInteger2) {
            this.filterDateInteger1 = filterDateInteger1;
            this.filterDateInteger2 = filterDateInteger2;
            this.truncateType1 = DateInteger.getDateIntegerType(filterDateInteger1);
            this.truncateType2 = DateInteger.getDateIntegerType(filterDateInteger2);
        }

        public boolean containsDateInteger(int dateInteger) {
            int dateInteger1 = DateInteger.troncateDateInteger(dateInteger, truncateType1);
            if (dateInteger1 < filterDateInteger1) {
                return false;
            }
            int dateInteger2 = DateInteger.troncateDateInteger(dateInteger, truncateType2);
            return (dateInteger2 <= filterDateInteger2);
        }

        public int getFilterDateInteger1() {
            return filterDateInteger1;
        }

        public int getFilterDateInteger2() {
            return filterDateInteger2;
        }

        public int getMinDateInteger() {
            return filterDateInteger1;
        }

        public int getMaxDateInteger() {
            return filterDateInteger2;
        }

        public int testInRange(int dateInteger) {
            int dateInteger1 = DateInteger.troncateDateInteger(dateInteger, truncateType1);
            if (dateInteger1 < filterDateInteger1) {
                return -1;
            }
            int dateInteger2 = DateInteger.troncateDateInteger(dateInteger, truncateType2);
            if (dateInteger2 <= filterDateInteger2) {
                return 0;
            } else {
                return -1;
            }
        }

    }

}
