/* UtilLib - Copyright (c) 2012 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.primitives;

import java.text.ParseException;
import net.mapeadores.util.text.StringUtils;


/**
 *
 * @author Vincent Calame
 */
public class DateIntegerFilterUtils {

    private DateIntegerFilterUtils() {
    }

    public static RangeDateIntegerFilter parseDateIntegerFilter(String s) throws ParseException {
        String[] tokens = StringUtils.getTechnicalTokens(s, false);
        int length = tokens.length;
        if (length == 0) {
            return null;
        }
        if (length > 2) {
            throw new ParseException("Too many tokens", s.length() - 1);
        }
        if (length == 1) {
            try {
                int dateInteger = DateInteger.parse(tokens[0]);
                return DateIntegerFilterFactory.newInstance(dateInteger);
            } catch (DateIntegerException die) {
                throw new ParseException("Wrong date: " + tokens[0], 0);
            }
        } else {
            int date1;
            int date2;
            try {
                date1 = DateInteger.parse(tokens[0]);
            } catch (DateIntegerException die) {
                throw new ParseException("Wrong date: " + tokens[0], 0);
            }
            try {
                date2 = DateInteger.parse(tokens[1]);
            } catch (DateIntegerException die) {
                throw new ParseException("Wrong date: " + tokens[1], 0);
            }
            return DateIntegerFilterFactory.newInstance(date1, date2);
        }
    }

    public static String toString(DateIntegerFilter dateIntegerFilter) {
        if (dateIntegerFilter instanceof RangeDateIntegerFilter) {
            RangeDateIntegerFilter rangeDateIntegerFilter = (RangeDateIntegerFilter) dateIntegerFilter;
            int min = rangeDateIntegerFilter.getMinDateInteger();
            int max = rangeDateIntegerFilter.getMaxDateInteger();
            if (min == max) {
                return DateInteger.toString(min);
            } else {
                return DateInteger.toString(min) + " ; " + DateInteger.toString(max);
            }
        }
        return "";
    }

    public static boolean intersects(int dateInteger, DateIntegerFilter dateIntegerFilter) {
        if (dateIntegerFilter.containsDateInteger(dateInteger)) {
            return true;
        }
        if (!(dateIntegerFilter instanceof RangeDateIntegerFilter)) {
            return false;
        }
        return intersects((RangeDateIntegerFilter) dateIntegerFilter, DateIntegerFilterFactory.newInstance(dateInteger));
    }

    public static boolean intersects(int date1, int date2, DateIntegerFilter dateIntegerFilter) {
        if (dateIntegerFilter.containsDateInteger(date1)) {
            return true;
        }
        if (dateIntegerFilter.containsDateInteger(date2)) {
            return true;
        }
        if (!(dateIntegerFilter instanceof RangeDateIntegerFilter)) {
            return false;
        }
        return intersects((RangeDateIntegerFilter) dateIntegerFilter, DateIntegerFilterFactory.newInstance(date1, date2));
    }

    private static boolean intersects(RangeDateIntegerFilter rangeDateIntegerFilter, DateIntegerFilter otherFilter) {
        int min = rangeDateIntegerFilter.getMinDateInteger();
        int max = rangeDateIntegerFilter.getMaxDateInteger();
        if (min == max) {
            return otherFilter.containsDateInteger(min);
        } else {
            return (otherFilter.containsDateInteger(min) || otherFilter.containsDateInteger(max));
        }
    }

}
