/* UtilLib - Copyright (c) 2012 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.primitives;

import java.text.DecimalFormatSymbols;


/**
 *
 * @author Vincent Calame
 */
public class MoneyConversion {

    private static Decimal UN = new Decimal(1L, (byte) 0, 0);
    private int pivotCurrency;
    private int pivotMultiplicator;
    private int otherCurrency;
    private int otherMultiplicator;
    private Decimal tauxDecimal;
    private double taux;

    /**
     * 1 pivot = taux * other
     */
    public MoneyConversion(int pivotCurrency, int otherCurrency, Decimal decimal) {
        this.pivotCurrency = pivotCurrency;
        this.otherCurrency = otherCurrency;
        this.pivotMultiplicator = CurrencyInteger.getSubunitMultiplicator(pivotCurrency);
        this.otherMultiplicator = CurrencyInteger.getSubunitMultiplicator(otherCurrency);
        this.tauxDecimal = decimal;
        this.taux = decimal.toDouble();
    }

    public long convertFromPivot(long pivotMoneyLong) {
        double pivotDouble = ((double) pivotMoneyLong) / pivotMultiplicator;
        double otherDouble = pivotDouble * taux;
        otherDouble = otherDouble * otherMultiplicator;
        long lg = (long) Math.round(otherDouble);
        return lg;
    }

    public long convertToPivot(long otherMoneyLong) {
        double otherDouble = ((double) otherMoneyLong) / otherMultiplicator;
        double pivotDouble = otherDouble / taux;
        pivotDouble = pivotDouble * pivotMultiplicator;
        long lg = (long) Math.round(pivotDouble);
        return lg;
    }

    public String toString(DecimalFormatSymbols symbols) {
        StringBuilder buf = new StringBuilder();
        buf.append(CurrencyInteger.toString(UN, pivotCurrency, symbols));
        buf.append(" = ");
        buf.append(CurrencyInteger.toString(tauxDecimal, otherCurrency, symbols));
        return buf.toString();
    }

}
