/* UtilLib - Copyright (c) 2012 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.primitives;

import java.text.DecimalFormatSymbols;
import net.mapeadores.util.exceptions.ShouldNotOccurException;


/**
 *
 * @author Vincent Calame
 */
public class MoneyLong {

    private MoneyLong() {
    }

    public static long toMoneyLong(Decimal decimal, int currencyInteger) {
        int multiplicator = CurrencyInteger.getSubunitMultiplicator(currencyInteger);
        long lg = decimal.getPartieEntiere() * multiplicator;
        int partieDecimale = decimal.getPartieDecimale();
        if (partieDecimale == 0) {
            return lg;
        }
        int subunitLength = CurrencyInteger.getSubunitLength(currencyInteger);
        byte zeroLength = decimal.getZeroLength();
        if (zeroLength > subunitLength) {
            return lg;
        }
        boolean neg = false;
        if (lg < 0) {
            lg = -lg;
            neg = true;
        }
        if (partieDecimale < 0) {
            partieDecimale = -partieDecimale;
        }
        lg = lg + getDecimalPart(zeroLength, partieDecimale, subunitLength);
        if (neg) {
            lg = -lg;
        }
        return lg;
    }

    public static Decimal toDecimal(long l, int currencyInteger) {
        int multiplicator = CurrencyInteger.getSubunitMultiplicator(currencyInteger);
        int length = CurrencyInteger.getSubunitLength(currencyInteger);
        long partieEntiere = l / multiplicator;
        int partieDecimale = (int) (l % multiplicator);
        byte zeroLength = 0;
        if (partieDecimale == 0) {
            zeroLength = (byte) length;
        } else if (Math.abs(partieDecimale) < multiplicator) {
            String s = String.valueOf(Math.abs(partieDecimale));
            zeroLength = (byte) (length - s.length());
        }
        return new Decimal(partieEntiere, zeroLength, partieDecimale);
    }

    public static String toString(long moneyLong, int currencyInteger, DecimalFormatSymbols symbols, boolean withCurrencySymbol) {
        return toString(moneyLong, currencyInteger, symbols.getDecimalSeparator(), symbols.getGroupingSeparator(), withCurrencySymbol);
    }

    public static String toString(long moneyLong, int currencyInteger, DecimalFormatSymbols symbols) {
        return toString(moneyLong, currencyInteger, symbols.getDecimalSeparator(), symbols.getGroupingSeparator());
    }

    public static String toString(long moneyLong, int currencyInteger, char decimalSeparator, char groupingSeparator, boolean withCurrencySymbol) {
        String s = toString(moneyLong, currencyInteger, decimalSeparator, groupingSeparator);
        if (withCurrencySymbol) {
            return CurrencyInteger.appendCurrencySymboltoString(s, currencyInteger);
        } else {
            return s;
        }
    }

    public static String toString(long moneyLong, int currencyInteger, char decimalSeparator, char groupingSeparator) {
        int subunitLength = CurrencyInteger.getSubunitLength(currencyInteger);
        StringBuilder buf = new StringBuilder();
        if (moneyLong == 0) {
            buf.append('0');
            if (subunitLength > 0) {
                buf.append(decimalSeparator);
                for (int i = 0; i < subunitLength; i++) {
                    buf.append('0');
                }
            }
            return buf.toString();
        }
        if (moneyLong < 0) {
            moneyLong = -moneyLong;
            buf.append("- ");
        }
        String s = Long.toString(moneyLong);
        int longLength = s.length();
        if (longLength <= subunitLength) {
            buf.append('0');
        } else {
            int entierLength = longLength - subunitLength;
            if (entierLength <= 3) {
                for (int i = 0; i < entierLength; i++) {
                    buf.append(s.charAt(i));
                }
            } else {
                int reste = entierLength % 3;
                if (reste > 0) {
                    buf.append(s.charAt(0));
                    if (reste > 1) {
                        buf.append(s.charAt(1));
                    }
                    buf.append(groupingSeparator);
                }
                int p = 0;
                for (int i = reste; i < entierLength; i++) {
                    buf.append(s.charAt(i));
                    p++;
                    if (p == 3) {
                        if (i < (entierLength - 1)) {
                            buf.append(groupingSeparator);
                        }
                        p = 0;
                    }
                }
            }
        }
        if (subunitLength > 0) {
            buf.append(decimalSeparator);
            int depart = (longLength - subunitLength);
            if (depart < 0) {
                depart = -depart;
                for (int i = 0; i < depart; i++) {
                    buf.append('0');
                }
                depart = 0;
            }
            for (int i = depart; i < longLength; i++) {
                buf.append(s.charAt(i));
            }
        }
        return buf.toString();
    }

    private static int getDecimalPart(byte zeroLength, int partieDecimale, int totalLength) {
        StringBuilder buf = new StringBuilder();
        for (int i = 0; i < zeroLength; i++) {
            buf.append('0');
            totalLength = totalLength - 1;
        }
        String s = String.valueOf(partieDecimale);
        int decimalLength = s.length();
        int min = Math.min(decimalLength, totalLength);
        for (int i = 0; i < min; i++) {
            buf.append(s.charAt(i));
        }
        int chiffrearrondi = 0;
        if (min < totalLength) {
            for (int i = min; i < totalLength; i++) {
                buf.append('0');
            }
        } else if (min < decimalLength) {
            chiffrearrondi = ((int) s.charAt(min)) - 48;
        }
        try {
            int result = 0;
            if (buf.length() > 0) {
                result = Integer.parseInt(buf.toString());
            }
            if (chiffrearrondi >= 5) {
                result = result + 1;
            }
            return result;
        } catch (NumberFormatException nfe) {
            throw new ShouldNotOccurException();
        }

    }

}
