/* UtilLib - Copyright (c) 2012 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.request;

import java.io.File;
import java.io.IOException;
import net.mapeadores.util.attr.AttributeHolder;
import net.mapeadores.util.attr.AttributeUtils;
import net.mapeadores.util.request.xml.RequestConfDOMReader;
import net.mapeadores.util.xml.DOMUtils;
import org.w3c.dom.Document;


/**
 *
 * @author Vincent Calame
 */
public class RequestUtils {

    public final static SupplementaryParameterList EMPTY_SUPPLEMENTARYPARAMETERLIST = new EmptySupplementaryParameterList();
    public final static RequestConf EMPTY_REQUESTCONF = new EmptyRequestConf();

    private RequestUtils() {
    }

    public static AppendableRequestMap merge(RequestMap requestMap, SupplementaryParameterList supplementaryParameterList) {
        AppendableRequestMap appendableRequestMap = new AppendableRequestMap(requestMap);
        appendableRequestMap.addSupplementaryParameterParameterList(supplementaryParameterList);
        return appendableRequestMap;
    }

    public static RequestConf parseRequestConf(File f) throws IOException {
        Document document = DOMUtils.readDocument(f);
        return parseRequestConf(document);
    }

    public static RequestConf parseRequestConf(Document document) {
        RequestConfBuilder requestConfBuilder = new RequestConfBuilder();
        RequestConfDOMReader reader = new RequestConfDOMReader(requestConfBuilder);
        reader.readConf(document.getDocumentElement());
        return requestConfBuilder.toRequestConf();
    }


    private static class EmptyRequestConf implements RequestConf {

        public SupplementaryParameterList getSupplementaryParameterList() {
            return EMPTY_SUPPLEMENTARYPARAMETERLIST;
        }

        public AttributeHolder getAttributes() {
            return AttributeUtils.EMPTY_ATTRIBUTEHOLDER;
        }

    }


    private static class EmptySupplementaryParameterList implements SupplementaryParameterList {

        public int getSupplementaryParameterCount() {
            return 0;
        }

        public SupplementaryParameter getSupplementaryParameter(int i) {
            throw new IndexOutOfBoundsException();
        }

    }

}
