/* UtilLib - Copyright (c) 2012 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.request.xml;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import net.mapeadores.util.attr.Attribute;
import net.mapeadores.util.attr.AttributeBuilder;
import net.mapeadores.util.attr.AttributeKey;
import net.mapeadores.util.request.RequestConfBuilder;
import net.mapeadores.util.text.CleanedString;
import net.mapeadores.util.xml.DOMUtils;
import net.mapeadores.util.xml.ElementHandler;
import net.mapeadores.util.xml.XMLUtils;
import org.w3c.dom.Element;


/**
 *
 * @author Vincent Calame
 */
public class RequestConfDOMReader {

    private RequestConfBuilder requestConfBuilder;

    public RequestConfDOMReader(RequestConfBuilder requestConfBuilder) {
        this.requestConfBuilder = requestConfBuilder;
    }

    public void readConf(Element element) {
        DOMUtils.readChildren(element, new ConfHandler());
    }


    private class ConfHandler implements ElementHandler {

        public void readElement(Element element) {
            String tagName = element.getTagName();
            if (tagName.equals("param")) {
                String name = element.getAttribute("name");
                if (name.length() > 0) {
                    String value = element.getAttribute("value");
                    if (value.length() > 0) {
                        requestConfBuilder.addParameter(name, Collections.singleton(value));
                    } else {
                        ParameterValueHandler parameterValueHandler = new ParameterValueHandler();
                        DOMUtils.readChildren(element, parameterValueHandler);
                        if (parameterValueHandler.isNotEmpty()) {
                            requestConfBuilder.addParameter(name, parameterValueHandler.getList());
                        }
                    }
                }
            } else if (tagName.equals("attr")) {
                String localKey = element.getAttribute("key");
                if (localKey.length() > 0) {
                    String nameSpace = element.getAttribute("ns");
                    AttributeHandler attributeHandler = new AttributeHandler(AttributeKey.newInstance(nameSpace, localKey));
                    DOMUtils.readChildren(element, attributeHandler);
                    Attribute attribute = attributeHandler.toAttribute();
                    if (attribute != null) {
                        requestConfBuilder.putAttribute(attribute);
                    }
                }
            }
        }

    }


    private class ParameterValueHandler implements ElementHandler {

        private List<String> values = new ArrayList<String>();

        public ParameterValueHandler() {
        }

        public void readElement(Element element) {
            String tagName = element.getTagName();
            if (tagName.equals("val")) {
                String val = XMLUtils.getData(element);
                if (val.length() > 0) {
                    values.add(val);
                }
            }
        }

        private boolean isNotEmpty() {
            return (!values.isEmpty());
        }

        private List<String> getList() {
            return values;
        }

    }


    private class AttributeHandler implements ElementHandler {

        private AttributeBuilder attributeBuilder;

        public AttributeHandler(AttributeKey attributeKey) {
            this.attributeBuilder = new AttributeBuilder(attributeKey);
        }

        public Attribute toAttribute() {
            return attributeBuilder.toAttribute();
        }

        public void readElement(Element element) {
            String tagName = element.getTagName();
            if (tagName.equals("val")) {
                CleanedString cs = XMLUtils.toCleanedString(element);
                if (cs != null) {
                    attributeBuilder.addValue(cs);
                }
            }
        }

    }

}
