/* UtilLib - Copyright (c) 2007-2012 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.text.lexie.xml;

import net.mapeadores.util.text.*;
import net.mapeadores.util.text.lexie.LexieMap;
import net.mapeadores.util.text.lexie.ParsedLexieSource;
import net.mapeadores.util.text.lexie.TextLexieUnit;
import net.mapeadores.util.text.lexie.Occurence;
import net.mapeadores.util.text.lexie.LexieSource;
import net.mapeadores.util.xml.DefaultXMLWriter;
import java.util.List;
import java.util.Iterator;
import java.io.IOException;


/**
 *
 * @author Vincent Calame
 */
public class LexieMapXMLWriter extends DefaultXMLWriter {

    public LexieMapXMLWriter() {
        super();
    }

    public void appendLexieMap(LexieMap lexieMap) throws IOException {
        openTag("lexiemap");
        openTag("lexies");
        for (Iterator it = lexieMap.getCollatedKeyMap().values().iterator(); it.hasNext();) {
            TextLexieUnit lexieUnit = (TextLexieUnit) it.next();
            appendLexieUnit(lexieUnit);
        }
        closeTag("lexies");
        openTag("sources");
        for (ParsedLexieSource parsedLexieSource : lexieMap.getParsedLexieSourceMap().values()) {
            appendParsedLexieSource(parsedLexieSource);
        }
        closeTag("sources");
        closeTag("lexiemap");
    }

    public void appendLexieUnit(TextLexieUnit lexieUnit) throws IOException {
        startOpenTag("lexie");
        addAttribute("lx-id", lexieUnit.getLexieId());
        addAttribute("value", lexieUnit.getCanonicalLexie());
        endOpenTag();
        List<Occurence> occurenceList = lexieUnit.getOccurenceList();
        for (Occurence occurence : occurenceList) {
            ParsedLexieSource parsedLexieSource = occurence.getParsedLexieSource();
            startOpenTag("occurence");
            addAttribute("src-id", parsedLexieSource.getLexieSource().getSourceId());
            addAttribute("pos", String.valueOf(occurence.getOccurencePosition()));
            closeEmptyTag();
        }
        closeTag("lexie");
    }

    public void appendParsedLexieSource(ParsedLexieSource parsedLexieSource) throws IOException {
        LexieSource lexieSource = parsedLexieSource.getLexieSource();
        startOpenTag("source");
        addAttribute("src-id", lexieSource.getSourceId());
        endOpenTag();
        openTag("text");
        appendText(lexieSource.getSourceText(), parsedLexieSource.getOccurenceList());
        closeTag("text", false, true);
        closeTag("source");
    }

    private void appendText(String text, List<Occurence> occurenceList) throws IOException {
        int size = occurenceList.size();
        if (size == 0) {
            addText(text);
            return;
        }
        Occurence currentOccurence = occurenceList.get(0);
        int currentOccurenceIndex = 0;
        SubstringPosition substringPosition = currentOccurence.getSubstringPosition();
        int startLexieIndex = substringPosition.getBeginIndex();
        int endLexieIndex = substringPosition.getEndIndex();
        if (startLexieIndex > 0) {
            addText(text.substring(0, startLexieIndex));
        }
        while (true) {
            startOpenTag("lx", false);
            addAttribute("lx-id", currentOccurence.getLexieUnit().getLexieId());
            addAttribute("pos", String.valueOf(currentOccurence.getOccurencePosition()));
            endOpenTag(false);
            addText(text.substring(startLexieIndex, endLexieIndex + 1));
            closeTag("lx", false, false);
            currentOccurenceIndex++;
            if (currentOccurenceIndex < size) {
                currentOccurence = occurenceList.get(currentOccurenceIndex);
                substringPosition = currentOccurence.getSubstringPosition();
                startLexieIndex = substringPosition.getBeginIndex();
                if (startLexieIndex > (endLexieIndex + 1)) {
                    addText(text.substring(endLexieIndex + 1, startLexieIndex));
                }
                endLexieIndex = substringPosition.getEndIndex();
            } else {
                break;
            }
        }
        if (endLexieIndex < (text.length() - 1)) {
            addText(text.substring(endLexieIndex + 1));
        }
    }

    public static String getLexieMapXML(LexieMap lexieMap, boolean withTabs) {
        StringBuilder buf = new StringBuilder(512);
        try {
            LexieMapXMLWriter lexieMapXMLWriter = new LexieMapXMLWriter();
            lexieMapXMLWriter.setAppendable(buf);
            lexieMapXMLWriter.setIndentLength((withTabs) ? 0 : -999);
            lexieMapXMLWriter.appendXMLDeclaration();
            lexieMapXMLWriter.appendLexieMap(lexieMap);
        } catch (IOException ioe) {
        }
        return buf.toString();
    }

}
